/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { FtrProviderContext } from '../../../ftr_provider_context';

export default function ({ getService }: FtrProviderContext) {
  const svlCommonApi = getService('svlCommonApi');

  describe('security/authorization', function () {
    describe('available features', () => {
      const svlUserManager = getService('svlUserManager');
      const supertestWithoutAuth = getService('supertestWithoutAuth');
      let adminCredentials: { Cookie: string };

      before(async () => {
        // get auth header for Viewer role
        adminCredentials = await svlUserManager.getM2MApiCredentialsWithRoleScope('admin');
      });

      it('composite features', async () => {
        const { body } = await supertestWithoutAuth
          .get('/api/security/privileges?includeActions=true')
          .set(svlCommonApi.getInternalRequestHeader())
          .set(adminCredentials)
          .expect(200);

        // The following features are composed of other features in a way that is
        // specific to the search solution.
        const compositeFeatureIds = ['dashboard', 'discover', 'reporting'];

        const features = Object.fromEntries(
          Object.entries(body.features).filter(([key]) => compositeFeatureIds.includes(key))
        );
        expectSnapshot(features).toMatchInline(`
          Object {
            "dashboard": Object {
              "all": Array [
                "login:",
                "api:bulkGetUserProfiles",
                "api:store_search_session",
                "api:generateReport",
                "api:downloadCsv",
                "app:dashboards",
                "app:kibana",
                "ui:catalogue/dashboard",
                "ui:management/kibana/search_sessions",
                "ui:management/insightsAndAlerting/reporting",
                "ui:navLinks/dashboards",
                "ui:navLinks/kibana",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:dashboard/create",
                "saved_object:dashboard/bulk_create",
                "saved_object:dashboard/update",
                "saved_object:dashboard/bulk_update",
                "saved_object:dashboard/delete",
                "saved_object:dashboard/bulk_delete",
                "saved_object:dashboard/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "ui:dashboard/createNew",
                "ui:dashboard/show",
                "ui:dashboard/showWriteControls",
                "ui:dashboard/saveQuery",
                "ui:dashboard/createShortUrl",
                "ui:dashboard/storeSearchSession",
                "ui:dashboard/generateScreenshot",
                "ui:dashboard/downloadCsv",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "saved_object:map/create",
                "saved_object:map/bulk_create",
                "saved_object:map/update",
                "saved_object:map/bulk_update",
                "saved_object:map/delete",
                "saved_object:map/bulk_delete",
                "saved_object:map/share_to_space",
                "ui:maps/save",
                "ui:maps/show",
                "ui:maps/saveQuery",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "saved_object:visualization/create",
                "saved_object:visualization/bulk_create",
                "saved_object:visualization/update",
                "saved_object:visualization/bulk_update",
                "saved_object:visualization/delete",
                "saved_object:visualization/bulk_delete",
                "saved_object:visualization/share_to_space",
                "saved_object:lens/create",
                "saved_object:lens/bulk_create",
                "saved_object:lens/update",
                "saved_object:lens/bulk_update",
                "saved_object:lens/delete",
                "saved_object:lens/bulk_delete",
                "saved_object:lens/share_to_space",
                "ui:visualize/show",
                "ui:visualize/delete",
                "ui:visualize/save",
                "ui:visualize/saveQuery",
                "ui:visualize/createShortUrl",
                "ui:visualize/generateScreenshot",
              ],
              "download_csv_report": Array [
                "login:",
                "api:downloadCsv",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/downloadCsv",
              ],
              "generate_report": Array [
                "login:",
                "api:generateReport",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/generateScreenshot",
              ],
              "minimal_all": Array [
                "login:",
                "api:bulkGetUserProfiles",
                "app:dashboards",
                "app:kibana",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "ui:navLinks/kibana",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:dashboard/create",
                "saved_object:dashboard/bulk_create",
                "saved_object:dashboard/update",
                "saved_object:dashboard/bulk_update",
                "saved_object:dashboard/delete",
                "saved_object:dashboard/bulk_delete",
                "saved_object:dashboard/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:dashboard/createNew",
                "ui:dashboard/show",
                "ui:dashboard/showWriteControls",
                "ui:dashboard/saveQuery",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "saved_object:map/create",
                "saved_object:map/bulk_create",
                "saved_object:map/update",
                "saved_object:map/bulk_update",
                "saved_object:map/delete",
                "saved_object:map/bulk_delete",
                "saved_object:map/share_to_space",
                "ui:maps/save",
                "ui:maps/show",
                "ui:maps/saveQuery",
                "api:generateReport",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:management/insightsAndAlerting/reporting",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "saved_object:visualization/create",
                "saved_object:visualization/bulk_create",
                "saved_object:visualization/update",
                "saved_object:visualization/bulk_update",
                "saved_object:visualization/delete",
                "saved_object:visualization/bulk_delete",
                "saved_object:visualization/share_to_space",
                "saved_object:lens/create",
                "saved_object:lens/bulk_create",
                "saved_object:lens/update",
                "saved_object:lens/bulk_update",
                "saved_object:lens/delete",
                "saved_object:lens/bulk_delete",
                "saved_object:lens/share_to_space",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "ui:visualize/show",
                "ui:visualize/delete",
                "ui:visualize/save",
                "ui:visualize/saveQuery",
                "ui:visualize/createShortUrl",
                "ui:visualize/generateScreenshot",
              ],
              "minimal_read": Array [
                "login:",
                "api:bulkGetUserProfiles",
                "app:dashboards",
                "app:kibana",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "ui:navLinks/kibana",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:dashboard/show",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "ui:maps/show",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "ui:visualize/show",
                "ui:visualize/createShortUrl",
              ],
              "read": Array [
                "login:",
                "api:bulkGetUserProfiles",
                "app:dashboards",
                "app:kibana",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "ui:navLinks/kibana",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "ui:dashboard/show",
                "ui:dashboard/createShortUrl",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "ui:maps/show",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "ui:visualize/show",
                "ui:visualize/createShortUrl",
              ],
              "store_search_session": Array [
                "login:",
                "api:store_search_session",
                "ui:management/kibana/search_sessions",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "ui:dashboard/storeSearchSession",
              ],
              "url_create": Array [
                "login:",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "ui:dashboard/createShortUrl",
              ],
            },
            "discover": Object {
              "all": Array [
                "login:",
                "api:fileUpload:analyzeFile",
                "api:store_search_session",
                "api:generateReport",
                "app:discover",
                "app:kibana",
                "ui:catalogue/discover",
                "ui:management/kibana/search_sessions",
                "ui:management/insightsAndAlerting/reporting",
                "ui:navLinks/discover",
                "ui:navLinks/kibana",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:search/create",
                "saved_object:search/bulk_create",
                "saved_object:search/update",
                "saved_object:search/bulk_update",
                "saved_object:search/delete",
                "saved_object:search/bulk_delete",
                "saved_object:search/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "ui:discover/show",
                "ui:discover/save",
                "ui:discover/saveQuery",
                "ui:discover/createShortUrl",
                "ui:discover/storeSearchSession",
                "ui:discover/generateCsv",
              ],
              "generate_report": Array [
                "login:",
                "api:generateReport",
                "ui:management/insightsAndAlerting/reporting",
                "ui:discover/generateCsv",
              ],
              "minimal_all": Array [
                "login:",
                "api:fileUpload:analyzeFile",
                "app:discover",
                "app:kibana",
                "ui:catalogue/discover",
                "ui:navLinks/discover",
                "ui:navLinks/kibana",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:search/create",
                "saved_object:search/bulk_create",
                "saved_object:search/update",
                "saved_object:search/bulk_update",
                "saved_object:search/delete",
                "saved_object:search/bulk_delete",
                "saved_object:search/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:discover/show",
                "ui:discover/save",
                "ui:discover/saveQuery",
              ],
              "minimal_read": Array [
                "login:",
                "app:discover",
                "app:kibana",
                "ui:catalogue/discover",
                "ui:navLinks/discover",
                "ui:navLinks/kibana",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:discover/show",
              ],
              "read": Array [
                "login:",
                "app:discover",
                "app:kibana",
                "ui:catalogue/discover",
                "ui:navLinks/discover",
                "ui:navLinks/kibana",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "ui:discover/show",
                "ui:discover/createShortUrl",
              ],
              "store_search_session": Array [
                "login:",
                "api:store_search_session",
                "ui:management/kibana/search_sessions",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "ui:discover/storeSearchSession",
              ],
              "url_create": Array [
                "login:",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "ui:discover/createShortUrl",
              ],
            },
            "reporting": Object {
              "all": Array [
                "login:",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "api:downloadCsv",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/downloadCsv",
                "api:generateReport",
                "ui:discover/generateCsv",
              ],
              "minimal_all": Array [
                "login:",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "api:downloadCsv",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/downloadCsv",
                "api:generateReport",
                "ui:discover/generateCsv",
              ],
              "minimal_read": Array [
                "login:",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
              ],
              "read": Array [
                "login:",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
              ],
            },
          }
        `);
      });
    });
  });
}
