/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { FtrProviderContext } from '../../../ftr_provider_context';

export default function ({ getService }: FtrProviderContext) {
  const svlCommonApi = getService('svlCommonApi');

  describe('security/authorization', function () {
    describe('available features', () => {
      const svlUserManager = getService('svlUserManager');
      const supertestWithoutAuth = getService('supertestWithoutAuth');
      let adminCredentials: { Cookie: string };

      before(async () => {
        // get auth header for Viewer role
        adminCredentials = await svlUserManager.getM2MApiCredentialsWithRoleScope('admin');
      });

      it('composite features', async () => {
        const { body } = await supertestWithoutAuth
          .get('/api/security/privileges?includeActions=true')
          .set(svlCommonApi.getInternalRequestHeader())
          .set(adminCredentials)
          .expect(200);

        // The following features are composed of other features in a way that is
        // specific to the security solution.
        const compositeFeatureIds = ['dashboard', 'discover', 'reporting', 'siem'];

        const features = Object.fromEntries(
          Object.entries(body.features).filter(([key]) => compositeFeatureIds.includes(key))
        );
        expectSnapshot(features).toMatchInline(`
          Object {
            "reporting": Object {
              "all": Array [
                "login:",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "api:downloadCsv",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/downloadCsv",
                "api:generateReport",
                "ui:discover/generateCsv",
              ],
              "minimal_all": Array [
                "login:",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "api:downloadCsv",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/downloadCsv",
                "api:generateReport",
                "ui:discover/generateCsv",
              ],
              "minimal_read": Array [
                "login:",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
              ],
              "read": Array [
                "login:",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
              ],
            },
            "siem": Object {
              "actions_log_management_all": Array [
                "login:",
                "api:securitySolution-writeActionsLogManagement",
                "api:securitySolution-readActionsLogManagement",
                "ui:siem/writeActionsLogManagement",
                "ui:siem/readActionsLogManagement",
              ],
              "actions_log_management_read": Array [
                "login:",
                "api:securitySolution-readActionsLogManagement",
                "ui:siem/readActionsLogManagement",
              ],
              "all": Array [
                "login:",
                "api:securitySolution",
                "api:lists-all",
                "api:lists-read",
                "api:lists-summary",
                "api:rac",
                "api:cloud-security-posture-all",
                "api:cloud-security-posture-read",
                "api:cloud-defend-all",
                "api:cloud-defend-read",
                "api:securitySolution-entity-analytics",
                "api:securitySolution-threat-intelligence",
                "api:securitySolution-showEndpointExceptions",
                "api:securitySolution-crudEndpointExceptions",
                "app:securitySolution",
                "app:csp",
                "app:cloudDefend",
                "app:kibana",
                "ui:catalogue/securitySolution",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/securitySolution",
                "ui:navLinks/csp",
                "ui:navLinks/cloudDefend",
                "ui:navLinks/kibana",
                "saved_object:alert/bulk_get",
                "saved_object:alert/get",
                "saved_object:alert/find",
                "saved_object:alert/open_point_in_time",
                "saved_object:alert/close_point_in_time",
                "saved_object:alert/create",
                "saved_object:alert/bulk_create",
                "saved_object:alert/update",
                "saved_object:alert/bulk_update",
                "saved_object:alert/delete",
                "saved_object:alert/bulk_delete",
                "saved_object:alert/share_to_space",
                "saved_object:exception-list/bulk_get",
                "saved_object:exception-list/get",
                "saved_object:exception-list/find",
                "saved_object:exception-list/open_point_in_time",
                "saved_object:exception-list/close_point_in_time",
                "saved_object:exception-list/create",
                "saved_object:exception-list/bulk_create",
                "saved_object:exception-list/update",
                "saved_object:exception-list/bulk_update",
                "saved_object:exception-list/delete",
                "saved_object:exception-list/bulk_delete",
                "saved_object:exception-list/share_to_space",
                "saved_object:exception-list-agnostic/bulk_get",
                "saved_object:exception-list-agnostic/get",
                "saved_object:exception-list-agnostic/find",
                "saved_object:exception-list-agnostic/open_point_in_time",
                "saved_object:exception-list-agnostic/close_point_in_time",
                "saved_object:exception-list-agnostic/create",
                "saved_object:exception-list-agnostic/bulk_create",
                "saved_object:exception-list-agnostic/update",
                "saved_object:exception-list-agnostic/bulk_update",
                "saved_object:exception-list-agnostic/delete",
                "saved_object:exception-list-agnostic/bulk_delete",
                "saved_object:exception-list-agnostic/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:index-pattern/create",
                "saved_object:index-pattern/bulk_create",
                "saved_object:index-pattern/update",
                "saved_object:index-pattern/bulk_update",
                "saved_object:index-pattern/delete",
                "saved_object:index-pattern/bulk_delete",
                "saved_object:index-pattern/share_to_space",
                "saved_object:siem-ui-timeline-note/bulk_get",
                "saved_object:siem-ui-timeline-note/get",
                "saved_object:siem-ui-timeline-note/find",
                "saved_object:siem-ui-timeline-note/open_point_in_time",
                "saved_object:siem-ui-timeline-note/close_point_in_time",
                "saved_object:siem-ui-timeline-note/create",
                "saved_object:siem-ui-timeline-note/bulk_create",
                "saved_object:siem-ui-timeline-note/update",
                "saved_object:siem-ui-timeline-note/bulk_update",
                "saved_object:siem-ui-timeline-note/delete",
                "saved_object:siem-ui-timeline-note/bulk_delete",
                "saved_object:siem-ui-timeline-note/share_to_space",
                "saved_object:siem-ui-timeline-pinned-event/bulk_get",
                "saved_object:siem-ui-timeline-pinned-event/get",
                "saved_object:siem-ui-timeline-pinned-event/find",
                "saved_object:siem-ui-timeline-pinned-event/open_point_in_time",
                "saved_object:siem-ui-timeline-pinned-event/close_point_in_time",
                "saved_object:siem-ui-timeline-pinned-event/create",
                "saved_object:siem-ui-timeline-pinned-event/bulk_create",
                "saved_object:siem-ui-timeline-pinned-event/update",
                "saved_object:siem-ui-timeline-pinned-event/bulk_update",
                "saved_object:siem-ui-timeline-pinned-event/delete",
                "saved_object:siem-ui-timeline-pinned-event/bulk_delete",
                "saved_object:siem-ui-timeline-pinned-event/share_to_space",
                "saved_object:siem-detection-engine-rule-actions/bulk_get",
                "saved_object:siem-detection-engine-rule-actions/get",
                "saved_object:siem-detection-engine-rule-actions/find",
                "saved_object:siem-detection-engine-rule-actions/open_point_in_time",
                "saved_object:siem-detection-engine-rule-actions/close_point_in_time",
                "saved_object:siem-detection-engine-rule-actions/create",
                "saved_object:siem-detection-engine-rule-actions/bulk_create",
                "saved_object:siem-detection-engine-rule-actions/update",
                "saved_object:siem-detection-engine-rule-actions/bulk_update",
                "saved_object:siem-detection-engine-rule-actions/delete",
                "saved_object:siem-detection-engine-rule-actions/bulk_delete",
                "saved_object:siem-detection-engine-rule-actions/share_to_space",
                "saved_object:security-rule/bulk_get",
                "saved_object:security-rule/get",
                "saved_object:security-rule/find",
                "saved_object:security-rule/open_point_in_time",
                "saved_object:security-rule/close_point_in_time",
                "saved_object:security-rule/create",
                "saved_object:security-rule/bulk_create",
                "saved_object:security-rule/update",
                "saved_object:security-rule/bulk_update",
                "saved_object:security-rule/delete",
                "saved_object:security-rule/bulk_delete",
                "saved_object:security-rule/share_to_space",
                "saved_object:siem-ui-timeline/bulk_get",
                "saved_object:siem-ui-timeline/get",
                "saved_object:siem-ui-timeline/find",
                "saved_object:siem-ui-timeline/open_point_in_time",
                "saved_object:siem-ui-timeline/close_point_in_time",
                "saved_object:siem-ui-timeline/create",
                "saved_object:siem-ui-timeline/bulk_create",
                "saved_object:siem-ui-timeline/update",
                "saved_object:siem-ui-timeline/bulk_update",
                "saved_object:siem-ui-timeline/delete",
                "saved_object:siem-ui-timeline/bulk_delete",
                "saved_object:siem-ui-timeline/share_to_space",
                "saved_object:endpoint:user-artifact-manifest/bulk_get",
                "saved_object:endpoint:user-artifact-manifest/get",
                "saved_object:endpoint:user-artifact-manifest/find",
                "saved_object:endpoint:user-artifact-manifest/open_point_in_time",
                "saved_object:endpoint:user-artifact-manifest/close_point_in_time",
                "saved_object:endpoint:user-artifact-manifest/create",
                "saved_object:endpoint:user-artifact-manifest/bulk_create",
                "saved_object:endpoint:user-artifact-manifest/update",
                "saved_object:endpoint:user-artifact-manifest/bulk_update",
                "saved_object:endpoint:user-artifact-manifest/delete",
                "saved_object:endpoint:user-artifact-manifest/bulk_delete",
                "saved_object:endpoint:user-artifact-manifest/share_to_space",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_get",
                "saved_object:endpoint:unified-user-artifact-manifest/get",
                "saved_object:endpoint:unified-user-artifact-manifest/find",
                "saved_object:endpoint:unified-user-artifact-manifest/open_point_in_time",
                "saved_object:endpoint:unified-user-artifact-manifest/close_point_in_time",
                "saved_object:endpoint:unified-user-artifact-manifest/create",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_create",
                "saved_object:endpoint:unified-user-artifact-manifest/update",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_update",
                "saved_object:endpoint:unified-user-artifact-manifest/delete",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_delete",
                "saved_object:endpoint:unified-user-artifact-manifest/share_to_space",
                "saved_object:security-solution-signals-migration/bulk_get",
                "saved_object:security-solution-signals-migration/get",
                "saved_object:security-solution-signals-migration/find",
                "saved_object:security-solution-signals-migration/open_point_in_time",
                "saved_object:security-solution-signals-migration/close_point_in_time",
                "saved_object:security-solution-signals-migration/create",
                "saved_object:security-solution-signals-migration/bulk_create",
                "saved_object:security-solution-signals-migration/update",
                "saved_object:security-solution-signals-migration/bulk_update",
                "saved_object:security-solution-signals-migration/delete",
                "saved_object:security-solution-signals-migration/bulk_delete",
                "saved_object:security-solution-signals-migration/share_to_space",
                "saved_object:risk-engine-configuration/bulk_get",
                "saved_object:risk-engine-configuration/get",
                "saved_object:risk-engine-configuration/find",
                "saved_object:risk-engine-configuration/open_point_in_time",
                "saved_object:risk-engine-configuration/close_point_in_time",
                "saved_object:risk-engine-configuration/create",
                "saved_object:risk-engine-configuration/bulk_create",
                "saved_object:risk-engine-configuration/update",
                "saved_object:risk-engine-configuration/bulk_update",
                "saved_object:risk-engine-configuration/delete",
                "saved_object:risk-engine-configuration/bulk_delete",
                "saved_object:risk-engine-configuration/share_to_space",
                "saved_object:policy-settings-protection-updates-note/bulk_get",
                "saved_object:policy-settings-protection-updates-note/get",
                "saved_object:policy-settings-protection-updates-note/find",
                "saved_object:policy-settings-protection-updates-note/open_point_in_time",
                "saved_object:policy-settings-protection-updates-note/close_point_in_time",
                "saved_object:policy-settings-protection-updates-note/create",
                "saved_object:policy-settings-protection-updates-note/bulk_create",
                "saved_object:policy-settings-protection-updates-note/update",
                "saved_object:policy-settings-protection-updates-note/bulk_update",
                "saved_object:policy-settings-protection-updates-note/delete",
                "saved_object:policy-settings-protection-updates-note/bulk_delete",
                "saved_object:policy-settings-protection-updates-note/share_to_space",
                "saved_object:csp_rule/bulk_get",
                "saved_object:csp_rule/get",
                "saved_object:csp_rule/find",
                "saved_object:csp_rule/open_point_in_time",
                "saved_object:csp_rule/close_point_in_time",
                "saved_object:csp_rule/create",
                "saved_object:csp_rule/bulk_create",
                "saved_object:csp_rule/update",
                "saved_object:csp_rule/bulk_update",
                "saved_object:csp_rule/delete",
                "saved_object:csp_rule/bulk_delete",
                "saved_object:csp_rule/share_to_space",
                "saved_object:cloud-security-posture-settings/bulk_get",
                "saved_object:cloud-security-posture-settings/get",
                "saved_object:cloud-security-posture-settings/find",
                "saved_object:cloud-security-posture-settings/open_point_in_time",
                "saved_object:cloud-security-posture-settings/close_point_in_time",
                "saved_object:cloud-security-posture-settings/create",
                "saved_object:cloud-security-posture-settings/bulk_create",
                "saved_object:cloud-security-posture-settings/update",
                "saved_object:cloud-security-posture-settings/bulk_update",
                "saved_object:cloud-security-posture-settings/delete",
                "saved_object:cloud-security-posture-settings/bulk_delete",
                "saved_object:cloud-security-posture-settings/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:siem/show",
                "ui:siem/crud",
                "ui:siem/entity-analytics",
                "ui:siem/investigation-guide",
                "ui:siem/investigation-guide-interactions",
                "ui:siem/threat-intelligence",
                "ui:siem/showEndpointExceptions",
                "ui:siem/crudEndpointExceptions",
                "alerting:siem.notifications/siem/rule/get",
                "alerting:siem.notifications/siem/rule/getRuleState",
                "alerting:siem.notifications/siem/rule/getAlertSummary",
                "alerting:siem.notifications/siem/rule/getExecutionLog",
                "alerting:siem.notifications/siem/rule/getActionErrorLog",
                "alerting:siem.notifications/siem/rule/find",
                "alerting:siem.notifications/siem/rule/getRuleExecutionKPI",
                "alerting:siem.notifications/siem/rule/getBackfill",
                "alerting:siem.notifications/siem/rule/findBackfill",
                "alerting:siem.notifications/siem/rule/create",
                "alerting:siem.notifications/siem/rule/delete",
                "alerting:siem.notifications/siem/rule/update",
                "alerting:siem.notifications/siem/rule/updateApiKey",
                "alerting:siem.notifications/siem/rule/enable",
                "alerting:siem.notifications/siem/rule/disable",
                "alerting:siem.notifications/siem/rule/muteAll",
                "alerting:siem.notifications/siem/rule/unmuteAll",
                "alerting:siem.notifications/siem/rule/muteAlert",
                "alerting:siem.notifications/siem/rule/unmuteAlert",
                "alerting:siem.notifications/siem/rule/snooze",
                "alerting:siem.notifications/siem/rule/bulkEdit",
                "alerting:siem.notifications/siem/rule/bulkDelete",
                "alerting:siem.notifications/siem/rule/bulkEnable",
                "alerting:siem.notifications/siem/rule/bulkDisable",
                "alerting:siem.notifications/siem/rule/unsnooze",
                "alerting:siem.notifications/siem/rule/runSoon",
                "alerting:siem.notifications/siem/rule/scheduleBackfill",
                "alerting:siem.notifications/siem/rule/deleteBackfill",
                "alerting:siem.esqlRule/siem/rule/get",
                "alerting:siem.esqlRule/siem/rule/getRuleState",
                "alerting:siem.esqlRule/siem/rule/getAlertSummary",
                "alerting:siem.esqlRule/siem/rule/getExecutionLog",
                "alerting:siem.esqlRule/siem/rule/getActionErrorLog",
                "alerting:siem.esqlRule/siem/rule/find",
                "alerting:siem.esqlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.esqlRule/siem/rule/getBackfill",
                "alerting:siem.esqlRule/siem/rule/findBackfill",
                "alerting:siem.esqlRule/siem/rule/create",
                "alerting:siem.esqlRule/siem/rule/delete",
                "alerting:siem.esqlRule/siem/rule/update",
                "alerting:siem.esqlRule/siem/rule/updateApiKey",
                "alerting:siem.esqlRule/siem/rule/enable",
                "alerting:siem.esqlRule/siem/rule/disable",
                "alerting:siem.esqlRule/siem/rule/muteAll",
                "alerting:siem.esqlRule/siem/rule/unmuteAll",
                "alerting:siem.esqlRule/siem/rule/muteAlert",
                "alerting:siem.esqlRule/siem/rule/unmuteAlert",
                "alerting:siem.esqlRule/siem/rule/snooze",
                "alerting:siem.esqlRule/siem/rule/bulkEdit",
                "alerting:siem.esqlRule/siem/rule/bulkDelete",
                "alerting:siem.esqlRule/siem/rule/bulkEnable",
                "alerting:siem.esqlRule/siem/rule/bulkDisable",
                "alerting:siem.esqlRule/siem/rule/unsnooze",
                "alerting:siem.esqlRule/siem/rule/runSoon",
                "alerting:siem.esqlRule/siem/rule/scheduleBackfill",
                "alerting:siem.esqlRule/siem/rule/deleteBackfill",
                "alerting:siem.eqlRule/siem/rule/get",
                "alerting:siem.eqlRule/siem/rule/getRuleState",
                "alerting:siem.eqlRule/siem/rule/getAlertSummary",
                "alerting:siem.eqlRule/siem/rule/getExecutionLog",
                "alerting:siem.eqlRule/siem/rule/getActionErrorLog",
                "alerting:siem.eqlRule/siem/rule/find",
                "alerting:siem.eqlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.eqlRule/siem/rule/getBackfill",
                "alerting:siem.eqlRule/siem/rule/findBackfill",
                "alerting:siem.eqlRule/siem/rule/create",
                "alerting:siem.eqlRule/siem/rule/delete",
                "alerting:siem.eqlRule/siem/rule/update",
                "alerting:siem.eqlRule/siem/rule/updateApiKey",
                "alerting:siem.eqlRule/siem/rule/enable",
                "alerting:siem.eqlRule/siem/rule/disable",
                "alerting:siem.eqlRule/siem/rule/muteAll",
                "alerting:siem.eqlRule/siem/rule/unmuteAll",
                "alerting:siem.eqlRule/siem/rule/muteAlert",
                "alerting:siem.eqlRule/siem/rule/unmuteAlert",
                "alerting:siem.eqlRule/siem/rule/snooze",
                "alerting:siem.eqlRule/siem/rule/bulkEdit",
                "alerting:siem.eqlRule/siem/rule/bulkDelete",
                "alerting:siem.eqlRule/siem/rule/bulkEnable",
                "alerting:siem.eqlRule/siem/rule/bulkDisable",
                "alerting:siem.eqlRule/siem/rule/unsnooze",
                "alerting:siem.eqlRule/siem/rule/runSoon",
                "alerting:siem.eqlRule/siem/rule/scheduleBackfill",
                "alerting:siem.eqlRule/siem/rule/deleteBackfill",
                "alerting:siem.indicatorRule/siem/rule/get",
                "alerting:siem.indicatorRule/siem/rule/getRuleState",
                "alerting:siem.indicatorRule/siem/rule/getAlertSummary",
                "alerting:siem.indicatorRule/siem/rule/getExecutionLog",
                "alerting:siem.indicatorRule/siem/rule/getActionErrorLog",
                "alerting:siem.indicatorRule/siem/rule/find",
                "alerting:siem.indicatorRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.indicatorRule/siem/rule/getBackfill",
                "alerting:siem.indicatorRule/siem/rule/findBackfill",
                "alerting:siem.indicatorRule/siem/rule/create",
                "alerting:siem.indicatorRule/siem/rule/delete",
                "alerting:siem.indicatorRule/siem/rule/update",
                "alerting:siem.indicatorRule/siem/rule/updateApiKey",
                "alerting:siem.indicatorRule/siem/rule/enable",
                "alerting:siem.indicatorRule/siem/rule/disable",
                "alerting:siem.indicatorRule/siem/rule/muteAll",
                "alerting:siem.indicatorRule/siem/rule/unmuteAll",
                "alerting:siem.indicatorRule/siem/rule/muteAlert",
                "alerting:siem.indicatorRule/siem/rule/unmuteAlert",
                "alerting:siem.indicatorRule/siem/rule/snooze",
                "alerting:siem.indicatorRule/siem/rule/bulkEdit",
                "alerting:siem.indicatorRule/siem/rule/bulkDelete",
                "alerting:siem.indicatorRule/siem/rule/bulkEnable",
                "alerting:siem.indicatorRule/siem/rule/bulkDisable",
                "alerting:siem.indicatorRule/siem/rule/unsnooze",
                "alerting:siem.indicatorRule/siem/rule/runSoon",
                "alerting:siem.indicatorRule/siem/rule/scheduleBackfill",
                "alerting:siem.indicatorRule/siem/rule/deleteBackfill",
                "alerting:siem.mlRule/siem/rule/get",
                "alerting:siem.mlRule/siem/rule/getRuleState",
                "alerting:siem.mlRule/siem/rule/getAlertSummary",
                "alerting:siem.mlRule/siem/rule/getExecutionLog",
                "alerting:siem.mlRule/siem/rule/getActionErrorLog",
                "alerting:siem.mlRule/siem/rule/find",
                "alerting:siem.mlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.mlRule/siem/rule/getBackfill",
                "alerting:siem.mlRule/siem/rule/findBackfill",
                "alerting:siem.mlRule/siem/rule/create",
                "alerting:siem.mlRule/siem/rule/delete",
                "alerting:siem.mlRule/siem/rule/update",
                "alerting:siem.mlRule/siem/rule/updateApiKey",
                "alerting:siem.mlRule/siem/rule/enable",
                "alerting:siem.mlRule/siem/rule/disable",
                "alerting:siem.mlRule/siem/rule/muteAll",
                "alerting:siem.mlRule/siem/rule/unmuteAll",
                "alerting:siem.mlRule/siem/rule/muteAlert",
                "alerting:siem.mlRule/siem/rule/unmuteAlert",
                "alerting:siem.mlRule/siem/rule/snooze",
                "alerting:siem.mlRule/siem/rule/bulkEdit",
                "alerting:siem.mlRule/siem/rule/bulkDelete",
                "alerting:siem.mlRule/siem/rule/bulkEnable",
                "alerting:siem.mlRule/siem/rule/bulkDisable",
                "alerting:siem.mlRule/siem/rule/unsnooze",
                "alerting:siem.mlRule/siem/rule/runSoon",
                "alerting:siem.mlRule/siem/rule/scheduleBackfill",
                "alerting:siem.mlRule/siem/rule/deleteBackfill",
                "alerting:siem.queryRule/siem/rule/get",
                "alerting:siem.queryRule/siem/rule/getRuleState",
                "alerting:siem.queryRule/siem/rule/getAlertSummary",
                "alerting:siem.queryRule/siem/rule/getExecutionLog",
                "alerting:siem.queryRule/siem/rule/getActionErrorLog",
                "alerting:siem.queryRule/siem/rule/find",
                "alerting:siem.queryRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.queryRule/siem/rule/getBackfill",
                "alerting:siem.queryRule/siem/rule/findBackfill",
                "alerting:siem.queryRule/siem/rule/create",
                "alerting:siem.queryRule/siem/rule/delete",
                "alerting:siem.queryRule/siem/rule/update",
                "alerting:siem.queryRule/siem/rule/updateApiKey",
                "alerting:siem.queryRule/siem/rule/enable",
                "alerting:siem.queryRule/siem/rule/disable",
                "alerting:siem.queryRule/siem/rule/muteAll",
                "alerting:siem.queryRule/siem/rule/unmuteAll",
                "alerting:siem.queryRule/siem/rule/muteAlert",
                "alerting:siem.queryRule/siem/rule/unmuteAlert",
                "alerting:siem.queryRule/siem/rule/snooze",
                "alerting:siem.queryRule/siem/rule/bulkEdit",
                "alerting:siem.queryRule/siem/rule/bulkDelete",
                "alerting:siem.queryRule/siem/rule/bulkEnable",
                "alerting:siem.queryRule/siem/rule/bulkDisable",
                "alerting:siem.queryRule/siem/rule/unsnooze",
                "alerting:siem.queryRule/siem/rule/runSoon",
                "alerting:siem.queryRule/siem/rule/scheduleBackfill",
                "alerting:siem.queryRule/siem/rule/deleteBackfill",
                "alerting:siem.savedQueryRule/siem/rule/get",
                "alerting:siem.savedQueryRule/siem/rule/getRuleState",
                "alerting:siem.savedQueryRule/siem/rule/getAlertSummary",
                "alerting:siem.savedQueryRule/siem/rule/getExecutionLog",
                "alerting:siem.savedQueryRule/siem/rule/getActionErrorLog",
                "alerting:siem.savedQueryRule/siem/rule/find",
                "alerting:siem.savedQueryRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.savedQueryRule/siem/rule/getBackfill",
                "alerting:siem.savedQueryRule/siem/rule/findBackfill",
                "alerting:siem.savedQueryRule/siem/rule/create",
                "alerting:siem.savedQueryRule/siem/rule/delete",
                "alerting:siem.savedQueryRule/siem/rule/update",
                "alerting:siem.savedQueryRule/siem/rule/updateApiKey",
                "alerting:siem.savedQueryRule/siem/rule/enable",
                "alerting:siem.savedQueryRule/siem/rule/disable",
                "alerting:siem.savedQueryRule/siem/rule/muteAll",
                "alerting:siem.savedQueryRule/siem/rule/unmuteAll",
                "alerting:siem.savedQueryRule/siem/rule/muteAlert",
                "alerting:siem.savedQueryRule/siem/rule/unmuteAlert",
                "alerting:siem.savedQueryRule/siem/rule/snooze",
                "alerting:siem.savedQueryRule/siem/rule/bulkEdit",
                "alerting:siem.savedQueryRule/siem/rule/bulkDelete",
                "alerting:siem.savedQueryRule/siem/rule/bulkEnable",
                "alerting:siem.savedQueryRule/siem/rule/bulkDisable",
                "alerting:siem.savedQueryRule/siem/rule/unsnooze",
                "alerting:siem.savedQueryRule/siem/rule/runSoon",
                "alerting:siem.savedQueryRule/siem/rule/scheduleBackfill",
                "alerting:siem.savedQueryRule/siem/rule/deleteBackfill",
                "alerting:siem.thresholdRule/siem/rule/get",
                "alerting:siem.thresholdRule/siem/rule/getRuleState",
                "alerting:siem.thresholdRule/siem/rule/getAlertSummary",
                "alerting:siem.thresholdRule/siem/rule/getExecutionLog",
                "alerting:siem.thresholdRule/siem/rule/getActionErrorLog",
                "alerting:siem.thresholdRule/siem/rule/find",
                "alerting:siem.thresholdRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.thresholdRule/siem/rule/getBackfill",
                "alerting:siem.thresholdRule/siem/rule/findBackfill",
                "alerting:siem.thresholdRule/siem/rule/create",
                "alerting:siem.thresholdRule/siem/rule/delete",
                "alerting:siem.thresholdRule/siem/rule/update",
                "alerting:siem.thresholdRule/siem/rule/updateApiKey",
                "alerting:siem.thresholdRule/siem/rule/enable",
                "alerting:siem.thresholdRule/siem/rule/disable",
                "alerting:siem.thresholdRule/siem/rule/muteAll",
                "alerting:siem.thresholdRule/siem/rule/unmuteAll",
                "alerting:siem.thresholdRule/siem/rule/muteAlert",
                "alerting:siem.thresholdRule/siem/rule/unmuteAlert",
                "alerting:siem.thresholdRule/siem/rule/snooze",
                "alerting:siem.thresholdRule/siem/rule/bulkEdit",
                "alerting:siem.thresholdRule/siem/rule/bulkDelete",
                "alerting:siem.thresholdRule/siem/rule/bulkEnable",
                "alerting:siem.thresholdRule/siem/rule/bulkDisable",
                "alerting:siem.thresholdRule/siem/rule/unsnooze",
                "alerting:siem.thresholdRule/siem/rule/runSoon",
                "alerting:siem.thresholdRule/siem/rule/scheduleBackfill",
                "alerting:siem.thresholdRule/siem/rule/deleteBackfill",
                "alerting:siem.newTermsRule/siem/rule/get",
                "alerting:siem.newTermsRule/siem/rule/getRuleState",
                "alerting:siem.newTermsRule/siem/rule/getAlertSummary",
                "alerting:siem.newTermsRule/siem/rule/getExecutionLog",
                "alerting:siem.newTermsRule/siem/rule/getActionErrorLog",
                "alerting:siem.newTermsRule/siem/rule/find",
                "alerting:siem.newTermsRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.newTermsRule/siem/rule/getBackfill",
                "alerting:siem.newTermsRule/siem/rule/findBackfill",
                "alerting:siem.newTermsRule/siem/rule/create",
                "alerting:siem.newTermsRule/siem/rule/delete",
                "alerting:siem.newTermsRule/siem/rule/update",
                "alerting:siem.newTermsRule/siem/rule/updateApiKey",
                "alerting:siem.newTermsRule/siem/rule/enable",
                "alerting:siem.newTermsRule/siem/rule/disable",
                "alerting:siem.newTermsRule/siem/rule/muteAll",
                "alerting:siem.newTermsRule/siem/rule/unmuteAll",
                "alerting:siem.newTermsRule/siem/rule/muteAlert",
                "alerting:siem.newTermsRule/siem/rule/unmuteAlert",
                "alerting:siem.newTermsRule/siem/rule/snooze",
                "alerting:siem.newTermsRule/siem/rule/bulkEdit",
                "alerting:siem.newTermsRule/siem/rule/bulkDelete",
                "alerting:siem.newTermsRule/siem/rule/bulkEnable",
                "alerting:siem.newTermsRule/siem/rule/bulkDisable",
                "alerting:siem.newTermsRule/siem/rule/unsnooze",
                "alerting:siem.newTermsRule/siem/rule/runSoon",
                "alerting:siem.newTermsRule/siem/rule/scheduleBackfill",
                "alerting:siem.newTermsRule/siem/rule/deleteBackfill",
                "alerting:siem.notifications/siem/alert/get",
                "alerting:siem.notifications/siem/alert/find",
                "alerting:siem.notifications/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.notifications/siem/alert/getAlertSummary",
                "alerting:siem.notifications/siem/alert/update",
                "alerting:siem.esqlRule/siem/alert/get",
                "alerting:siem.esqlRule/siem/alert/find",
                "alerting:siem.esqlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.esqlRule/siem/alert/getAlertSummary",
                "alerting:siem.esqlRule/siem/alert/update",
                "alerting:siem.eqlRule/siem/alert/get",
                "alerting:siem.eqlRule/siem/alert/find",
                "alerting:siem.eqlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.eqlRule/siem/alert/getAlertSummary",
                "alerting:siem.eqlRule/siem/alert/update",
                "alerting:siem.indicatorRule/siem/alert/get",
                "alerting:siem.indicatorRule/siem/alert/find",
                "alerting:siem.indicatorRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.indicatorRule/siem/alert/getAlertSummary",
                "alerting:siem.indicatorRule/siem/alert/update",
                "alerting:siem.mlRule/siem/alert/get",
                "alerting:siem.mlRule/siem/alert/find",
                "alerting:siem.mlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.mlRule/siem/alert/getAlertSummary",
                "alerting:siem.mlRule/siem/alert/update",
                "alerting:siem.queryRule/siem/alert/get",
                "alerting:siem.queryRule/siem/alert/find",
                "alerting:siem.queryRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.queryRule/siem/alert/getAlertSummary",
                "alerting:siem.queryRule/siem/alert/update",
                "alerting:siem.savedQueryRule/siem/alert/get",
                "alerting:siem.savedQueryRule/siem/alert/find",
                "alerting:siem.savedQueryRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.savedQueryRule/siem/alert/getAlertSummary",
                "alerting:siem.savedQueryRule/siem/alert/update",
                "alerting:siem.thresholdRule/siem/alert/get",
                "alerting:siem.thresholdRule/siem/alert/find",
                "alerting:siem.thresholdRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.thresholdRule/siem/alert/getAlertSummary",
                "alerting:siem.thresholdRule/siem/alert/update",
                "alerting:siem.newTermsRule/siem/alert/get",
                "alerting:siem.newTermsRule/siem/alert/find",
                "alerting:siem.newTermsRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.newTermsRule/siem/alert/getAlertSummary",
                "alerting:siem.newTermsRule/siem/alert/update",
                "api:fileUpload:analyzeFile",
                "api:store_search_session",
                "api:generateReport",
                "app:discover",
                "ui:catalogue/discover",
                "ui:management/kibana/search_sessions",
                "ui:management/insightsAndAlerting/reporting",
                "ui:navLinks/discover",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:search/create",
                "saved_object:search/bulk_create",
                "saved_object:search/update",
                "saved_object:search/bulk_update",
                "saved_object:search/delete",
                "saved_object:search/bulk_delete",
                "saved_object:search/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "ui:discover/show",
                "ui:discover/save",
                "ui:discover/saveQuery",
                "ui:discover/createShortUrl",
                "ui:discover/storeSearchSession",
                "ui:discover/generateCsv",
                "api:bulkGetUserProfiles",
                "api:downloadCsv",
                "app:dashboards",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:dashboard/create",
                "saved_object:dashboard/bulk_create",
                "saved_object:dashboard/update",
                "saved_object:dashboard/bulk_update",
                "saved_object:dashboard/delete",
                "saved_object:dashboard/bulk_delete",
                "saved_object:dashboard/share_to_space",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "ui:dashboard/createNew",
                "ui:dashboard/show",
                "ui:dashboard/showWriteControls",
                "ui:dashboard/saveQuery",
                "ui:dashboard/createShortUrl",
                "ui:dashboard/storeSearchSession",
                "ui:dashboard/generateScreenshot",
                "ui:dashboard/downloadCsv",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "saved_object:visualization/create",
                "saved_object:visualization/bulk_create",
                "saved_object:visualization/update",
                "saved_object:visualization/bulk_update",
                "saved_object:visualization/delete",
                "saved_object:visualization/bulk_delete",
                "saved_object:visualization/share_to_space",
                "saved_object:lens/create",
                "saved_object:lens/bulk_create",
                "saved_object:lens/update",
                "saved_object:lens/bulk_update",
                "saved_object:lens/delete",
                "saved_object:lens/bulk_delete",
                "saved_object:lens/share_to_space",
                "ui:visualize/show",
                "ui:visualize/delete",
                "ui:visualize/save",
                "ui:visualize/saveQuery",
                "ui:visualize/createShortUrl",
                "ui:visualize/generateScreenshot",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "saved_object:map/create",
                "saved_object:map/bulk_create",
                "saved_object:map/update",
                "saved_object:map/bulk_update",
                "saved_object:map/delete",
                "saved_object:map/bulk_delete",
                "saved_object:map/share_to_space",
                "ui:maps/save",
                "ui:maps/show",
                "ui:maps/saveQuery",
              ],
              "blocklist_all": Array [
                "login:",
                "api:lists-all",
                "api:lists-read",
                "api:lists-summary",
                "api:securitySolution-writeBlocklist",
                "api:securitySolution-readBlocklist",
                "saved_object:exception-list-agnostic/bulk_get",
                "saved_object:exception-list-agnostic/get",
                "saved_object:exception-list-agnostic/find",
                "saved_object:exception-list-agnostic/open_point_in_time",
                "saved_object:exception-list-agnostic/close_point_in_time",
                "saved_object:exception-list-agnostic/create",
                "saved_object:exception-list-agnostic/bulk_create",
                "saved_object:exception-list-agnostic/update",
                "saved_object:exception-list-agnostic/bulk_update",
                "saved_object:exception-list-agnostic/delete",
                "saved_object:exception-list-agnostic/bulk_delete",
                "saved_object:exception-list-agnostic/share_to_space",
                "ui:siem/writeBlocklist",
                "ui:siem/readBlocklist",
              ],
              "blocklist_read": Array [
                "login:",
                "api:lists-read",
                "api:lists-summary",
                "api:securitySolution-readBlocklist",
                "ui:siem/readBlocklist",
              ],
              "endpoint_exceptions_all": Array [
                "login:",
                "api:securitySolution-showEndpointExceptions",
                "api:securitySolution-crudEndpointExceptions",
                "ui:siem/showEndpointExceptions",
                "ui:siem/crudEndpointExceptions",
              ],
              "endpoint_exceptions_read": Array [
                "login:",
                "api:securitySolution-showEndpointExceptions",
                "ui:siem/showEndpointExceptions",
              ],
              "endpoint_list_all": Array [
                "login:",
                "api:securitySolution-writeEndpointList",
                "api:securitySolution-readEndpointList",
                "ui:siem/writeEndpointList",
                "ui:siem/readEndpointList",
              ],
              "endpoint_list_read": Array [
                "login:",
                "api:securitySolution-readEndpointList",
                "ui:siem/readEndpointList",
              ],
              "event_filters_all": Array [
                "login:",
                "api:lists-all",
                "api:lists-read",
                "api:lists-summary",
                "api:securitySolution-writeEventFilters",
                "api:securitySolution-readEventFilters",
                "saved_object:exception-list-agnostic/bulk_get",
                "saved_object:exception-list-agnostic/get",
                "saved_object:exception-list-agnostic/find",
                "saved_object:exception-list-agnostic/open_point_in_time",
                "saved_object:exception-list-agnostic/close_point_in_time",
                "saved_object:exception-list-agnostic/create",
                "saved_object:exception-list-agnostic/bulk_create",
                "saved_object:exception-list-agnostic/update",
                "saved_object:exception-list-agnostic/bulk_update",
                "saved_object:exception-list-agnostic/delete",
                "saved_object:exception-list-agnostic/bulk_delete",
                "saved_object:exception-list-agnostic/share_to_space",
                "ui:siem/writeEventFilters",
                "ui:siem/readEventFilters",
              ],
              "event_filters_read": Array [
                "login:",
                "api:lists-read",
                "api:lists-summary",
                "api:securitySolution-readEventFilters",
                "ui:siem/readEventFilters",
              ],
              "execute_operations_all": Array [
                "login:",
                "api:securitySolution-writeExecuteOperations",
                "ui:siem/writeExecuteOperations",
              ],
              "file_operations_all": Array [
                "login:",
                "api:securitySolution-writeFileOperations",
                "ui:siem/writeFileOperations",
              ],
              "host_isolation_all": Array [
                "login:",
                "api:securitySolution-writeHostIsolationRelease",
                "api:securitySolution-writeHostIsolation",
                "ui:siem/writeHostIsolationRelease",
                "ui:siem/writeHostIsolation",
              ],
              "host_isolation_exceptions_all": Array [
                "login:",
                "api:lists-all",
                "api:lists-read",
                "api:lists-summary",
                "api:securitySolution-deleteHostIsolationExceptions",
                "api:securitySolution-readHostIsolationExceptions",
                "api:securitySolution-accessHostIsolationExceptions",
                "api:securitySolution-writeHostIsolationExceptions",
                "saved_object:exception-list-agnostic/bulk_get",
                "saved_object:exception-list-agnostic/get",
                "saved_object:exception-list-agnostic/find",
                "saved_object:exception-list-agnostic/open_point_in_time",
                "saved_object:exception-list-agnostic/close_point_in_time",
                "saved_object:exception-list-agnostic/create",
                "saved_object:exception-list-agnostic/bulk_create",
                "saved_object:exception-list-agnostic/update",
                "saved_object:exception-list-agnostic/bulk_update",
                "saved_object:exception-list-agnostic/delete",
                "saved_object:exception-list-agnostic/bulk_delete",
                "saved_object:exception-list-agnostic/share_to_space",
                "ui:siem/readHostIsolationExceptions",
                "ui:siem/deleteHostIsolationExceptions",
                "ui:siem/accessHostIsolationExceptions",
                "ui:siem/writeHostIsolationExceptions",
              ],
              "host_isolation_exceptions_read": Array [
                "login:",
                "api:lists-read",
                "api:lists-summary",
                "api:securitySolution-readHostIsolationExceptions",
                "api:securitySolution-accessHostIsolationExceptions",
                "ui:siem/readHostIsolationExceptions",
                "ui:siem/accessHostIsolationExceptions",
              ],
              "minimal_all": Array [
                "login:",
                "api:securitySolution",
                "api:lists-all",
                "api:lists-read",
                "api:lists-summary",
                "api:rac",
                "api:cloud-security-posture-all",
                "api:cloud-security-posture-read",
                "api:cloud-defend-all",
                "api:cloud-defend-read",
                "api:securitySolution-entity-analytics",
                "api:securitySolution-threat-intelligence",
                "app:securitySolution",
                "app:csp",
                "app:cloudDefend",
                "app:kibana",
                "ui:catalogue/securitySolution",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/securitySolution",
                "ui:navLinks/csp",
                "ui:navLinks/cloudDefend",
                "ui:navLinks/kibana",
                "saved_object:alert/bulk_get",
                "saved_object:alert/get",
                "saved_object:alert/find",
                "saved_object:alert/open_point_in_time",
                "saved_object:alert/close_point_in_time",
                "saved_object:alert/create",
                "saved_object:alert/bulk_create",
                "saved_object:alert/update",
                "saved_object:alert/bulk_update",
                "saved_object:alert/delete",
                "saved_object:alert/bulk_delete",
                "saved_object:alert/share_to_space",
                "saved_object:exception-list/bulk_get",
                "saved_object:exception-list/get",
                "saved_object:exception-list/find",
                "saved_object:exception-list/open_point_in_time",
                "saved_object:exception-list/close_point_in_time",
                "saved_object:exception-list/create",
                "saved_object:exception-list/bulk_create",
                "saved_object:exception-list/update",
                "saved_object:exception-list/bulk_update",
                "saved_object:exception-list/delete",
                "saved_object:exception-list/bulk_delete",
                "saved_object:exception-list/share_to_space",
                "saved_object:exception-list-agnostic/bulk_get",
                "saved_object:exception-list-agnostic/get",
                "saved_object:exception-list-agnostic/find",
                "saved_object:exception-list-agnostic/open_point_in_time",
                "saved_object:exception-list-agnostic/close_point_in_time",
                "saved_object:exception-list-agnostic/create",
                "saved_object:exception-list-agnostic/bulk_create",
                "saved_object:exception-list-agnostic/update",
                "saved_object:exception-list-agnostic/bulk_update",
                "saved_object:exception-list-agnostic/delete",
                "saved_object:exception-list-agnostic/bulk_delete",
                "saved_object:exception-list-agnostic/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:index-pattern/create",
                "saved_object:index-pattern/bulk_create",
                "saved_object:index-pattern/update",
                "saved_object:index-pattern/bulk_update",
                "saved_object:index-pattern/delete",
                "saved_object:index-pattern/bulk_delete",
                "saved_object:index-pattern/share_to_space",
                "saved_object:siem-ui-timeline-note/bulk_get",
                "saved_object:siem-ui-timeline-note/get",
                "saved_object:siem-ui-timeline-note/find",
                "saved_object:siem-ui-timeline-note/open_point_in_time",
                "saved_object:siem-ui-timeline-note/close_point_in_time",
                "saved_object:siem-ui-timeline-note/create",
                "saved_object:siem-ui-timeline-note/bulk_create",
                "saved_object:siem-ui-timeline-note/update",
                "saved_object:siem-ui-timeline-note/bulk_update",
                "saved_object:siem-ui-timeline-note/delete",
                "saved_object:siem-ui-timeline-note/bulk_delete",
                "saved_object:siem-ui-timeline-note/share_to_space",
                "saved_object:siem-ui-timeline-pinned-event/bulk_get",
                "saved_object:siem-ui-timeline-pinned-event/get",
                "saved_object:siem-ui-timeline-pinned-event/find",
                "saved_object:siem-ui-timeline-pinned-event/open_point_in_time",
                "saved_object:siem-ui-timeline-pinned-event/close_point_in_time",
                "saved_object:siem-ui-timeline-pinned-event/create",
                "saved_object:siem-ui-timeline-pinned-event/bulk_create",
                "saved_object:siem-ui-timeline-pinned-event/update",
                "saved_object:siem-ui-timeline-pinned-event/bulk_update",
                "saved_object:siem-ui-timeline-pinned-event/delete",
                "saved_object:siem-ui-timeline-pinned-event/bulk_delete",
                "saved_object:siem-ui-timeline-pinned-event/share_to_space",
                "saved_object:siem-detection-engine-rule-actions/bulk_get",
                "saved_object:siem-detection-engine-rule-actions/get",
                "saved_object:siem-detection-engine-rule-actions/find",
                "saved_object:siem-detection-engine-rule-actions/open_point_in_time",
                "saved_object:siem-detection-engine-rule-actions/close_point_in_time",
                "saved_object:siem-detection-engine-rule-actions/create",
                "saved_object:siem-detection-engine-rule-actions/bulk_create",
                "saved_object:siem-detection-engine-rule-actions/update",
                "saved_object:siem-detection-engine-rule-actions/bulk_update",
                "saved_object:siem-detection-engine-rule-actions/delete",
                "saved_object:siem-detection-engine-rule-actions/bulk_delete",
                "saved_object:siem-detection-engine-rule-actions/share_to_space",
                "saved_object:security-rule/bulk_get",
                "saved_object:security-rule/get",
                "saved_object:security-rule/find",
                "saved_object:security-rule/open_point_in_time",
                "saved_object:security-rule/close_point_in_time",
                "saved_object:security-rule/create",
                "saved_object:security-rule/bulk_create",
                "saved_object:security-rule/update",
                "saved_object:security-rule/bulk_update",
                "saved_object:security-rule/delete",
                "saved_object:security-rule/bulk_delete",
                "saved_object:security-rule/share_to_space",
                "saved_object:siem-ui-timeline/bulk_get",
                "saved_object:siem-ui-timeline/get",
                "saved_object:siem-ui-timeline/find",
                "saved_object:siem-ui-timeline/open_point_in_time",
                "saved_object:siem-ui-timeline/close_point_in_time",
                "saved_object:siem-ui-timeline/create",
                "saved_object:siem-ui-timeline/bulk_create",
                "saved_object:siem-ui-timeline/update",
                "saved_object:siem-ui-timeline/bulk_update",
                "saved_object:siem-ui-timeline/delete",
                "saved_object:siem-ui-timeline/bulk_delete",
                "saved_object:siem-ui-timeline/share_to_space",
                "saved_object:endpoint:user-artifact-manifest/bulk_get",
                "saved_object:endpoint:user-artifact-manifest/get",
                "saved_object:endpoint:user-artifact-manifest/find",
                "saved_object:endpoint:user-artifact-manifest/open_point_in_time",
                "saved_object:endpoint:user-artifact-manifest/close_point_in_time",
                "saved_object:endpoint:user-artifact-manifest/create",
                "saved_object:endpoint:user-artifact-manifest/bulk_create",
                "saved_object:endpoint:user-artifact-manifest/update",
                "saved_object:endpoint:user-artifact-manifest/bulk_update",
                "saved_object:endpoint:user-artifact-manifest/delete",
                "saved_object:endpoint:user-artifact-manifest/bulk_delete",
                "saved_object:endpoint:user-artifact-manifest/share_to_space",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_get",
                "saved_object:endpoint:unified-user-artifact-manifest/get",
                "saved_object:endpoint:unified-user-artifact-manifest/find",
                "saved_object:endpoint:unified-user-artifact-manifest/open_point_in_time",
                "saved_object:endpoint:unified-user-artifact-manifest/close_point_in_time",
                "saved_object:endpoint:unified-user-artifact-manifest/create",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_create",
                "saved_object:endpoint:unified-user-artifact-manifest/update",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_update",
                "saved_object:endpoint:unified-user-artifact-manifest/delete",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_delete",
                "saved_object:endpoint:unified-user-artifact-manifest/share_to_space",
                "saved_object:security-solution-signals-migration/bulk_get",
                "saved_object:security-solution-signals-migration/get",
                "saved_object:security-solution-signals-migration/find",
                "saved_object:security-solution-signals-migration/open_point_in_time",
                "saved_object:security-solution-signals-migration/close_point_in_time",
                "saved_object:security-solution-signals-migration/create",
                "saved_object:security-solution-signals-migration/bulk_create",
                "saved_object:security-solution-signals-migration/update",
                "saved_object:security-solution-signals-migration/bulk_update",
                "saved_object:security-solution-signals-migration/delete",
                "saved_object:security-solution-signals-migration/bulk_delete",
                "saved_object:security-solution-signals-migration/share_to_space",
                "saved_object:risk-engine-configuration/bulk_get",
                "saved_object:risk-engine-configuration/get",
                "saved_object:risk-engine-configuration/find",
                "saved_object:risk-engine-configuration/open_point_in_time",
                "saved_object:risk-engine-configuration/close_point_in_time",
                "saved_object:risk-engine-configuration/create",
                "saved_object:risk-engine-configuration/bulk_create",
                "saved_object:risk-engine-configuration/update",
                "saved_object:risk-engine-configuration/bulk_update",
                "saved_object:risk-engine-configuration/delete",
                "saved_object:risk-engine-configuration/bulk_delete",
                "saved_object:risk-engine-configuration/share_to_space",
                "saved_object:policy-settings-protection-updates-note/bulk_get",
                "saved_object:policy-settings-protection-updates-note/get",
                "saved_object:policy-settings-protection-updates-note/find",
                "saved_object:policy-settings-protection-updates-note/open_point_in_time",
                "saved_object:policy-settings-protection-updates-note/close_point_in_time",
                "saved_object:policy-settings-protection-updates-note/create",
                "saved_object:policy-settings-protection-updates-note/bulk_create",
                "saved_object:policy-settings-protection-updates-note/update",
                "saved_object:policy-settings-protection-updates-note/bulk_update",
                "saved_object:policy-settings-protection-updates-note/delete",
                "saved_object:policy-settings-protection-updates-note/bulk_delete",
                "saved_object:policy-settings-protection-updates-note/share_to_space",
                "saved_object:csp_rule/bulk_get",
                "saved_object:csp_rule/get",
                "saved_object:csp_rule/find",
                "saved_object:csp_rule/open_point_in_time",
                "saved_object:csp_rule/close_point_in_time",
                "saved_object:csp_rule/create",
                "saved_object:csp_rule/bulk_create",
                "saved_object:csp_rule/update",
                "saved_object:csp_rule/bulk_update",
                "saved_object:csp_rule/delete",
                "saved_object:csp_rule/bulk_delete",
                "saved_object:csp_rule/share_to_space",
                "saved_object:cloud-security-posture-settings/bulk_get",
                "saved_object:cloud-security-posture-settings/get",
                "saved_object:cloud-security-posture-settings/find",
                "saved_object:cloud-security-posture-settings/open_point_in_time",
                "saved_object:cloud-security-posture-settings/close_point_in_time",
                "saved_object:cloud-security-posture-settings/create",
                "saved_object:cloud-security-posture-settings/bulk_create",
                "saved_object:cloud-security-posture-settings/update",
                "saved_object:cloud-security-posture-settings/bulk_update",
                "saved_object:cloud-security-posture-settings/delete",
                "saved_object:cloud-security-posture-settings/bulk_delete",
                "saved_object:cloud-security-posture-settings/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:siem/show",
                "ui:siem/crud",
                "ui:siem/entity-analytics",
                "ui:siem/investigation-guide",
                "ui:siem/investigation-guide-interactions",
                "ui:siem/threat-intelligence",
                "alerting:siem.notifications/siem/rule/get",
                "alerting:siem.notifications/siem/rule/getRuleState",
                "alerting:siem.notifications/siem/rule/getAlertSummary",
                "alerting:siem.notifications/siem/rule/getExecutionLog",
                "alerting:siem.notifications/siem/rule/getActionErrorLog",
                "alerting:siem.notifications/siem/rule/find",
                "alerting:siem.notifications/siem/rule/getRuleExecutionKPI",
                "alerting:siem.notifications/siem/rule/getBackfill",
                "alerting:siem.notifications/siem/rule/findBackfill",
                "alerting:siem.notifications/siem/rule/create",
                "alerting:siem.notifications/siem/rule/delete",
                "alerting:siem.notifications/siem/rule/update",
                "alerting:siem.notifications/siem/rule/updateApiKey",
                "alerting:siem.notifications/siem/rule/enable",
                "alerting:siem.notifications/siem/rule/disable",
                "alerting:siem.notifications/siem/rule/muteAll",
                "alerting:siem.notifications/siem/rule/unmuteAll",
                "alerting:siem.notifications/siem/rule/muteAlert",
                "alerting:siem.notifications/siem/rule/unmuteAlert",
                "alerting:siem.notifications/siem/rule/snooze",
                "alerting:siem.notifications/siem/rule/bulkEdit",
                "alerting:siem.notifications/siem/rule/bulkDelete",
                "alerting:siem.notifications/siem/rule/bulkEnable",
                "alerting:siem.notifications/siem/rule/bulkDisable",
                "alerting:siem.notifications/siem/rule/unsnooze",
                "alerting:siem.notifications/siem/rule/runSoon",
                "alerting:siem.notifications/siem/rule/scheduleBackfill",
                "alerting:siem.notifications/siem/rule/deleteBackfill",
                "alerting:siem.esqlRule/siem/rule/get",
                "alerting:siem.esqlRule/siem/rule/getRuleState",
                "alerting:siem.esqlRule/siem/rule/getAlertSummary",
                "alerting:siem.esqlRule/siem/rule/getExecutionLog",
                "alerting:siem.esqlRule/siem/rule/getActionErrorLog",
                "alerting:siem.esqlRule/siem/rule/find",
                "alerting:siem.esqlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.esqlRule/siem/rule/getBackfill",
                "alerting:siem.esqlRule/siem/rule/findBackfill",
                "alerting:siem.esqlRule/siem/rule/create",
                "alerting:siem.esqlRule/siem/rule/delete",
                "alerting:siem.esqlRule/siem/rule/update",
                "alerting:siem.esqlRule/siem/rule/updateApiKey",
                "alerting:siem.esqlRule/siem/rule/enable",
                "alerting:siem.esqlRule/siem/rule/disable",
                "alerting:siem.esqlRule/siem/rule/muteAll",
                "alerting:siem.esqlRule/siem/rule/unmuteAll",
                "alerting:siem.esqlRule/siem/rule/muteAlert",
                "alerting:siem.esqlRule/siem/rule/unmuteAlert",
                "alerting:siem.esqlRule/siem/rule/snooze",
                "alerting:siem.esqlRule/siem/rule/bulkEdit",
                "alerting:siem.esqlRule/siem/rule/bulkDelete",
                "alerting:siem.esqlRule/siem/rule/bulkEnable",
                "alerting:siem.esqlRule/siem/rule/bulkDisable",
                "alerting:siem.esqlRule/siem/rule/unsnooze",
                "alerting:siem.esqlRule/siem/rule/runSoon",
                "alerting:siem.esqlRule/siem/rule/scheduleBackfill",
                "alerting:siem.esqlRule/siem/rule/deleteBackfill",
                "alerting:siem.eqlRule/siem/rule/get",
                "alerting:siem.eqlRule/siem/rule/getRuleState",
                "alerting:siem.eqlRule/siem/rule/getAlertSummary",
                "alerting:siem.eqlRule/siem/rule/getExecutionLog",
                "alerting:siem.eqlRule/siem/rule/getActionErrorLog",
                "alerting:siem.eqlRule/siem/rule/find",
                "alerting:siem.eqlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.eqlRule/siem/rule/getBackfill",
                "alerting:siem.eqlRule/siem/rule/findBackfill",
                "alerting:siem.eqlRule/siem/rule/create",
                "alerting:siem.eqlRule/siem/rule/delete",
                "alerting:siem.eqlRule/siem/rule/update",
                "alerting:siem.eqlRule/siem/rule/updateApiKey",
                "alerting:siem.eqlRule/siem/rule/enable",
                "alerting:siem.eqlRule/siem/rule/disable",
                "alerting:siem.eqlRule/siem/rule/muteAll",
                "alerting:siem.eqlRule/siem/rule/unmuteAll",
                "alerting:siem.eqlRule/siem/rule/muteAlert",
                "alerting:siem.eqlRule/siem/rule/unmuteAlert",
                "alerting:siem.eqlRule/siem/rule/snooze",
                "alerting:siem.eqlRule/siem/rule/bulkEdit",
                "alerting:siem.eqlRule/siem/rule/bulkDelete",
                "alerting:siem.eqlRule/siem/rule/bulkEnable",
                "alerting:siem.eqlRule/siem/rule/bulkDisable",
                "alerting:siem.eqlRule/siem/rule/unsnooze",
                "alerting:siem.eqlRule/siem/rule/runSoon",
                "alerting:siem.eqlRule/siem/rule/scheduleBackfill",
                "alerting:siem.eqlRule/siem/rule/deleteBackfill",
                "alerting:siem.indicatorRule/siem/rule/get",
                "alerting:siem.indicatorRule/siem/rule/getRuleState",
                "alerting:siem.indicatorRule/siem/rule/getAlertSummary",
                "alerting:siem.indicatorRule/siem/rule/getExecutionLog",
                "alerting:siem.indicatorRule/siem/rule/getActionErrorLog",
                "alerting:siem.indicatorRule/siem/rule/find",
                "alerting:siem.indicatorRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.indicatorRule/siem/rule/getBackfill",
                "alerting:siem.indicatorRule/siem/rule/findBackfill",
                "alerting:siem.indicatorRule/siem/rule/create",
                "alerting:siem.indicatorRule/siem/rule/delete",
                "alerting:siem.indicatorRule/siem/rule/update",
                "alerting:siem.indicatorRule/siem/rule/updateApiKey",
                "alerting:siem.indicatorRule/siem/rule/enable",
                "alerting:siem.indicatorRule/siem/rule/disable",
                "alerting:siem.indicatorRule/siem/rule/muteAll",
                "alerting:siem.indicatorRule/siem/rule/unmuteAll",
                "alerting:siem.indicatorRule/siem/rule/muteAlert",
                "alerting:siem.indicatorRule/siem/rule/unmuteAlert",
                "alerting:siem.indicatorRule/siem/rule/snooze",
                "alerting:siem.indicatorRule/siem/rule/bulkEdit",
                "alerting:siem.indicatorRule/siem/rule/bulkDelete",
                "alerting:siem.indicatorRule/siem/rule/bulkEnable",
                "alerting:siem.indicatorRule/siem/rule/bulkDisable",
                "alerting:siem.indicatorRule/siem/rule/unsnooze",
                "alerting:siem.indicatorRule/siem/rule/runSoon",
                "alerting:siem.indicatorRule/siem/rule/scheduleBackfill",
                "alerting:siem.indicatorRule/siem/rule/deleteBackfill",
                "alerting:siem.mlRule/siem/rule/get",
                "alerting:siem.mlRule/siem/rule/getRuleState",
                "alerting:siem.mlRule/siem/rule/getAlertSummary",
                "alerting:siem.mlRule/siem/rule/getExecutionLog",
                "alerting:siem.mlRule/siem/rule/getActionErrorLog",
                "alerting:siem.mlRule/siem/rule/find",
                "alerting:siem.mlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.mlRule/siem/rule/getBackfill",
                "alerting:siem.mlRule/siem/rule/findBackfill",
                "alerting:siem.mlRule/siem/rule/create",
                "alerting:siem.mlRule/siem/rule/delete",
                "alerting:siem.mlRule/siem/rule/update",
                "alerting:siem.mlRule/siem/rule/updateApiKey",
                "alerting:siem.mlRule/siem/rule/enable",
                "alerting:siem.mlRule/siem/rule/disable",
                "alerting:siem.mlRule/siem/rule/muteAll",
                "alerting:siem.mlRule/siem/rule/unmuteAll",
                "alerting:siem.mlRule/siem/rule/muteAlert",
                "alerting:siem.mlRule/siem/rule/unmuteAlert",
                "alerting:siem.mlRule/siem/rule/snooze",
                "alerting:siem.mlRule/siem/rule/bulkEdit",
                "alerting:siem.mlRule/siem/rule/bulkDelete",
                "alerting:siem.mlRule/siem/rule/bulkEnable",
                "alerting:siem.mlRule/siem/rule/bulkDisable",
                "alerting:siem.mlRule/siem/rule/unsnooze",
                "alerting:siem.mlRule/siem/rule/runSoon",
                "alerting:siem.mlRule/siem/rule/scheduleBackfill",
                "alerting:siem.mlRule/siem/rule/deleteBackfill",
                "alerting:siem.queryRule/siem/rule/get",
                "alerting:siem.queryRule/siem/rule/getRuleState",
                "alerting:siem.queryRule/siem/rule/getAlertSummary",
                "alerting:siem.queryRule/siem/rule/getExecutionLog",
                "alerting:siem.queryRule/siem/rule/getActionErrorLog",
                "alerting:siem.queryRule/siem/rule/find",
                "alerting:siem.queryRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.queryRule/siem/rule/getBackfill",
                "alerting:siem.queryRule/siem/rule/findBackfill",
                "alerting:siem.queryRule/siem/rule/create",
                "alerting:siem.queryRule/siem/rule/delete",
                "alerting:siem.queryRule/siem/rule/update",
                "alerting:siem.queryRule/siem/rule/updateApiKey",
                "alerting:siem.queryRule/siem/rule/enable",
                "alerting:siem.queryRule/siem/rule/disable",
                "alerting:siem.queryRule/siem/rule/muteAll",
                "alerting:siem.queryRule/siem/rule/unmuteAll",
                "alerting:siem.queryRule/siem/rule/muteAlert",
                "alerting:siem.queryRule/siem/rule/unmuteAlert",
                "alerting:siem.queryRule/siem/rule/snooze",
                "alerting:siem.queryRule/siem/rule/bulkEdit",
                "alerting:siem.queryRule/siem/rule/bulkDelete",
                "alerting:siem.queryRule/siem/rule/bulkEnable",
                "alerting:siem.queryRule/siem/rule/bulkDisable",
                "alerting:siem.queryRule/siem/rule/unsnooze",
                "alerting:siem.queryRule/siem/rule/runSoon",
                "alerting:siem.queryRule/siem/rule/scheduleBackfill",
                "alerting:siem.queryRule/siem/rule/deleteBackfill",
                "alerting:siem.savedQueryRule/siem/rule/get",
                "alerting:siem.savedQueryRule/siem/rule/getRuleState",
                "alerting:siem.savedQueryRule/siem/rule/getAlertSummary",
                "alerting:siem.savedQueryRule/siem/rule/getExecutionLog",
                "alerting:siem.savedQueryRule/siem/rule/getActionErrorLog",
                "alerting:siem.savedQueryRule/siem/rule/find",
                "alerting:siem.savedQueryRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.savedQueryRule/siem/rule/getBackfill",
                "alerting:siem.savedQueryRule/siem/rule/findBackfill",
                "alerting:siem.savedQueryRule/siem/rule/create",
                "alerting:siem.savedQueryRule/siem/rule/delete",
                "alerting:siem.savedQueryRule/siem/rule/update",
                "alerting:siem.savedQueryRule/siem/rule/updateApiKey",
                "alerting:siem.savedQueryRule/siem/rule/enable",
                "alerting:siem.savedQueryRule/siem/rule/disable",
                "alerting:siem.savedQueryRule/siem/rule/muteAll",
                "alerting:siem.savedQueryRule/siem/rule/unmuteAll",
                "alerting:siem.savedQueryRule/siem/rule/muteAlert",
                "alerting:siem.savedQueryRule/siem/rule/unmuteAlert",
                "alerting:siem.savedQueryRule/siem/rule/snooze",
                "alerting:siem.savedQueryRule/siem/rule/bulkEdit",
                "alerting:siem.savedQueryRule/siem/rule/bulkDelete",
                "alerting:siem.savedQueryRule/siem/rule/bulkEnable",
                "alerting:siem.savedQueryRule/siem/rule/bulkDisable",
                "alerting:siem.savedQueryRule/siem/rule/unsnooze",
                "alerting:siem.savedQueryRule/siem/rule/runSoon",
                "alerting:siem.savedQueryRule/siem/rule/scheduleBackfill",
                "alerting:siem.savedQueryRule/siem/rule/deleteBackfill",
                "alerting:siem.thresholdRule/siem/rule/get",
                "alerting:siem.thresholdRule/siem/rule/getRuleState",
                "alerting:siem.thresholdRule/siem/rule/getAlertSummary",
                "alerting:siem.thresholdRule/siem/rule/getExecutionLog",
                "alerting:siem.thresholdRule/siem/rule/getActionErrorLog",
                "alerting:siem.thresholdRule/siem/rule/find",
                "alerting:siem.thresholdRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.thresholdRule/siem/rule/getBackfill",
                "alerting:siem.thresholdRule/siem/rule/findBackfill",
                "alerting:siem.thresholdRule/siem/rule/create",
                "alerting:siem.thresholdRule/siem/rule/delete",
                "alerting:siem.thresholdRule/siem/rule/update",
                "alerting:siem.thresholdRule/siem/rule/updateApiKey",
                "alerting:siem.thresholdRule/siem/rule/enable",
                "alerting:siem.thresholdRule/siem/rule/disable",
                "alerting:siem.thresholdRule/siem/rule/muteAll",
                "alerting:siem.thresholdRule/siem/rule/unmuteAll",
                "alerting:siem.thresholdRule/siem/rule/muteAlert",
                "alerting:siem.thresholdRule/siem/rule/unmuteAlert",
                "alerting:siem.thresholdRule/siem/rule/snooze",
                "alerting:siem.thresholdRule/siem/rule/bulkEdit",
                "alerting:siem.thresholdRule/siem/rule/bulkDelete",
                "alerting:siem.thresholdRule/siem/rule/bulkEnable",
                "alerting:siem.thresholdRule/siem/rule/bulkDisable",
                "alerting:siem.thresholdRule/siem/rule/unsnooze",
                "alerting:siem.thresholdRule/siem/rule/runSoon",
                "alerting:siem.thresholdRule/siem/rule/scheduleBackfill",
                "alerting:siem.thresholdRule/siem/rule/deleteBackfill",
                "alerting:siem.newTermsRule/siem/rule/get",
                "alerting:siem.newTermsRule/siem/rule/getRuleState",
                "alerting:siem.newTermsRule/siem/rule/getAlertSummary",
                "alerting:siem.newTermsRule/siem/rule/getExecutionLog",
                "alerting:siem.newTermsRule/siem/rule/getActionErrorLog",
                "alerting:siem.newTermsRule/siem/rule/find",
                "alerting:siem.newTermsRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.newTermsRule/siem/rule/getBackfill",
                "alerting:siem.newTermsRule/siem/rule/findBackfill",
                "alerting:siem.newTermsRule/siem/rule/create",
                "alerting:siem.newTermsRule/siem/rule/delete",
                "alerting:siem.newTermsRule/siem/rule/update",
                "alerting:siem.newTermsRule/siem/rule/updateApiKey",
                "alerting:siem.newTermsRule/siem/rule/enable",
                "alerting:siem.newTermsRule/siem/rule/disable",
                "alerting:siem.newTermsRule/siem/rule/muteAll",
                "alerting:siem.newTermsRule/siem/rule/unmuteAll",
                "alerting:siem.newTermsRule/siem/rule/muteAlert",
                "alerting:siem.newTermsRule/siem/rule/unmuteAlert",
                "alerting:siem.newTermsRule/siem/rule/snooze",
                "alerting:siem.newTermsRule/siem/rule/bulkEdit",
                "alerting:siem.newTermsRule/siem/rule/bulkDelete",
                "alerting:siem.newTermsRule/siem/rule/bulkEnable",
                "alerting:siem.newTermsRule/siem/rule/bulkDisable",
                "alerting:siem.newTermsRule/siem/rule/unsnooze",
                "alerting:siem.newTermsRule/siem/rule/runSoon",
                "alerting:siem.newTermsRule/siem/rule/scheduleBackfill",
                "alerting:siem.newTermsRule/siem/rule/deleteBackfill",
                "alerting:siem.notifications/siem/alert/get",
                "alerting:siem.notifications/siem/alert/find",
                "alerting:siem.notifications/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.notifications/siem/alert/getAlertSummary",
                "alerting:siem.notifications/siem/alert/update",
                "alerting:siem.esqlRule/siem/alert/get",
                "alerting:siem.esqlRule/siem/alert/find",
                "alerting:siem.esqlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.esqlRule/siem/alert/getAlertSummary",
                "alerting:siem.esqlRule/siem/alert/update",
                "alerting:siem.eqlRule/siem/alert/get",
                "alerting:siem.eqlRule/siem/alert/find",
                "alerting:siem.eqlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.eqlRule/siem/alert/getAlertSummary",
                "alerting:siem.eqlRule/siem/alert/update",
                "alerting:siem.indicatorRule/siem/alert/get",
                "alerting:siem.indicatorRule/siem/alert/find",
                "alerting:siem.indicatorRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.indicatorRule/siem/alert/getAlertSummary",
                "alerting:siem.indicatorRule/siem/alert/update",
                "alerting:siem.mlRule/siem/alert/get",
                "alerting:siem.mlRule/siem/alert/find",
                "alerting:siem.mlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.mlRule/siem/alert/getAlertSummary",
                "alerting:siem.mlRule/siem/alert/update",
                "alerting:siem.queryRule/siem/alert/get",
                "alerting:siem.queryRule/siem/alert/find",
                "alerting:siem.queryRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.queryRule/siem/alert/getAlertSummary",
                "alerting:siem.queryRule/siem/alert/update",
                "alerting:siem.savedQueryRule/siem/alert/get",
                "alerting:siem.savedQueryRule/siem/alert/find",
                "alerting:siem.savedQueryRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.savedQueryRule/siem/alert/getAlertSummary",
                "alerting:siem.savedQueryRule/siem/alert/update",
                "alerting:siem.thresholdRule/siem/alert/get",
                "alerting:siem.thresholdRule/siem/alert/find",
                "alerting:siem.thresholdRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.thresholdRule/siem/alert/getAlertSummary",
                "alerting:siem.thresholdRule/siem/alert/update",
                "alerting:siem.newTermsRule/siem/alert/get",
                "alerting:siem.newTermsRule/siem/alert/find",
                "alerting:siem.newTermsRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.newTermsRule/siem/alert/getAlertSummary",
                "alerting:siem.newTermsRule/siem/alert/update",
                "api:fileUpload:analyzeFile",
                "api:store_search_session",
                "api:generateReport",
                "app:discover",
                "ui:catalogue/discover",
                "ui:management/kibana/search_sessions",
                "ui:management/insightsAndAlerting/reporting",
                "ui:navLinks/discover",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:search/create",
                "saved_object:search/bulk_create",
                "saved_object:search/update",
                "saved_object:search/bulk_update",
                "saved_object:search/delete",
                "saved_object:search/bulk_delete",
                "saved_object:search/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "ui:discover/show",
                "ui:discover/save",
                "ui:discover/saveQuery",
                "ui:discover/createShortUrl",
                "ui:discover/storeSearchSession",
                "ui:discover/generateCsv",
                "api:bulkGetUserProfiles",
                "api:downloadCsv",
                "app:dashboards",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:dashboard/create",
                "saved_object:dashboard/bulk_create",
                "saved_object:dashboard/update",
                "saved_object:dashboard/bulk_update",
                "saved_object:dashboard/delete",
                "saved_object:dashboard/bulk_delete",
                "saved_object:dashboard/share_to_space",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "ui:dashboard/createNew",
                "ui:dashboard/show",
                "ui:dashboard/showWriteControls",
                "ui:dashboard/saveQuery",
                "ui:dashboard/createShortUrl",
                "ui:dashboard/storeSearchSession",
                "ui:dashboard/generateScreenshot",
                "ui:dashboard/downloadCsv",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "saved_object:visualization/create",
                "saved_object:visualization/bulk_create",
                "saved_object:visualization/update",
                "saved_object:visualization/bulk_update",
                "saved_object:visualization/delete",
                "saved_object:visualization/bulk_delete",
                "saved_object:visualization/share_to_space",
                "saved_object:lens/create",
                "saved_object:lens/bulk_create",
                "saved_object:lens/update",
                "saved_object:lens/bulk_update",
                "saved_object:lens/delete",
                "saved_object:lens/bulk_delete",
                "saved_object:lens/share_to_space",
                "ui:visualize/show",
                "ui:visualize/delete",
                "ui:visualize/save",
                "ui:visualize/saveQuery",
                "ui:visualize/createShortUrl",
                "ui:visualize/generateScreenshot",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "saved_object:map/create",
                "saved_object:map/bulk_create",
                "saved_object:map/update",
                "saved_object:map/bulk_update",
                "saved_object:map/delete",
                "saved_object:map/bulk_delete",
                "saved_object:map/share_to_space",
                "ui:maps/save",
                "ui:maps/show",
                "ui:maps/saveQuery",
              ],
              "minimal_read": Array [
                "login:",
                "api:securitySolution",
                "api:lists-read",
                "api:rac",
                "api:cloud-security-posture-read",
                "api:cloud-defend-read",
                "api:securitySolution-entity-analytics",
                "api:securitySolution-threat-intelligence",
                "app:securitySolution",
                "app:csp",
                "app:cloudDefend",
                "app:kibana",
                "ui:catalogue/securitySolution",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/securitySolution",
                "ui:navLinks/csp",
                "ui:navLinks/cloudDefend",
                "ui:navLinks/kibana",
                "saved_object:exception-list/bulk_get",
                "saved_object:exception-list/get",
                "saved_object:exception-list/find",
                "saved_object:exception-list/open_point_in_time",
                "saved_object:exception-list/close_point_in_time",
                "saved_object:exception-list-agnostic/bulk_get",
                "saved_object:exception-list-agnostic/get",
                "saved_object:exception-list-agnostic/find",
                "saved_object:exception-list-agnostic/open_point_in_time",
                "saved_object:exception-list-agnostic/close_point_in_time",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:siem-ui-timeline-note/bulk_get",
                "saved_object:siem-ui-timeline-note/get",
                "saved_object:siem-ui-timeline-note/find",
                "saved_object:siem-ui-timeline-note/open_point_in_time",
                "saved_object:siem-ui-timeline-note/close_point_in_time",
                "saved_object:siem-ui-timeline-pinned-event/bulk_get",
                "saved_object:siem-ui-timeline-pinned-event/get",
                "saved_object:siem-ui-timeline-pinned-event/find",
                "saved_object:siem-ui-timeline-pinned-event/open_point_in_time",
                "saved_object:siem-ui-timeline-pinned-event/close_point_in_time",
                "saved_object:siem-detection-engine-rule-actions/bulk_get",
                "saved_object:siem-detection-engine-rule-actions/get",
                "saved_object:siem-detection-engine-rule-actions/find",
                "saved_object:siem-detection-engine-rule-actions/open_point_in_time",
                "saved_object:siem-detection-engine-rule-actions/close_point_in_time",
                "saved_object:security-rule/bulk_get",
                "saved_object:security-rule/get",
                "saved_object:security-rule/find",
                "saved_object:security-rule/open_point_in_time",
                "saved_object:security-rule/close_point_in_time",
                "saved_object:siem-ui-timeline/bulk_get",
                "saved_object:siem-ui-timeline/get",
                "saved_object:siem-ui-timeline/find",
                "saved_object:siem-ui-timeline/open_point_in_time",
                "saved_object:siem-ui-timeline/close_point_in_time",
                "saved_object:endpoint:user-artifact-manifest/bulk_get",
                "saved_object:endpoint:user-artifact-manifest/get",
                "saved_object:endpoint:user-artifact-manifest/find",
                "saved_object:endpoint:user-artifact-manifest/open_point_in_time",
                "saved_object:endpoint:user-artifact-manifest/close_point_in_time",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_get",
                "saved_object:endpoint:unified-user-artifact-manifest/get",
                "saved_object:endpoint:unified-user-artifact-manifest/find",
                "saved_object:endpoint:unified-user-artifact-manifest/open_point_in_time",
                "saved_object:endpoint:unified-user-artifact-manifest/close_point_in_time",
                "saved_object:security-solution-signals-migration/bulk_get",
                "saved_object:security-solution-signals-migration/get",
                "saved_object:security-solution-signals-migration/find",
                "saved_object:security-solution-signals-migration/open_point_in_time",
                "saved_object:security-solution-signals-migration/close_point_in_time",
                "saved_object:risk-engine-configuration/bulk_get",
                "saved_object:risk-engine-configuration/get",
                "saved_object:risk-engine-configuration/find",
                "saved_object:risk-engine-configuration/open_point_in_time",
                "saved_object:risk-engine-configuration/close_point_in_time",
                "saved_object:policy-settings-protection-updates-note/bulk_get",
                "saved_object:policy-settings-protection-updates-note/get",
                "saved_object:policy-settings-protection-updates-note/find",
                "saved_object:policy-settings-protection-updates-note/open_point_in_time",
                "saved_object:policy-settings-protection-updates-note/close_point_in_time",
                "saved_object:csp_rule/bulk_get",
                "saved_object:csp_rule/get",
                "saved_object:csp_rule/find",
                "saved_object:csp_rule/open_point_in_time",
                "saved_object:csp_rule/close_point_in_time",
                "saved_object:cloud-security-posture-settings/bulk_get",
                "saved_object:cloud-security-posture-settings/get",
                "saved_object:cloud-security-posture-settings/find",
                "saved_object:cloud-security-posture-settings/open_point_in_time",
                "saved_object:cloud-security-posture-settings/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:siem/show",
                "ui:siem/entity-analytics",
                "ui:siem/investigation-guide",
                "ui:siem/investigation-guide-interactions",
                "ui:siem/threat-intelligence",
                "alerting:siem.notifications/siem/rule/get",
                "alerting:siem.notifications/siem/rule/getRuleState",
                "alerting:siem.notifications/siem/rule/getAlertSummary",
                "alerting:siem.notifications/siem/rule/getExecutionLog",
                "alerting:siem.notifications/siem/rule/getActionErrorLog",
                "alerting:siem.notifications/siem/rule/find",
                "alerting:siem.notifications/siem/rule/getRuleExecutionKPI",
                "alerting:siem.notifications/siem/rule/getBackfill",
                "alerting:siem.notifications/siem/rule/findBackfill",
                "alerting:siem.esqlRule/siem/rule/get",
                "alerting:siem.esqlRule/siem/rule/getRuleState",
                "alerting:siem.esqlRule/siem/rule/getAlertSummary",
                "alerting:siem.esqlRule/siem/rule/getExecutionLog",
                "alerting:siem.esqlRule/siem/rule/getActionErrorLog",
                "alerting:siem.esqlRule/siem/rule/find",
                "alerting:siem.esqlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.esqlRule/siem/rule/getBackfill",
                "alerting:siem.esqlRule/siem/rule/findBackfill",
                "alerting:siem.eqlRule/siem/rule/get",
                "alerting:siem.eqlRule/siem/rule/getRuleState",
                "alerting:siem.eqlRule/siem/rule/getAlertSummary",
                "alerting:siem.eqlRule/siem/rule/getExecutionLog",
                "alerting:siem.eqlRule/siem/rule/getActionErrorLog",
                "alerting:siem.eqlRule/siem/rule/find",
                "alerting:siem.eqlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.eqlRule/siem/rule/getBackfill",
                "alerting:siem.eqlRule/siem/rule/findBackfill",
                "alerting:siem.indicatorRule/siem/rule/get",
                "alerting:siem.indicatorRule/siem/rule/getRuleState",
                "alerting:siem.indicatorRule/siem/rule/getAlertSummary",
                "alerting:siem.indicatorRule/siem/rule/getExecutionLog",
                "alerting:siem.indicatorRule/siem/rule/getActionErrorLog",
                "alerting:siem.indicatorRule/siem/rule/find",
                "alerting:siem.indicatorRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.indicatorRule/siem/rule/getBackfill",
                "alerting:siem.indicatorRule/siem/rule/findBackfill",
                "alerting:siem.mlRule/siem/rule/get",
                "alerting:siem.mlRule/siem/rule/getRuleState",
                "alerting:siem.mlRule/siem/rule/getAlertSummary",
                "alerting:siem.mlRule/siem/rule/getExecutionLog",
                "alerting:siem.mlRule/siem/rule/getActionErrorLog",
                "alerting:siem.mlRule/siem/rule/find",
                "alerting:siem.mlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.mlRule/siem/rule/getBackfill",
                "alerting:siem.mlRule/siem/rule/findBackfill",
                "alerting:siem.queryRule/siem/rule/get",
                "alerting:siem.queryRule/siem/rule/getRuleState",
                "alerting:siem.queryRule/siem/rule/getAlertSummary",
                "alerting:siem.queryRule/siem/rule/getExecutionLog",
                "alerting:siem.queryRule/siem/rule/getActionErrorLog",
                "alerting:siem.queryRule/siem/rule/find",
                "alerting:siem.queryRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.queryRule/siem/rule/getBackfill",
                "alerting:siem.queryRule/siem/rule/findBackfill",
                "alerting:siem.savedQueryRule/siem/rule/get",
                "alerting:siem.savedQueryRule/siem/rule/getRuleState",
                "alerting:siem.savedQueryRule/siem/rule/getAlertSummary",
                "alerting:siem.savedQueryRule/siem/rule/getExecutionLog",
                "alerting:siem.savedQueryRule/siem/rule/getActionErrorLog",
                "alerting:siem.savedQueryRule/siem/rule/find",
                "alerting:siem.savedQueryRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.savedQueryRule/siem/rule/getBackfill",
                "alerting:siem.savedQueryRule/siem/rule/findBackfill",
                "alerting:siem.thresholdRule/siem/rule/get",
                "alerting:siem.thresholdRule/siem/rule/getRuleState",
                "alerting:siem.thresholdRule/siem/rule/getAlertSummary",
                "alerting:siem.thresholdRule/siem/rule/getExecutionLog",
                "alerting:siem.thresholdRule/siem/rule/getActionErrorLog",
                "alerting:siem.thresholdRule/siem/rule/find",
                "alerting:siem.thresholdRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.thresholdRule/siem/rule/getBackfill",
                "alerting:siem.thresholdRule/siem/rule/findBackfill",
                "alerting:siem.newTermsRule/siem/rule/get",
                "alerting:siem.newTermsRule/siem/rule/getRuleState",
                "alerting:siem.newTermsRule/siem/rule/getAlertSummary",
                "alerting:siem.newTermsRule/siem/rule/getExecutionLog",
                "alerting:siem.newTermsRule/siem/rule/getActionErrorLog",
                "alerting:siem.newTermsRule/siem/rule/find",
                "alerting:siem.newTermsRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.newTermsRule/siem/rule/getBackfill",
                "alerting:siem.newTermsRule/siem/rule/findBackfill",
                "alerting:siem.notifications/siem/alert/get",
                "alerting:siem.notifications/siem/alert/find",
                "alerting:siem.notifications/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.notifications/siem/alert/getAlertSummary",
                "alerting:siem.notifications/siem/alert/update",
                "alerting:siem.esqlRule/siem/alert/get",
                "alerting:siem.esqlRule/siem/alert/find",
                "alerting:siem.esqlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.esqlRule/siem/alert/getAlertSummary",
                "alerting:siem.esqlRule/siem/alert/update",
                "alerting:siem.eqlRule/siem/alert/get",
                "alerting:siem.eqlRule/siem/alert/find",
                "alerting:siem.eqlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.eqlRule/siem/alert/getAlertSummary",
                "alerting:siem.eqlRule/siem/alert/update",
                "alerting:siem.indicatorRule/siem/alert/get",
                "alerting:siem.indicatorRule/siem/alert/find",
                "alerting:siem.indicatorRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.indicatorRule/siem/alert/getAlertSummary",
                "alerting:siem.indicatorRule/siem/alert/update",
                "alerting:siem.mlRule/siem/alert/get",
                "alerting:siem.mlRule/siem/alert/find",
                "alerting:siem.mlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.mlRule/siem/alert/getAlertSummary",
                "alerting:siem.mlRule/siem/alert/update",
                "alerting:siem.queryRule/siem/alert/get",
                "alerting:siem.queryRule/siem/alert/find",
                "alerting:siem.queryRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.queryRule/siem/alert/getAlertSummary",
                "alerting:siem.queryRule/siem/alert/update",
                "alerting:siem.savedQueryRule/siem/alert/get",
                "alerting:siem.savedQueryRule/siem/alert/find",
                "alerting:siem.savedQueryRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.savedQueryRule/siem/alert/getAlertSummary",
                "alerting:siem.savedQueryRule/siem/alert/update",
                "alerting:siem.thresholdRule/siem/alert/get",
                "alerting:siem.thresholdRule/siem/alert/find",
                "alerting:siem.thresholdRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.thresholdRule/siem/alert/getAlertSummary",
                "alerting:siem.thresholdRule/siem/alert/update",
                "alerting:siem.newTermsRule/siem/alert/get",
                "alerting:siem.newTermsRule/siem/alert/find",
                "alerting:siem.newTermsRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.newTermsRule/siem/alert/getAlertSummary",
                "alerting:siem.newTermsRule/siem/alert/update",
                "app:discover",
                "ui:catalogue/discover",
                "ui:navLinks/discover",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "ui:discover/show",
                "ui:discover/createShortUrl",
                "api:bulkGetUserProfiles",
                "app:dashboards",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "ui:dashboard/show",
                "ui:dashboard/createShortUrl",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "ui:visualize/show",
                "ui:visualize/createShortUrl",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "ui:maps/show",
              ],
              "policy_management_all": Array [
                "login:",
                "api:securitySolution-writePolicyManagement",
                "api:securitySolution-readPolicyManagement",
                "saved_object:policy-settings-protection-updates-note/bulk_get",
                "saved_object:policy-settings-protection-updates-note/get",
                "saved_object:policy-settings-protection-updates-note/find",
                "saved_object:policy-settings-protection-updates-note/open_point_in_time",
                "saved_object:policy-settings-protection-updates-note/close_point_in_time",
                "saved_object:policy-settings-protection-updates-note/create",
                "saved_object:policy-settings-protection-updates-note/bulk_create",
                "saved_object:policy-settings-protection-updates-note/update",
                "saved_object:policy-settings-protection-updates-note/bulk_update",
                "saved_object:policy-settings-protection-updates-note/delete",
                "saved_object:policy-settings-protection-updates-note/bulk_delete",
                "saved_object:policy-settings-protection-updates-note/share_to_space",
                "ui:siem/writePolicyManagement",
                "ui:siem/readPolicyManagement",
              ],
              "policy_management_read": Array [
                "login:",
                "api:securitySolution-readPolicyManagement",
                "saved_object:policy-settings-protection-updates-note/bulk_get",
                "saved_object:policy-settings-protection-updates-note/get",
                "saved_object:policy-settings-protection-updates-note/find",
                "saved_object:policy-settings-protection-updates-note/open_point_in_time",
                "saved_object:policy-settings-protection-updates-note/close_point_in_time",
                "ui:siem/readPolicyManagement",
              ],
              "process_operations_all": Array [
                "login:",
                "api:securitySolution-writeProcessOperations",
                "ui:siem/writeProcessOperations",
              ],
              "read": Array [
                "login:",
                "api:securitySolution",
                "api:lists-read",
                "api:rac",
                "api:cloud-security-posture-read",
                "api:cloud-defend-read",
                "api:securitySolution-entity-analytics",
                "api:securitySolution-threat-intelligence",
                "api:securitySolution-showEndpointExceptions",
                "app:securitySolution",
                "app:csp",
                "app:cloudDefend",
                "app:kibana",
                "ui:catalogue/securitySolution",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/securitySolution",
                "ui:navLinks/csp",
                "ui:navLinks/cloudDefend",
                "ui:navLinks/kibana",
                "saved_object:exception-list/bulk_get",
                "saved_object:exception-list/get",
                "saved_object:exception-list/find",
                "saved_object:exception-list/open_point_in_time",
                "saved_object:exception-list/close_point_in_time",
                "saved_object:exception-list-agnostic/bulk_get",
                "saved_object:exception-list-agnostic/get",
                "saved_object:exception-list-agnostic/find",
                "saved_object:exception-list-agnostic/open_point_in_time",
                "saved_object:exception-list-agnostic/close_point_in_time",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:siem-ui-timeline-note/bulk_get",
                "saved_object:siem-ui-timeline-note/get",
                "saved_object:siem-ui-timeline-note/find",
                "saved_object:siem-ui-timeline-note/open_point_in_time",
                "saved_object:siem-ui-timeline-note/close_point_in_time",
                "saved_object:siem-ui-timeline-pinned-event/bulk_get",
                "saved_object:siem-ui-timeline-pinned-event/get",
                "saved_object:siem-ui-timeline-pinned-event/find",
                "saved_object:siem-ui-timeline-pinned-event/open_point_in_time",
                "saved_object:siem-ui-timeline-pinned-event/close_point_in_time",
                "saved_object:siem-detection-engine-rule-actions/bulk_get",
                "saved_object:siem-detection-engine-rule-actions/get",
                "saved_object:siem-detection-engine-rule-actions/find",
                "saved_object:siem-detection-engine-rule-actions/open_point_in_time",
                "saved_object:siem-detection-engine-rule-actions/close_point_in_time",
                "saved_object:security-rule/bulk_get",
                "saved_object:security-rule/get",
                "saved_object:security-rule/find",
                "saved_object:security-rule/open_point_in_time",
                "saved_object:security-rule/close_point_in_time",
                "saved_object:siem-ui-timeline/bulk_get",
                "saved_object:siem-ui-timeline/get",
                "saved_object:siem-ui-timeline/find",
                "saved_object:siem-ui-timeline/open_point_in_time",
                "saved_object:siem-ui-timeline/close_point_in_time",
                "saved_object:endpoint:user-artifact-manifest/bulk_get",
                "saved_object:endpoint:user-artifact-manifest/get",
                "saved_object:endpoint:user-artifact-manifest/find",
                "saved_object:endpoint:user-artifact-manifest/open_point_in_time",
                "saved_object:endpoint:user-artifact-manifest/close_point_in_time",
                "saved_object:endpoint:unified-user-artifact-manifest/bulk_get",
                "saved_object:endpoint:unified-user-artifact-manifest/get",
                "saved_object:endpoint:unified-user-artifact-manifest/find",
                "saved_object:endpoint:unified-user-artifact-manifest/open_point_in_time",
                "saved_object:endpoint:unified-user-artifact-manifest/close_point_in_time",
                "saved_object:security-solution-signals-migration/bulk_get",
                "saved_object:security-solution-signals-migration/get",
                "saved_object:security-solution-signals-migration/find",
                "saved_object:security-solution-signals-migration/open_point_in_time",
                "saved_object:security-solution-signals-migration/close_point_in_time",
                "saved_object:risk-engine-configuration/bulk_get",
                "saved_object:risk-engine-configuration/get",
                "saved_object:risk-engine-configuration/find",
                "saved_object:risk-engine-configuration/open_point_in_time",
                "saved_object:risk-engine-configuration/close_point_in_time",
                "saved_object:policy-settings-protection-updates-note/bulk_get",
                "saved_object:policy-settings-protection-updates-note/get",
                "saved_object:policy-settings-protection-updates-note/find",
                "saved_object:policy-settings-protection-updates-note/open_point_in_time",
                "saved_object:policy-settings-protection-updates-note/close_point_in_time",
                "saved_object:csp_rule/bulk_get",
                "saved_object:csp_rule/get",
                "saved_object:csp_rule/find",
                "saved_object:csp_rule/open_point_in_time",
                "saved_object:csp_rule/close_point_in_time",
                "saved_object:cloud-security-posture-settings/bulk_get",
                "saved_object:cloud-security-posture-settings/get",
                "saved_object:cloud-security-posture-settings/find",
                "saved_object:cloud-security-posture-settings/open_point_in_time",
                "saved_object:cloud-security-posture-settings/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:siem/show",
                "ui:siem/entity-analytics",
                "ui:siem/investigation-guide",
                "ui:siem/investigation-guide-interactions",
                "ui:siem/threat-intelligence",
                "ui:siem/showEndpointExceptions",
                "alerting:siem.notifications/siem/rule/get",
                "alerting:siem.notifications/siem/rule/getRuleState",
                "alerting:siem.notifications/siem/rule/getAlertSummary",
                "alerting:siem.notifications/siem/rule/getExecutionLog",
                "alerting:siem.notifications/siem/rule/getActionErrorLog",
                "alerting:siem.notifications/siem/rule/find",
                "alerting:siem.notifications/siem/rule/getRuleExecutionKPI",
                "alerting:siem.notifications/siem/rule/getBackfill",
                "alerting:siem.notifications/siem/rule/findBackfill",
                "alerting:siem.esqlRule/siem/rule/get",
                "alerting:siem.esqlRule/siem/rule/getRuleState",
                "alerting:siem.esqlRule/siem/rule/getAlertSummary",
                "alerting:siem.esqlRule/siem/rule/getExecutionLog",
                "alerting:siem.esqlRule/siem/rule/getActionErrorLog",
                "alerting:siem.esqlRule/siem/rule/find",
                "alerting:siem.esqlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.esqlRule/siem/rule/getBackfill",
                "alerting:siem.esqlRule/siem/rule/findBackfill",
                "alerting:siem.eqlRule/siem/rule/get",
                "alerting:siem.eqlRule/siem/rule/getRuleState",
                "alerting:siem.eqlRule/siem/rule/getAlertSummary",
                "alerting:siem.eqlRule/siem/rule/getExecutionLog",
                "alerting:siem.eqlRule/siem/rule/getActionErrorLog",
                "alerting:siem.eqlRule/siem/rule/find",
                "alerting:siem.eqlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.eqlRule/siem/rule/getBackfill",
                "alerting:siem.eqlRule/siem/rule/findBackfill",
                "alerting:siem.indicatorRule/siem/rule/get",
                "alerting:siem.indicatorRule/siem/rule/getRuleState",
                "alerting:siem.indicatorRule/siem/rule/getAlertSummary",
                "alerting:siem.indicatorRule/siem/rule/getExecutionLog",
                "alerting:siem.indicatorRule/siem/rule/getActionErrorLog",
                "alerting:siem.indicatorRule/siem/rule/find",
                "alerting:siem.indicatorRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.indicatorRule/siem/rule/getBackfill",
                "alerting:siem.indicatorRule/siem/rule/findBackfill",
                "alerting:siem.mlRule/siem/rule/get",
                "alerting:siem.mlRule/siem/rule/getRuleState",
                "alerting:siem.mlRule/siem/rule/getAlertSummary",
                "alerting:siem.mlRule/siem/rule/getExecutionLog",
                "alerting:siem.mlRule/siem/rule/getActionErrorLog",
                "alerting:siem.mlRule/siem/rule/find",
                "alerting:siem.mlRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.mlRule/siem/rule/getBackfill",
                "alerting:siem.mlRule/siem/rule/findBackfill",
                "alerting:siem.queryRule/siem/rule/get",
                "alerting:siem.queryRule/siem/rule/getRuleState",
                "alerting:siem.queryRule/siem/rule/getAlertSummary",
                "alerting:siem.queryRule/siem/rule/getExecutionLog",
                "alerting:siem.queryRule/siem/rule/getActionErrorLog",
                "alerting:siem.queryRule/siem/rule/find",
                "alerting:siem.queryRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.queryRule/siem/rule/getBackfill",
                "alerting:siem.queryRule/siem/rule/findBackfill",
                "alerting:siem.savedQueryRule/siem/rule/get",
                "alerting:siem.savedQueryRule/siem/rule/getRuleState",
                "alerting:siem.savedQueryRule/siem/rule/getAlertSummary",
                "alerting:siem.savedQueryRule/siem/rule/getExecutionLog",
                "alerting:siem.savedQueryRule/siem/rule/getActionErrorLog",
                "alerting:siem.savedQueryRule/siem/rule/find",
                "alerting:siem.savedQueryRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.savedQueryRule/siem/rule/getBackfill",
                "alerting:siem.savedQueryRule/siem/rule/findBackfill",
                "alerting:siem.thresholdRule/siem/rule/get",
                "alerting:siem.thresholdRule/siem/rule/getRuleState",
                "alerting:siem.thresholdRule/siem/rule/getAlertSummary",
                "alerting:siem.thresholdRule/siem/rule/getExecutionLog",
                "alerting:siem.thresholdRule/siem/rule/getActionErrorLog",
                "alerting:siem.thresholdRule/siem/rule/find",
                "alerting:siem.thresholdRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.thresholdRule/siem/rule/getBackfill",
                "alerting:siem.thresholdRule/siem/rule/findBackfill",
                "alerting:siem.newTermsRule/siem/rule/get",
                "alerting:siem.newTermsRule/siem/rule/getRuleState",
                "alerting:siem.newTermsRule/siem/rule/getAlertSummary",
                "alerting:siem.newTermsRule/siem/rule/getExecutionLog",
                "alerting:siem.newTermsRule/siem/rule/getActionErrorLog",
                "alerting:siem.newTermsRule/siem/rule/find",
                "alerting:siem.newTermsRule/siem/rule/getRuleExecutionKPI",
                "alerting:siem.newTermsRule/siem/rule/getBackfill",
                "alerting:siem.newTermsRule/siem/rule/findBackfill",
                "alerting:siem.notifications/siem/alert/get",
                "alerting:siem.notifications/siem/alert/find",
                "alerting:siem.notifications/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.notifications/siem/alert/getAlertSummary",
                "alerting:siem.notifications/siem/alert/update",
                "alerting:siem.esqlRule/siem/alert/get",
                "alerting:siem.esqlRule/siem/alert/find",
                "alerting:siem.esqlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.esqlRule/siem/alert/getAlertSummary",
                "alerting:siem.esqlRule/siem/alert/update",
                "alerting:siem.eqlRule/siem/alert/get",
                "alerting:siem.eqlRule/siem/alert/find",
                "alerting:siem.eqlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.eqlRule/siem/alert/getAlertSummary",
                "alerting:siem.eqlRule/siem/alert/update",
                "alerting:siem.indicatorRule/siem/alert/get",
                "alerting:siem.indicatorRule/siem/alert/find",
                "alerting:siem.indicatorRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.indicatorRule/siem/alert/getAlertSummary",
                "alerting:siem.indicatorRule/siem/alert/update",
                "alerting:siem.mlRule/siem/alert/get",
                "alerting:siem.mlRule/siem/alert/find",
                "alerting:siem.mlRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.mlRule/siem/alert/getAlertSummary",
                "alerting:siem.mlRule/siem/alert/update",
                "alerting:siem.queryRule/siem/alert/get",
                "alerting:siem.queryRule/siem/alert/find",
                "alerting:siem.queryRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.queryRule/siem/alert/getAlertSummary",
                "alerting:siem.queryRule/siem/alert/update",
                "alerting:siem.savedQueryRule/siem/alert/get",
                "alerting:siem.savedQueryRule/siem/alert/find",
                "alerting:siem.savedQueryRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.savedQueryRule/siem/alert/getAlertSummary",
                "alerting:siem.savedQueryRule/siem/alert/update",
                "alerting:siem.thresholdRule/siem/alert/get",
                "alerting:siem.thresholdRule/siem/alert/find",
                "alerting:siem.thresholdRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.thresholdRule/siem/alert/getAlertSummary",
                "alerting:siem.thresholdRule/siem/alert/update",
                "alerting:siem.newTermsRule/siem/alert/get",
                "alerting:siem.newTermsRule/siem/alert/find",
                "alerting:siem.newTermsRule/siem/alert/getAuthorizedAlertsIndices",
                "alerting:siem.newTermsRule/siem/alert/getAlertSummary",
                "alerting:siem.newTermsRule/siem/alert/update",
                "app:discover",
                "ui:catalogue/discover",
                "ui:navLinks/discover",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "ui:discover/show",
                "ui:discover/createShortUrl",
                "api:bulkGetUserProfiles",
                "app:dashboards",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "ui:dashboard/show",
                "ui:dashboard/createShortUrl",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "ui:visualize/show",
                "ui:visualize/createShortUrl",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "ui:maps/show",
              ],
              "trusted_applications_all": Array [
                "login:",
                "api:lists-all",
                "api:lists-read",
                "api:lists-summary",
                "api:securitySolution-writeTrustedApplications",
                "api:securitySolution-readTrustedApplications",
                "saved_object:exception-list-agnostic/bulk_get",
                "saved_object:exception-list-agnostic/get",
                "saved_object:exception-list-agnostic/find",
                "saved_object:exception-list-agnostic/open_point_in_time",
                "saved_object:exception-list-agnostic/close_point_in_time",
                "saved_object:exception-list-agnostic/create",
                "saved_object:exception-list-agnostic/bulk_create",
                "saved_object:exception-list-agnostic/update",
                "saved_object:exception-list-agnostic/bulk_update",
                "saved_object:exception-list-agnostic/delete",
                "saved_object:exception-list-agnostic/bulk_delete",
                "saved_object:exception-list-agnostic/share_to_space",
                "ui:siem/writeTrustedApplications",
                "ui:siem/readTrustedApplications",
              ],
              "trusted_applications_read": Array [
                "login:",
                "api:lists-read",
                "api:lists-summary",
                "api:securitySolution-readTrustedApplications",
                "ui:siem/readTrustedApplications",
              ],
            },
          }
        `);
      });
    });
  });
}
