/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { FtrProviderContext } from '../../../ftr_provider_context';

export default function ({ getService }: FtrProviderContext) {
  const svlCommonApi = getService('svlCommonApi');

  describe('security/authorization', function () {
    describe('available features', () => {
      const svlUserManager = getService('svlUserManager');
      const supertestWithoutAuth = getService('supertestWithoutAuth');
      let adminCredentials: { Cookie: string };

      before(async () => {
        // get auth header for Viewer role
        adminCredentials = await svlUserManager.getM2MApiCredentialsWithRoleScope('admin');
      });

      it('composite features', async () => {
        const { body } = await supertestWithoutAuth
          .get('/api/security/privileges?includeActions=true')
          .set(svlCommonApi.getInternalRequestHeader())
          .set(adminCredentials)
          .expect(200);

        // The following features are composed of other features in a way that is
        // specific to the observability solution.
        const compositeFeatureIds = [
          'apm',
          'dashboard',
          'discover',
          'fleetv2',
          'infrastructure',
          'reporting',
          'slo',
          'uptime',
        ];

        const features = Object.fromEntries(
          Object.entries(body.features).filter(([key]) => compositeFeatureIds.includes(key))
        );
        expectSnapshot(features).toMatchInline(`
          Object {
            "apm": Object {
              "all": Array [
                "login:",
                "api:apm",
                "api:apm_write",
                "api:rac",
                "app:apm",
                "app:ux",
                "app:kibana",
                "ui:catalogue/apm",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/apm",
                "ui:navLinks/ux",
                "ui:navLinks/kibana",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:apm-indices/bulk_get",
                "saved_object:apm-indices/get",
                "saved_object:apm-indices/find",
                "saved_object:apm-indices/open_point_in_time",
                "saved_object:apm-indices/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:apm/show",
                "ui:apm/save",
                "ui:apm/alerting:show",
                "ui:apm/alerting:save",
                "alerting:apm.error_rate/apm/rule/get",
                "alerting:apm.error_rate/apm/rule/getRuleState",
                "alerting:apm.error_rate/apm/rule/getAlertSummary",
                "alerting:apm.error_rate/apm/rule/getExecutionLog",
                "alerting:apm.error_rate/apm/rule/getActionErrorLog",
                "alerting:apm.error_rate/apm/rule/find",
                "alerting:apm.error_rate/apm/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/apm/rule/getBackfill",
                "alerting:apm.error_rate/apm/rule/findBackfill",
                "alerting:apm.error_rate/apm/rule/create",
                "alerting:apm.error_rate/apm/rule/delete",
                "alerting:apm.error_rate/apm/rule/update",
                "alerting:apm.error_rate/apm/rule/updateApiKey",
                "alerting:apm.error_rate/apm/rule/enable",
                "alerting:apm.error_rate/apm/rule/disable",
                "alerting:apm.error_rate/apm/rule/muteAll",
                "alerting:apm.error_rate/apm/rule/unmuteAll",
                "alerting:apm.error_rate/apm/rule/muteAlert",
                "alerting:apm.error_rate/apm/rule/unmuteAlert",
                "alerting:apm.error_rate/apm/rule/snooze",
                "alerting:apm.error_rate/apm/rule/bulkEdit",
                "alerting:apm.error_rate/apm/rule/bulkDelete",
                "alerting:apm.error_rate/apm/rule/bulkEnable",
                "alerting:apm.error_rate/apm/rule/bulkDisable",
                "alerting:apm.error_rate/apm/rule/unsnooze",
                "alerting:apm.error_rate/apm/rule/runSoon",
                "alerting:apm.error_rate/apm/rule/scheduleBackfill",
                "alerting:apm.error_rate/apm/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/get",
                "alerting:apm.transaction_error_rate/apm/rule/getRuleState",
                "alerting:apm.transaction_error_rate/apm/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/apm/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/apm/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/apm/rule/find",
                "alerting:apm.transaction_error_rate/apm/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/apm/rule/getBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/findBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/create",
                "alerting:apm.transaction_error_rate/apm/rule/delete",
                "alerting:apm.transaction_error_rate/apm/rule/update",
                "alerting:apm.transaction_error_rate/apm/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/apm/rule/enable",
                "alerting:apm.transaction_error_rate/apm/rule/disable",
                "alerting:apm.transaction_error_rate/apm/rule/muteAll",
                "alerting:apm.transaction_error_rate/apm/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/apm/rule/muteAlert",
                "alerting:apm.transaction_error_rate/apm/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/apm/rule/snooze",
                "alerting:apm.transaction_error_rate/apm/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/apm/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/apm/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/apm/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/apm/rule/unsnooze",
                "alerting:apm.transaction_error_rate/apm/rule/runSoon",
                "alerting:apm.transaction_error_rate/apm/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/deleteBackfill",
                "alerting:apm.transaction_duration/apm/rule/get",
                "alerting:apm.transaction_duration/apm/rule/getRuleState",
                "alerting:apm.transaction_duration/apm/rule/getAlertSummary",
                "alerting:apm.transaction_duration/apm/rule/getExecutionLog",
                "alerting:apm.transaction_duration/apm/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/apm/rule/find",
                "alerting:apm.transaction_duration/apm/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/apm/rule/getBackfill",
                "alerting:apm.transaction_duration/apm/rule/findBackfill",
                "alerting:apm.transaction_duration/apm/rule/create",
                "alerting:apm.transaction_duration/apm/rule/delete",
                "alerting:apm.transaction_duration/apm/rule/update",
                "alerting:apm.transaction_duration/apm/rule/updateApiKey",
                "alerting:apm.transaction_duration/apm/rule/enable",
                "alerting:apm.transaction_duration/apm/rule/disable",
                "alerting:apm.transaction_duration/apm/rule/muteAll",
                "alerting:apm.transaction_duration/apm/rule/unmuteAll",
                "alerting:apm.transaction_duration/apm/rule/muteAlert",
                "alerting:apm.transaction_duration/apm/rule/unmuteAlert",
                "alerting:apm.transaction_duration/apm/rule/snooze",
                "alerting:apm.transaction_duration/apm/rule/bulkEdit",
                "alerting:apm.transaction_duration/apm/rule/bulkDelete",
                "alerting:apm.transaction_duration/apm/rule/bulkEnable",
                "alerting:apm.transaction_duration/apm/rule/bulkDisable",
                "alerting:apm.transaction_duration/apm/rule/unsnooze",
                "alerting:apm.transaction_duration/apm/rule/runSoon",
                "alerting:apm.transaction_duration/apm/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/apm/rule/deleteBackfill",
                "alerting:apm.anomaly/apm/rule/get",
                "alerting:apm.anomaly/apm/rule/getRuleState",
                "alerting:apm.anomaly/apm/rule/getAlertSummary",
                "alerting:apm.anomaly/apm/rule/getExecutionLog",
                "alerting:apm.anomaly/apm/rule/getActionErrorLog",
                "alerting:apm.anomaly/apm/rule/find",
                "alerting:apm.anomaly/apm/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/apm/rule/getBackfill",
                "alerting:apm.anomaly/apm/rule/findBackfill",
                "alerting:apm.anomaly/apm/rule/create",
                "alerting:apm.anomaly/apm/rule/delete",
                "alerting:apm.anomaly/apm/rule/update",
                "alerting:apm.anomaly/apm/rule/updateApiKey",
                "alerting:apm.anomaly/apm/rule/enable",
                "alerting:apm.anomaly/apm/rule/disable",
                "alerting:apm.anomaly/apm/rule/muteAll",
                "alerting:apm.anomaly/apm/rule/unmuteAll",
                "alerting:apm.anomaly/apm/rule/muteAlert",
                "alerting:apm.anomaly/apm/rule/unmuteAlert",
                "alerting:apm.anomaly/apm/rule/snooze",
                "alerting:apm.anomaly/apm/rule/bulkEdit",
                "alerting:apm.anomaly/apm/rule/bulkDelete",
                "alerting:apm.anomaly/apm/rule/bulkEnable",
                "alerting:apm.anomaly/apm/rule/bulkDisable",
                "alerting:apm.anomaly/apm/rule/unsnooze",
                "alerting:apm.anomaly/apm/rule/runSoon",
                "alerting:apm.anomaly/apm/rule/scheduleBackfill",
                "alerting:apm.anomaly/apm/rule/deleteBackfill",
                "alerting:apm.error_rate/apm/alert/get",
                "alerting:apm.error_rate/apm/alert/find",
                "alerting:apm.error_rate/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/apm/alert/getAlertSummary",
                "alerting:apm.error_rate/apm/alert/update",
                "alerting:apm.transaction_error_rate/apm/alert/get",
                "alerting:apm.transaction_error_rate/apm/alert/find",
                "alerting:apm.transaction_error_rate/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/apm/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/apm/alert/update",
                "alerting:apm.transaction_duration/apm/alert/get",
                "alerting:apm.transaction_duration/apm/alert/find",
                "alerting:apm.transaction_duration/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/apm/alert/getAlertSummary",
                "alerting:apm.transaction_duration/apm/alert/update",
                "alerting:apm.anomaly/apm/alert/get",
                "alerting:apm.anomaly/apm/alert/find",
                "alerting:apm.anomaly/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/apm/alert/getAlertSummary",
                "alerting:apm.anomaly/apm/alert/update",
                "api:infra",
                "app:infra",
                "app:logs",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:navLinks/infra",
                "ui:navLinks/logs",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-ui-source/create",
                "saved_object:infrastructure-ui-source/bulk_create",
                "saved_object:infrastructure-ui-source/update",
                "saved_object:infrastructure-ui-source/bulk_update",
                "saved_object:infrastructure-ui-source/delete",
                "saved_object:infrastructure-ui-source/bulk_delete",
                "saved_object:infrastructure-ui-source/share_to_space",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/create",
                "saved_object:infrastructure-monitoring-log-view/bulk_create",
                "saved_object:infrastructure-monitoring-log-view/update",
                "saved_object:infrastructure-monitoring-log-view/bulk_update",
                "saved_object:infrastructure-monitoring-log-view/delete",
                "saved_object:infrastructure-monitoring-log-view/bulk_delete",
                "saved_object:infrastructure-monitoring-log-view/share_to_space",
                "ui:logs/show",
                "ui:logs/configureSource",
                "ui:logs/save",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/rule/create",
                "alerting:logs.alert.document.count/logs/rule/delete",
                "alerting:logs.alert.document.count/logs/rule/update",
                "alerting:logs.alert.document.count/logs/rule/updateApiKey",
                "alerting:logs.alert.document.count/logs/rule/enable",
                "alerting:logs.alert.document.count/logs/rule/disable",
                "alerting:logs.alert.document.count/logs/rule/muteAll",
                "alerting:logs.alert.document.count/logs/rule/unmuteAll",
                "alerting:logs.alert.document.count/logs/rule/muteAlert",
                "alerting:logs.alert.document.count/logs/rule/unmuteAlert",
                "alerting:logs.alert.document.count/logs/rule/snooze",
                "alerting:logs.alert.document.count/logs/rule/bulkEdit",
                "alerting:logs.alert.document.count/logs/rule/bulkDelete",
                "alerting:logs.alert.document.count/logs/rule/bulkEnable",
                "alerting:logs.alert.document.count/logs/rule/bulkDisable",
                "alerting:logs.alert.document.count/logs/rule/unsnooze",
                "alerting:logs.alert.document.count/logs/rule/runSoon",
                "alerting:logs.alert.document.count/logs/rule/scheduleBackfill",
                "alerting:logs.alert.document.count/logs/rule/deleteBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/create",
                "alerting:.es-query/logs/rule/delete",
                "alerting:.es-query/logs/rule/update",
                "alerting:.es-query/logs/rule/updateApiKey",
                "alerting:.es-query/logs/rule/enable",
                "alerting:.es-query/logs/rule/disable",
                "alerting:.es-query/logs/rule/muteAll",
                "alerting:.es-query/logs/rule/unmuteAll",
                "alerting:.es-query/logs/rule/muteAlert",
                "alerting:.es-query/logs/rule/unmuteAlert",
                "alerting:.es-query/logs/rule/snooze",
                "alerting:.es-query/logs/rule/bulkEdit",
                "alerting:.es-query/logs/rule/bulkDelete",
                "alerting:.es-query/logs/rule/bulkEnable",
                "alerting:.es-query/logs/rule/bulkDisable",
                "alerting:.es-query/logs/rule/unsnooze",
                "alerting:.es-query/logs/rule/runSoon",
                "alerting:.es-query/logs/rule/scheduleBackfill",
                "alerting:.es-query/logs/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/create",
                "alerting:observability.rules.custom_threshold/logs/rule/delete",
                "alerting:observability.rules.custom_threshold/logs/rule/update",
                "alerting:observability.rules.custom_threshold/logs/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/logs/rule/enable",
                "alerting:observability.rules.custom_threshold/logs/rule/disable",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/snooze",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/logs/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/logs/rule/runSoon",
                "alerting:observability.rules.custom_threshold/logs/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/deleteBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:logs.alert.document.count/logs/alert/update",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/update",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/update",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "minimal_all": Array [
                "login:",
                "api:apm",
                "api:apm_write",
                "api:rac",
                "app:apm",
                "app:ux",
                "app:kibana",
                "ui:catalogue/apm",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/apm",
                "ui:navLinks/ux",
                "ui:navLinks/kibana",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:apm-indices/bulk_get",
                "saved_object:apm-indices/get",
                "saved_object:apm-indices/find",
                "saved_object:apm-indices/open_point_in_time",
                "saved_object:apm-indices/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:apm/show",
                "ui:apm/save",
                "ui:apm/alerting:show",
                "ui:apm/alerting:save",
                "alerting:apm.error_rate/apm/rule/get",
                "alerting:apm.error_rate/apm/rule/getRuleState",
                "alerting:apm.error_rate/apm/rule/getAlertSummary",
                "alerting:apm.error_rate/apm/rule/getExecutionLog",
                "alerting:apm.error_rate/apm/rule/getActionErrorLog",
                "alerting:apm.error_rate/apm/rule/find",
                "alerting:apm.error_rate/apm/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/apm/rule/getBackfill",
                "alerting:apm.error_rate/apm/rule/findBackfill",
                "alerting:apm.error_rate/apm/rule/create",
                "alerting:apm.error_rate/apm/rule/delete",
                "alerting:apm.error_rate/apm/rule/update",
                "alerting:apm.error_rate/apm/rule/updateApiKey",
                "alerting:apm.error_rate/apm/rule/enable",
                "alerting:apm.error_rate/apm/rule/disable",
                "alerting:apm.error_rate/apm/rule/muteAll",
                "alerting:apm.error_rate/apm/rule/unmuteAll",
                "alerting:apm.error_rate/apm/rule/muteAlert",
                "alerting:apm.error_rate/apm/rule/unmuteAlert",
                "alerting:apm.error_rate/apm/rule/snooze",
                "alerting:apm.error_rate/apm/rule/bulkEdit",
                "alerting:apm.error_rate/apm/rule/bulkDelete",
                "alerting:apm.error_rate/apm/rule/bulkEnable",
                "alerting:apm.error_rate/apm/rule/bulkDisable",
                "alerting:apm.error_rate/apm/rule/unsnooze",
                "alerting:apm.error_rate/apm/rule/runSoon",
                "alerting:apm.error_rate/apm/rule/scheduleBackfill",
                "alerting:apm.error_rate/apm/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/get",
                "alerting:apm.transaction_error_rate/apm/rule/getRuleState",
                "alerting:apm.transaction_error_rate/apm/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/apm/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/apm/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/apm/rule/find",
                "alerting:apm.transaction_error_rate/apm/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/apm/rule/getBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/findBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/create",
                "alerting:apm.transaction_error_rate/apm/rule/delete",
                "alerting:apm.transaction_error_rate/apm/rule/update",
                "alerting:apm.transaction_error_rate/apm/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/apm/rule/enable",
                "alerting:apm.transaction_error_rate/apm/rule/disable",
                "alerting:apm.transaction_error_rate/apm/rule/muteAll",
                "alerting:apm.transaction_error_rate/apm/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/apm/rule/muteAlert",
                "alerting:apm.transaction_error_rate/apm/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/apm/rule/snooze",
                "alerting:apm.transaction_error_rate/apm/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/apm/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/apm/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/apm/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/apm/rule/unsnooze",
                "alerting:apm.transaction_error_rate/apm/rule/runSoon",
                "alerting:apm.transaction_error_rate/apm/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/deleteBackfill",
                "alerting:apm.transaction_duration/apm/rule/get",
                "alerting:apm.transaction_duration/apm/rule/getRuleState",
                "alerting:apm.transaction_duration/apm/rule/getAlertSummary",
                "alerting:apm.transaction_duration/apm/rule/getExecutionLog",
                "alerting:apm.transaction_duration/apm/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/apm/rule/find",
                "alerting:apm.transaction_duration/apm/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/apm/rule/getBackfill",
                "alerting:apm.transaction_duration/apm/rule/findBackfill",
                "alerting:apm.transaction_duration/apm/rule/create",
                "alerting:apm.transaction_duration/apm/rule/delete",
                "alerting:apm.transaction_duration/apm/rule/update",
                "alerting:apm.transaction_duration/apm/rule/updateApiKey",
                "alerting:apm.transaction_duration/apm/rule/enable",
                "alerting:apm.transaction_duration/apm/rule/disable",
                "alerting:apm.transaction_duration/apm/rule/muteAll",
                "alerting:apm.transaction_duration/apm/rule/unmuteAll",
                "alerting:apm.transaction_duration/apm/rule/muteAlert",
                "alerting:apm.transaction_duration/apm/rule/unmuteAlert",
                "alerting:apm.transaction_duration/apm/rule/snooze",
                "alerting:apm.transaction_duration/apm/rule/bulkEdit",
                "alerting:apm.transaction_duration/apm/rule/bulkDelete",
                "alerting:apm.transaction_duration/apm/rule/bulkEnable",
                "alerting:apm.transaction_duration/apm/rule/bulkDisable",
                "alerting:apm.transaction_duration/apm/rule/unsnooze",
                "alerting:apm.transaction_duration/apm/rule/runSoon",
                "alerting:apm.transaction_duration/apm/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/apm/rule/deleteBackfill",
                "alerting:apm.anomaly/apm/rule/get",
                "alerting:apm.anomaly/apm/rule/getRuleState",
                "alerting:apm.anomaly/apm/rule/getAlertSummary",
                "alerting:apm.anomaly/apm/rule/getExecutionLog",
                "alerting:apm.anomaly/apm/rule/getActionErrorLog",
                "alerting:apm.anomaly/apm/rule/find",
                "alerting:apm.anomaly/apm/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/apm/rule/getBackfill",
                "alerting:apm.anomaly/apm/rule/findBackfill",
                "alerting:apm.anomaly/apm/rule/create",
                "alerting:apm.anomaly/apm/rule/delete",
                "alerting:apm.anomaly/apm/rule/update",
                "alerting:apm.anomaly/apm/rule/updateApiKey",
                "alerting:apm.anomaly/apm/rule/enable",
                "alerting:apm.anomaly/apm/rule/disable",
                "alerting:apm.anomaly/apm/rule/muteAll",
                "alerting:apm.anomaly/apm/rule/unmuteAll",
                "alerting:apm.anomaly/apm/rule/muteAlert",
                "alerting:apm.anomaly/apm/rule/unmuteAlert",
                "alerting:apm.anomaly/apm/rule/snooze",
                "alerting:apm.anomaly/apm/rule/bulkEdit",
                "alerting:apm.anomaly/apm/rule/bulkDelete",
                "alerting:apm.anomaly/apm/rule/bulkEnable",
                "alerting:apm.anomaly/apm/rule/bulkDisable",
                "alerting:apm.anomaly/apm/rule/unsnooze",
                "alerting:apm.anomaly/apm/rule/runSoon",
                "alerting:apm.anomaly/apm/rule/scheduleBackfill",
                "alerting:apm.anomaly/apm/rule/deleteBackfill",
                "alerting:apm.error_rate/apm/alert/get",
                "alerting:apm.error_rate/apm/alert/find",
                "alerting:apm.error_rate/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/apm/alert/getAlertSummary",
                "alerting:apm.error_rate/apm/alert/update",
                "alerting:apm.transaction_error_rate/apm/alert/get",
                "alerting:apm.transaction_error_rate/apm/alert/find",
                "alerting:apm.transaction_error_rate/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/apm/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/apm/alert/update",
                "alerting:apm.transaction_duration/apm/alert/get",
                "alerting:apm.transaction_duration/apm/alert/find",
                "alerting:apm.transaction_duration/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/apm/alert/getAlertSummary",
                "alerting:apm.transaction_duration/apm/alert/update",
                "alerting:apm.anomaly/apm/alert/get",
                "alerting:apm.anomaly/apm/alert/find",
                "alerting:apm.anomaly/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/apm/alert/getAlertSummary",
                "alerting:apm.anomaly/apm/alert/update",
                "api:infra",
                "app:infra",
                "app:logs",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:navLinks/infra",
                "ui:navLinks/logs",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-ui-source/create",
                "saved_object:infrastructure-ui-source/bulk_create",
                "saved_object:infrastructure-ui-source/update",
                "saved_object:infrastructure-ui-source/bulk_update",
                "saved_object:infrastructure-ui-source/delete",
                "saved_object:infrastructure-ui-source/bulk_delete",
                "saved_object:infrastructure-ui-source/share_to_space",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/create",
                "saved_object:infrastructure-monitoring-log-view/bulk_create",
                "saved_object:infrastructure-monitoring-log-view/update",
                "saved_object:infrastructure-monitoring-log-view/bulk_update",
                "saved_object:infrastructure-monitoring-log-view/delete",
                "saved_object:infrastructure-monitoring-log-view/bulk_delete",
                "saved_object:infrastructure-monitoring-log-view/share_to_space",
                "ui:logs/show",
                "ui:logs/configureSource",
                "ui:logs/save",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/rule/create",
                "alerting:logs.alert.document.count/logs/rule/delete",
                "alerting:logs.alert.document.count/logs/rule/update",
                "alerting:logs.alert.document.count/logs/rule/updateApiKey",
                "alerting:logs.alert.document.count/logs/rule/enable",
                "alerting:logs.alert.document.count/logs/rule/disable",
                "alerting:logs.alert.document.count/logs/rule/muteAll",
                "alerting:logs.alert.document.count/logs/rule/unmuteAll",
                "alerting:logs.alert.document.count/logs/rule/muteAlert",
                "alerting:logs.alert.document.count/logs/rule/unmuteAlert",
                "alerting:logs.alert.document.count/logs/rule/snooze",
                "alerting:logs.alert.document.count/logs/rule/bulkEdit",
                "alerting:logs.alert.document.count/logs/rule/bulkDelete",
                "alerting:logs.alert.document.count/logs/rule/bulkEnable",
                "alerting:logs.alert.document.count/logs/rule/bulkDisable",
                "alerting:logs.alert.document.count/logs/rule/unsnooze",
                "alerting:logs.alert.document.count/logs/rule/runSoon",
                "alerting:logs.alert.document.count/logs/rule/scheduleBackfill",
                "alerting:logs.alert.document.count/logs/rule/deleteBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/create",
                "alerting:.es-query/logs/rule/delete",
                "alerting:.es-query/logs/rule/update",
                "alerting:.es-query/logs/rule/updateApiKey",
                "alerting:.es-query/logs/rule/enable",
                "alerting:.es-query/logs/rule/disable",
                "alerting:.es-query/logs/rule/muteAll",
                "alerting:.es-query/logs/rule/unmuteAll",
                "alerting:.es-query/logs/rule/muteAlert",
                "alerting:.es-query/logs/rule/unmuteAlert",
                "alerting:.es-query/logs/rule/snooze",
                "alerting:.es-query/logs/rule/bulkEdit",
                "alerting:.es-query/logs/rule/bulkDelete",
                "alerting:.es-query/logs/rule/bulkEnable",
                "alerting:.es-query/logs/rule/bulkDisable",
                "alerting:.es-query/logs/rule/unsnooze",
                "alerting:.es-query/logs/rule/runSoon",
                "alerting:.es-query/logs/rule/scheduleBackfill",
                "alerting:.es-query/logs/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/create",
                "alerting:observability.rules.custom_threshold/logs/rule/delete",
                "alerting:observability.rules.custom_threshold/logs/rule/update",
                "alerting:observability.rules.custom_threshold/logs/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/logs/rule/enable",
                "alerting:observability.rules.custom_threshold/logs/rule/disable",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/snooze",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/logs/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/logs/rule/runSoon",
                "alerting:observability.rules.custom_threshold/logs/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/deleteBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:logs.alert.document.count/logs/alert/update",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/update",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/update",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "minimal_read": Array [
                "login:",
                "api:apm",
                "api:rac",
                "app:apm",
                "app:ux",
                "app:kibana",
                "ui:catalogue/apm",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/apm",
                "ui:navLinks/ux",
                "ui:navLinks/kibana",
                "saved_object:apm-indices/bulk_get",
                "saved_object:apm-indices/get",
                "saved_object:apm-indices/find",
                "saved_object:apm-indices/open_point_in_time",
                "saved_object:apm-indices/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:apm/show",
                "ui:apm/alerting:show",
                "alerting:apm.error_rate/apm/rule/get",
                "alerting:apm.error_rate/apm/rule/getRuleState",
                "alerting:apm.error_rate/apm/rule/getAlertSummary",
                "alerting:apm.error_rate/apm/rule/getExecutionLog",
                "alerting:apm.error_rate/apm/rule/getActionErrorLog",
                "alerting:apm.error_rate/apm/rule/find",
                "alerting:apm.error_rate/apm/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/apm/rule/getBackfill",
                "alerting:apm.error_rate/apm/rule/findBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/get",
                "alerting:apm.transaction_error_rate/apm/rule/getRuleState",
                "alerting:apm.transaction_error_rate/apm/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/apm/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/apm/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/apm/rule/find",
                "alerting:apm.transaction_error_rate/apm/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/apm/rule/getBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/findBackfill",
                "alerting:apm.transaction_duration/apm/rule/get",
                "alerting:apm.transaction_duration/apm/rule/getRuleState",
                "alerting:apm.transaction_duration/apm/rule/getAlertSummary",
                "alerting:apm.transaction_duration/apm/rule/getExecutionLog",
                "alerting:apm.transaction_duration/apm/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/apm/rule/find",
                "alerting:apm.transaction_duration/apm/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/apm/rule/getBackfill",
                "alerting:apm.transaction_duration/apm/rule/findBackfill",
                "alerting:apm.anomaly/apm/rule/get",
                "alerting:apm.anomaly/apm/rule/getRuleState",
                "alerting:apm.anomaly/apm/rule/getAlertSummary",
                "alerting:apm.anomaly/apm/rule/getExecutionLog",
                "alerting:apm.anomaly/apm/rule/getActionErrorLog",
                "alerting:apm.anomaly/apm/rule/find",
                "alerting:apm.anomaly/apm/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/apm/rule/getBackfill",
                "alerting:apm.anomaly/apm/rule/findBackfill",
                "alerting:apm.error_rate/apm/alert/get",
                "alerting:apm.error_rate/apm/alert/find",
                "alerting:apm.error_rate/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/apm/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/apm/alert/get",
                "alerting:apm.transaction_error_rate/apm/alert/find",
                "alerting:apm.transaction_error_rate/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/apm/alert/getAlertSummary",
                "alerting:apm.transaction_duration/apm/alert/get",
                "alerting:apm.transaction_duration/apm/alert/find",
                "alerting:apm.transaction_duration/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/apm/alert/getAlertSummary",
                "alerting:apm.anomaly/apm/alert/get",
                "alerting:apm.anomaly/apm/alert/find",
                "alerting:apm.anomaly/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/apm/alert/getAlertSummary",
                "api:infra",
                "app:infra",
                "app:logs",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:navLinks/infra",
                "ui:navLinks/logs",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "ui:logs/show",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
              "read": Array [
                "login:",
                "api:apm",
                "api:rac",
                "app:apm",
                "app:ux",
                "app:kibana",
                "ui:catalogue/apm",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/apm",
                "ui:navLinks/ux",
                "ui:navLinks/kibana",
                "saved_object:apm-indices/bulk_get",
                "saved_object:apm-indices/get",
                "saved_object:apm-indices/find",
                "saved_object:apm-indices/open_point_in_time",
                "saved_object:apm-indices/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:apm/show",
                "ui:apm/alerting:show",
                "alerting:apm.error_rate/apm/rule/get",
                "alerting:apm.error_rate/apm/rule/getRuleState",
                "alerting:apm.error_rate/apm/rule/getAlertSummary",
                "alerting:apm.error_rate/apm/rule/getExecutionLog",
                "alerting:apm.error_rate/apm/rule/getActionErrorLog",
                "alerting:apm.error_rate/apm/rule/find",
                "alerting:apm.error_rate/apm/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/apm/rule/getBackfill",
                "alerting:apm.error_rate/apm/rule/findBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/get",
                "alerting:apm.transaction_error_rate/apm/rule/getRuleState",
                "alerting:apm.transaction_error_rate/apm/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/apm/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/apm/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/apm/rule/find",
                "alerting:apm.transaction_error_rate/apm/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/apm/rule/getBackfill",
                "alerting:apm.transaction_error_rate/apm/rule/findBackfill",
                "alerting:apm.transaction_duration/apm/rule/get",
                "alerting:apm.transaction_duration/apm/rule/getRuleState",
                "alerting:apm.transaction_duration/apm/rule/getAlertSummary",
                "alerting:apm.transaction_duration/apm/rule/getExecutionLog",
                "alerting:apm.transaction_duration/apm/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/apm/rule/find",
                "alerting:apm.transaction_duration/apm/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/apm/rule/getBackfill",
                "alerting:apm.transaction_duration/apm/rule/findBackfill",
                "alerting:apm.anomaly/apm/rule/get",
                "alerting:apm.anomaly/apm/rule/getRuleState",
                "alerting:apm.anomaly/apm/rule/getAlertSummary",
                "alerting:apm.anomaly/apm/rule/getExecutionLog",
                "alerting:apm.anomaly/apm/rule/getActionErrorLog",
                "alerting:apm.anomaly/apm/rule/find",
                "alerting:apm.anomaly/apm/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/apm/rule/getBackfill",
                "alerting:apm.anomaly/apm/rule/findBackfill",
                "alerting:apm.error_rate/apm/alert/get",
                "alerting:apm.error_rate/apm/alert/find",
                "alerting:apm.error_rate/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/apm/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/apm/alert/get",
                "alerting:apm.transaction_error_rate/apm/alert/find",
                "alerting:apm.transaction_error_rate/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/apm/alert/getAlertSummary",
                "alerting:apm.transaction_duration/apm/alert/get",
                "alerting:apm.transaction_duration/apm/alert/find",
                "alerting:apm.transaction_duration/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/apm/alert/getAlertSummary",
                "alerting:apm.anomaly/apm/alert/get",
                "alerting:apm.anomaly/apm/alert/find",
                "alerting:apm.anomaly/apm/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/apm/alert/getAlertSummary",
                "api:infra",
                "app:infra",
                "app:logs",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:navLinks/infra",
                "ui:navLinks/logs",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "ui:logs/show",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
            },
            "dashboard": Object {
              "all": Array [
                "login:",
                "api:bulkGetUserProfiles",
                "api:store_search_session",
                "api:generateReport",
                "api:downloadCsv",
                "app:dashboards",
                "app:kibana",
                "ui:catalogue/dashboard",
                "ui:management/kibana/search_sessions",
                "ui:management/insightsAndAlerting/reporting",
                "ui:navLinks/dashboards",
                "ui:navLinks/kibana",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:dashboard/create",
                "saved_object:dashboard/bulk_create",
                "saved_object:dashboard/update",
                "saved_object:dashboard/bulk_update",
                "saved_object:dashboard/delete",
                "saved_object:dashboard/bulk_delete",
                "saved_object:dashboard/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "ui:dashboard/createNew",
                "ui:dashboard/show",
                "ui:dashboard/showWriteControls",
                "ui:dashboard/saveQuery",
                "ui:dashboard/createShortUrl",
                "ui:dashboard/storeSearchSession",
                "ui:dashboard/generateScreenshot",
                "ui:dashboard/downloadCsv",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "saved_object:map/create",
                "saved_object:map/bulk_create",
                "saved_object:map/update",
                "saved_object:map/bulk_update",
                "saved_object:map/delete",
                "saved_object:map/bulk_delete",
                "saved_object:map/share_to_space",
                "ui:maps/save",
                "ui:maps/show",
                "ui:maps/saveQuery",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "saved_object:visualization/create",
                "saved_object:visualization/bulk_create",
                "saved_object:visualization/update",
                "saved_object:visualization/bulk_update",
                "saved_object:visualization/delete",
                "saved_object:visualization/bulk_delete",
                "saved_object:visualization/share_to_space",
                "saved_object:lens/create",
                "saved_object:lens/bulk_create",
                "saved_object:lens/update",
                "saved_object:lens/bulk_update",
                "saved_object:lens/delete",
                "saved_object:lens/bulk_delete",
                "saved_object:lens/share_to_space",
                "ui:visualize/show",
                "ui:visualize/delete",
                "ui:visualize/save",
                "ui:visualize/saveQuery",
                "ui:visualize/createShortUrl",
                "ui:visualize/generateScreenshot",
              ],
              "download_csv_report": Array [
                "login:",
                "api:downloadCsv",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/downloadCsv",
              ],
              "generate_report": Array [
                "login:",
                "api:generateReport",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/generateScreenshot",
              ],
              "minimal_all": Array [
                "login:",
                "api:bulkGetUserProfiles",
                "app:dashboards",
                "app:kibana",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "ui:navLinks/kibana",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:dashboard/create",
                "saved_object:dashboard/bulk_create",
                "saved_object:dashboard/update",
                "saved_object:dashboard/bulk_update",
                "saved_object:dashboard/delete",
                "saved_object:dashboard/bulk_delete",
                "saved_object:dashboard/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:dashboard/createNew",
                "ui:dashboard/show",
                "ui:dashboard/showWriteControls",
                "ui:dashboard/saveQuery",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "saved_object:map/create",
                "saved_object:map/bulk_create",
                "saved_object:map/update",
                "saved_object:map/bulk_update",
                "saved_object:map/delete",
                "saved_object:map/bulk_delete",
                "saved_object:map/share_to_space",
                "ui:maps/save",
                "ui:maps/show",
                "ui:maps/saveQuery",
                "api:generateReport",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:management/insightsAndAlerting/reporting",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "saved_object:visualization/create",
                "saved_object:visualization/bulk_create",
                "saved_object:visualization/update",
                "saved_object:visualization/bulk_update",
                "saved_object:visualization/delete",
                "saved_object:visualization/bulk_delete",
                "saved_object:visualization/share_to_space",
                "saved_object:lens/create",
                "saved_object:lens/bulk_create",
                "saved_object:lens/update",
                "saved_object:lens/bulk_update",
                "saved_object:lens/delete",
                "saved_object:lens/bulk_delete",
                "saved_object:lens/share_to_space",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "ui:visualize/show",
                "ui:visualize/delete",
                "ui:visualize/save",
                "ui:visualize/saveQuery",
                "ui:visualize/createShortUrl",
                "ui:visualize/generateScreenshot",
              ],
              "minimal_read": Array [
                "login:",
                "api:bulkGetUserProfiles",
                "app:dashboards",
                "app:kibana",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "ui:navLinks/kibana",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:dashboard/show",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "ui:maps/show",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "ui:visualize/show",
                "ui:visualize/createShortUrl",
              ],
              "read": Array [
                "login:",
                "api:bulkGetUserProfiles",
                "app:dashboards",
                "app:kibana",
                "ui:catalogue/dashboard",
                "ui:navLinks/dashboards",
                "ui:navLinks/kibana",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:visualization/bulk_get",
                "saved_object:visualization/get",
                "saved_object:visualization/find",
                "saved_object:visualization/open_point_in_time",
                "saved_object:visualization/close_point_in_time",
                "saved_object:canvas-workpad/bulk_get",
                "saved_object:canvas-workpad/get",
                "saved_object:canvas-workpad/find",
                "saved_object:canvas-workpad/open_point_in_time",
                "saved_object:canvas-workpad/close_point_in_time",
                "saved_object:lens/bulk_get",
                "saved_object:lens/get",
                "saved_object:lens/find",
                "saved_object:lens/open_point_in_time",
                "saved_object:lens/close_point_in_time",
                "saved_object:links/bulk_get",
                "saved_object:links/get",
                "saved_object:links/find",
                "saved_object:links/open_point_in_time",
                "saved_object:links/close_point_in_time",
                "saved_object:map/bulk_get",
                "saved_object:map/get",
                "saved_object:map/find",
                "saved_object:map/open_point_in_time",
                "saved_object:map/close_point_in_time",
                "saved_object:dashboard/bulk_get",
                "saved_object:dashboard/get",
                "saved_object:dashboard/find",
                "saved_object:dashboard/open_point_in_time",
                "saved_object:dashboard/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:tag/bulk_get",
                "saved_object:tag/get",
                "saved_object:tag/find",
                "saved_object:tag/open_point_in_time",
                "saved_object:tag/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "ui:dashboard/show",
                "ui:dashboard/createShortUrl",
                "app:maps",
                "ui:catalogue/maps",
                "ui:navLinks/maps",
                "ui:maps/show",
                "app:visualize",
                "app:lens",
                "ui:catalogue/visualize",
                "ui:navLinks/visualize",
                "ui:navLinks/lens",
                "ui:visualize/show",
                "ui:visualize/createShortUrl",
              ],
              "store_search_session": Array [
                "login:",
                "api:store_search_session",
                "ui:management/kibana/search_sessions",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "ui:dashboard/storeSearchSession",
              ],
              "url_create": Array [
                "login:",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "ui:dashboard/createShortUrl",
              ],
            },
            "discover": Object {
              "all": Array [
                "login:",
                "api:fileUpload:analyzeFile",
                "api:store_search_session",
                "api:generateReport",
                "app:discover",
                "app:kibana",
                "ui:catalogue/discover",
                "ui:management/kibana/search_sessions",
                "ui:management/insightsAndAlerting/reporting",
                "ui:navLinks/discover",
                "ui:navLinks/kibana",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:search/create",
                "saved_object:search/bulk_create",
                "saved_object:search/update",
                "saved_object:search/bulk_update",
                "saved_object:search/delete",
                "saved_object:search/bulk_delete",
                "saved_object:search/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "ui:discover/show",
                "ui:discover/save",
                "ui:discover/saveQuery",
                "ui:discover/createShortUrl",
                "ui:discover/storeSearchSession",
                "ui:discover/generateCsv",
                "api:rac",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "generate_report": Array [
                "login:",
                "api:generateReport",
                "ui:management/insightsAndAlerting/reporting",
                "ui:discover/generateCsv",
              ],
              "minimal_all": Array [
                "login:",
                "api:fileUpload:analyzeFile",
                "app:discover",
                "app:kibana",
                "ui:catalogue/discover",
                "ui:navLinks/discover",
                "ui:navLinks/kibana",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:search/create",
                "saved_object:search/bulk_create",
                "saved_object:search/update",
                "saved_object:search/bulk_update",
                "saved_object:search/delete",
                "saved_object:search/bulk_delete",
                "saved_object:search/share_to_space",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:query/create",
                "saved_object:query/bulk_create",
                "saved_object:query/update",
                "saved_object:query/bulk_update",
                "saved_object:query/delete",
                "saved_object:query/bulk_delete",
                "saved_object:query/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:discover/show",
                "ui:discover/save",
                "ui:discover/saveQuery",
                "api:rac",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "minimal_read": Array [
                "login:",
                "app:discover",
                "app:kibana",
                "ui:catalogue/discover",
                "ui:navLinks/discover",
                "ui:navLinks/kibana",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:discover/show",
                "api:rac",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
              "read": Array [
                "login:",
                "app:discover",
                "app:kibana",
                "ui:catalogue/discover",
                "ui:navLinks/discover",
                "ui:navLinks/kibana",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:search/bulk_get",
                "saved_object:search/get",
                "saved_object:search/find",
                "saved_object:search/open_point_in_time",
                "saved_object:search/close_point_in_time",
                "saved_object:query/bulk_get",
                "saved_object:query/get",
                "saved_object:query/find",
                "saved_object:query/open_point_in_time",
                "saved_object:query/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "ui:discover/show",
                "ui:discover/createShortUrl",
                "api:rac",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
              "store_search_session": Array [
                "login:",
                "api:store_search_session",
                "ui:management/kibana/search_sessions",
                "saved_object:search-session/bulk_get",
                "saved_object:search-session/get",
                "saved_object:search-session/find",
                "saved_object:search-session/open_point_in_time",
                "saved_object:search-session/close_point_in_time",
                "saved_object:search-session/create",
                "saved_object:search-session/bulk_create",
                "saved_object:search-session/update",
                "saved_object:search-session/bulk_update",
                "saved_object:search-session/delete",
                "saved_object:search-session/bulk_delete",
                "saved_object:search-session/share_to_space",
                "ui:discover/storeSearchSession",
              ],
              "url_create": Array [
                "login:",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "saved_object:url/create",
                "saved_object:url/bulk_create",
                "saved_object:url/update",
                "saved_object:url/bulk_update",
                "saved_object:url/delete",
                "saved_object:url/bulk_delete",
                "saved_object:url/share_to_space",
                "ui:discover/createShortUrl",
              ],
            },
            "fleetv2": Object {
              "all": Array [
                "login:",
                "api:fleet-read",
                "api:fleet-all",
                "app:fleet",
                "ui:catalogue/fleet",
                "ui:navLinks/fleet",
                "saved_object:ingest-outputs/bulk_get",
                "saved_object:ingest-outputs/get",
                "saved_object:ingest-outputs/find",
                "saved_object:ingest-outputs/open_point_in_time",
                "saved_object:ingest-outputs/close_point_in_time",
                "saved_object:ingest-outputs/create",
                "saved_object:ingest-outputs/bulk_create",
                "saved_object:ingest-outputs/update",
                "saved_object:ingest-outputs/bulk_update",
                "saved_object:ingest-outputs/delete",
                "saved_object:ingest-outputs/bulk_delete",
                "saved_object:ingest-outputs/share_to_space",
                "saved_object:ingest-agent-policies/bulk_get",
                "saved_object:ingest-agent-policies/get",
                "saved_object:ingest-agent-policies/find",
                "saved_object:ingest-agent-policies/open_point_in_time",
                "saved_object:ingest-agent-policies/close_point_in_time",
                "saved_object:ingest-agent-policies/create",
                "saved_object:ingest-agent-policies/bulk_create",
                "saved_object:ingest-agent-policies/update",
                "saved_object:ingest-agent-policies/bulk_update",
                "saved_object:ingest-agent-policies/delete",
                "saved_object:ingest-agent-policies/bulk_delete",
                "saved_object:ingest-agent-policies/share_to_space",
                "saved_object:ingest-package-policies/bulk_get",
                "saved_object:ingest-package-policies/get",
                "saved_object:ingest-package-policies/find",
                "saved_object:ingest-package-policies/open_point_in_time",
                "saved_object:ingest-package-policies/close_point_in_time",
                "saved_object:ingest-package-policies/create",
                "saved_object:ingest-package-policies/bulk_create",
                "saved_object:ingest-package-policies/update",
                "saved_object:ingest-package-policies/bulk_update",
                "saved_object:ingest-package-policies/delete",
                "saved_object:ingest-package-policies/bulk_delete",
                "saved_object:ingest-package-policies/share_to_space",
                "saved_object:epm-packages/bulk_get",
                "saved_object:epm-packages/get",
                "saved_object:epm-packages/find",
                "saved_object:epm-packages/open_point_in_time",
                "saved_object:epm-packages/close_point_in_time",
                "saved_object:epm-packages/create",
                "saved_object:epm-packages/bulk_create",
                "saved_object:epm-packages/update",
                "saved_object:epm-packages/bulk_update",
                "saved_object:epm-packages/delete",
                "saved_object:epm-packages/bulk_delete",
                "saved_object:epm-packages/share_to_space",
                "saved_object:epm-packages-assets/bulk_get",
                "saved_object:epm-packages-assets/get",
                "saved_object:epm-packages-assets/find",
                "saved_object:epm-packages-assets/open_point_in_time",
                "saved_object:epm-packages-assets/close_point_in_time",
                "saved_object:epm-packages-assets/create",
                "saved_object:epm-packages-assets/bulk_create",
                "saved_object:epm-packages-assets/update",
                "saved_object:epm-packages-assets/bulk_update",
                "saved_object:epm-packages-assets/delete",
                "saved_object:epm-packages-assets/bulk_delete",
                "saved_object:epm-packages-assets/share_to_space",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_get",
                "saved_object:fleet-preconfiguration-deletion-record/get",
                "saved_object:fleet-preconfiguration-deletion-record/find",
                "saved_object:fleet-preconfiguration-deletion-record/open_point_in_time",
                "saved_object:fleet-preconfiguration-deletion-record/close_point_in_time",
                "saved_object:fleet-preconfiguration-deletion-record/create",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_create",
                "saved_object:fleet-preconfiguration-deletion-record/update",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_update",
                "saved_object:fleet-preconfiguration-deletion-record/delete",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_delete",
                "saved_object:fleet-preconfiguration-deletion-record/share_to_space",
                "saved_object:ingest-download-sources/bulk_get",
                "saved_object:ingest-download-sources/get",
                "saved_object:ingest-download-sources/find",
                "saved_object:ingest-download-sources/open_point_in_time",
                "saved_object:ingest-download-sources/close_point_in_time",
                "saved_object:ingest-download-sources/create",
                "saved_object:ingest-download-sources/bulk_create",
                "saved_object:ingest-download-sources/update",
                "saved_object:ingest-download-sources/bulk_update",
                "saved_object:ingest-download-sources/delete",
                "saved_object:ingest-download-sources/bulk_delete",
                "saved_object:ingest-download-sources/share_to_space",
                "saved_object:fleet-fleet-server-host/bulk_get",
                "saved_object:fleet-fleet-server-host/get",
                "saved_object:fleet-fleet-server-host/find",
                "saved_object:fleet-fleet-server-host/open_point_in_time",
                "saved_object:fleet-fleet-server-host/close_point_in_time",
                "saved_object:fleet-fleet-server-host/create",
                "saved_object:fleet-fleet-server-host/bulk_create",
                "saved_object:fleet-fleet-server-host/update",
                "saved_object:fleet-fleet-server-host/bulk_update",
                "saved_object:fleet-fleet-server-host/delete",
                "saved_object:fleet-fleet-server-host/bulk_delete",
                "saved_object:fleet-fleet-server-host/share_to_space",
                "saved_object:fleet-proxy/bulk_get",
                "saved_object:fleet-proxy/get",
                "saved_object:fleet-proxy/find",
                "saved_object:fleet-proxy/open_point_in_time",
                "saved_object:fleet-proxy/close_point_in_time",
                "saved_object:fleet-proxy/create",
                "saved_object:fleet-proxy/bulk_create",
                "saved_object:fleet-proxy/update",
                "saved_object:fleet-proxy/bulk_update",
                "saved_object:fleet-proxy/delete",
                "saved_object:fleet-proxy/bulk_delete",
                "saved_object:fleet-proxy/share_to_space",
                "saved_object:fleet-space-settings/bulk_get",
                "saved_object:fleet-space-settings/get",
                "saved_object:fleet-space-settings/find",
                "saved_object:fleet-space-settings/open_point_in_time",
                "saved_object:fleet-space-settings/close_point_in_time",
                "saved_object:fleet-space-settings/create",
                "saved_object:fleet-space-settings/bulk_create",
                "saved_object:fleet-space-settings/update",
                "saved_object:fleet-space-settings/bulk_update",
                "saved_object:fleet-space-settings/delete",
                "saved_object:fleet-space-settings/bulk_delete",
                "saved_object:fleet-space-settings/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:fleetv2/read",
                "ui:fleetv2/all",
                "api:infra",
                "api:rac",
                "app:infra",
                "app:logs",
                "app:kibana",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/infra",
                "ui:navLinks/logs",
                "ui:navLinks/kibana",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-ui-source/create",
                "saved_object:infrastructure-ui-source/bulk_create",
                "saved_object:infrastructure-ui-source/update",
                "saved_object:infrastructure-ui-source/bulk_update",
                "saved_object:infrastructure-ui-source/delete",
                "saved_object:infrastructure-ui-source/bulk_delete",
                "saved_object:infrastructure-ui-source/share_to_space",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/create",
                "saved_object:infrastructure-monitoring-log-view/bulk_create",
                "saved_object:infrastructure-monitoring-log-view/update",
                "saved_object:infrastructure-monitoring-log-view/bulk_update",
                "saved_object:infrastructure-monitoring-log-view/delete",
                "saved_object:infrastructure-monitoring-log-view/bulk_delete",
                "saved_object:infrastructure-monitoring-log-view/share_to_space",
                "ui:logs/show",
                "ui:logs/configureSource",
                "ui:logs/save",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/rule/create",
                "alerting:logs.alert.document.count/logs/rule/delete",
                "alerting:logs.alert.document.count/logs/rule/update",
                "alerting:logs.alert.document.count/logs/rule/updateApiKey",
                "alerting:logs.alert.document.count/logs/rule/enable",
                "alerting:logs.alert.document.count/logs/rule/disable",
                "alerting:logs.alert.document.count/logs/rule/muteAll",
                "alerting:logs.alert.document.count/logs/rule/unmuteAll",
                "alerting:logs.alert.document.count/logs/rule/muteAlert",
                "alerting:logs.alert.document.count/logs/rule/unmuteAlert",
                "alerting:logs.alert.document.count/logs/rule/snooze",
                "alerting:logs.alert.document.count/logs/rule/bulkEdit",
                "alerting:logs.alert.document.count/logs/rule/bulkDelete",
                "alerting:logs.alert.document.count/logs/rule/bulkEnable",
                "alerting:logs.alert.document.count/logs/rule/bulkDisable",
                "alerting:logs.alert.document.count/logs/rule/unsnooze",
                "alerting:logs.alert.document.count/logs/rule/runSoon",
                "alerting:logs.alert.document.count/logs/rule/scheduleBackfill",
                "alerting:logs.alert.document.count/logs/rule/deleteBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/create",
                "alerting:.es-query/logs/rule/delete",
                "alerting:.es-query/logs/rule/update",
                "alerting:.es-query/logs/rule/updateApiKey",
                "alerting:.es-query/logs/rule/enable",
                "alerting:.es-query/logs/rule/disable",
                "alerting:.es-query/logs/rule/muteAll",
                "alerting:.es-query/logs/rule/unmuteAll",
                "alerting:.es-query/logs/rule/muteAlert",
                "alerting:.es-query/logs/rule/unmuteAlert",
                "alerting:.es-query/logs/rule/snooze",
                "alerting:.es-query/logs/rule/bulkEdit",
                "alerting:.es-query/logs/rule/bulkDelete",
                "alerting:.es-query/logs/rule/bulkEnable",
                "alerting:.es-query/logs/rule/bulkDisable",
                "alerting:.es-query/logs/rule/unsnooze",
                "alerting:.es-query/logs/rule/runSoon",
                "alerting:.es-query/logs/rule/scheduleBackfill",
                "alerting:.es-query/logs/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/create",
                "alerting:observability.rules.custom_threshold/logs/rule/delete",
                "alerting:observability.rules.custom_threshold/logs/rule/update",
                "alerting:observability.rules.custom_threshold/logs/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/logs/rule/enable",
                "alerting:observability.rules.custom_threshold/logs/rule/disable",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/snooze",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/logs/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/logs/rule/runSoon",
                "alerting:observability.rules.custom_threshold/logs/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/deleteBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:logs.alert.document.count/logs/alert/update",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/update",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/update",
              ],
              "minimal_all": Array [
                "login:",
                "api:fleet-read",
                "api:fleet-all",
                "app:fleet",
                "ui:catalogue/fleet",
                "ui:navLinks/fleet",
                "saved_object:ingest-outputs/bulk_get",
                "saved_object:ingest-outputs/get",
                "saved_object:ingest-outputs/find",
                "saved_object:ingest-outputs/open_point_in_time",
                "saved_object:ingest-outputs/close_point_in_time",
                "saved_object:ingest-outputs/create",
                "saved_object:ingest-outputs/bulk_create",
                "saved_object:ingest-outputs/update",
                "saved_object:ingest-outputs/bulk_update",
                "saved_object:ingest-outputs/delete",
                "saved_object:ingest-outputs/bulk_delete",
                "saved_object:ingest-outputs/share_to_space",
                "saved_object:ingest-agent-policies/bulk_get",
                "saved_object:ingest-agent-policies/get",
                "saved_object:ingest-agent-policies/find",
                "saved_object:ingest-agent-policies/open_point_in_time",
                "saved_object:ingest-agent-policies/close_point_in_time",
                "saved_object:ingest-agent-policies/create",
                "saved_object:ingest-agent-policies/bulk_create",
                "saved_object:ingest-agent-policies/update",
                "saved_object:ingest-agent-policies/bulk_update",
                "saved_object:ingest-agent-policies/delete",
                "saved_object:ingest-agent-policies/bulk_delete",
                "saved_object:ingest-agent-policies/share_to_space",
                "saved_object:ingest-package-policies/bulk_get",
                "saved_object:ingest-package-policies/get",
                "saved_object:ingest-package-policies/find",
                "saved_object:ingest-package-policies/open_point_in_time",
                "saved_object:ingest-package-policies/close_point_in_time",
                "saved_object:ingest-package-policies/create",
                "saved_object:ingest-package-policies/bulk_create",
                "saved_object:ingest-package-policies/update",
                "saved_object:ingest-package-policies/bulk_update",
                "saved_object:ingest-package-policies/delete",
                "saved_object:ingest-package-policies/bulk_delete",
                "saved_object:ingest-package-policies/share_to_space",
                "saved_object:epm-packages/bulk_get",
                "saved_object:epm-packages/get",
                "saved_object:epm-packages/find",
                "saved_object:epm-packages/open_point_in_time",
                "saved_object:epm-packages/close_point_in_time",
                "saved_object:epm-packages/create",
                "saved_object:epm-packages/bulk_create",
                "saved_object:epm-packages/update",
                "saved_object:epm-packages/bulk_update",
                "saved_object:epm-packages/delete",
                "saved_object:epm-packages/bulk_delete",
                "saved_object:epm-packages/share_to_space",
                "saved_object:epm-packages-assets/bulk_get",
                "saved_object:epm-packages-assets/get",
                "saved_object:epm-packages-assets/find",
                "saved_object:epm-packages-assets/open_point_in_time",
                "saved_object:epm-packages-assets/close_point_in_time",
                "saved_object:epm-packages-assets/create",
                "saved_object:epm-packages-assets/bulk_create",
                "saved_object:epm-packages-assets/update",
                "saved_object:epm-packages-assets/bulk_update",
                "saved_object:epm-packages-assets/delete",
                "saved_object:epm-packages-assets/bulk_delete",
                "saved_object:epm-packages-assets/share_to_space",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_get",
                "saved_object:fleet-preconfiguration-deletion-record/get",
                "saved_object:fleet-preconfiguration-deletion-record/find",
                "saved_object:fleet-preconfiguration-deletion-record/open_point_in_time",
                "saved_object:fleet-preconfiguration-deletion-record/close_point_in_time",
                "saved_object:fleet-preconfiguration-deletion-record/create",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_create",
                "saved_object:fleet-preconfiguration-deletion-record/update",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_update",
                "saved_object:fleet-preconfiguration-deletion-record/delete",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_delete",
                "saved_object:fleet-preconfiguration-deletion-record/share_to_space",
                "saved_object:ingest-download-sources/bulk_get",
                "saved_object:ingest-download-sources/get",
                "saved_object:ingest-download-sources/find",
                "saved_object:ingest-download-sources/open_point_in_time",
                "saved_object:ingest-download-sources/close_point_in_time",
                "saved_object:ingest-download-sources/create",
                "saved_object:ingest-download-sources/bulk_create",
                "saved_object:ingest-download-sources/update",
                "saved_object:ingest-download-sources/bulk_update",
                "saved_object:ingest-download-sources/delete",
                "saved_object:ingest-download-sources/bulk_delete",
                "saved_object:ingest-download-sources/share_to_space",
                "saved_object:fleet-fleet-server-host/bulk_get",
                "saved_object:fleet-fleet-server-host/get",
                "saved_object:fleet-fleet-server-host/find",
                "saved_object:fleet-fleet-server-host/open_point_in_time",
                "saved_object:fleet-fleet-server-host/close_point_in_time",
                "saved_object:fleet-fleet-server-host/create",
                "saved_object:fleet-fleet-server-host/bulk_create",
                "saved_object:fleet-fleet-server-host/update",
                "saved_object:fleet-fleet-server-host/bulk_update",
                "saved_object:fleet-fleet-server-host/delete",
                "saved_object:fleet-fleet-server-host/bulk_delete",
                "saved_object:fleet-fleet-server-host/share_to_space",
                "saved_object:fleet-proxy/bulk_get",
                "saved_object:fleet-proxy/get",
                "saved_object:fleet-proxy/find",
                "saved_object:fleet-proxy/open_point_in_time",
                "saved_object:fleet-proxy/close_point_in_time",
                "saved_object:fleet-proxy/create",
                "saved_object:fleet-proxy/bulk_create",
                "saved_object:fleet-proxy/update",
                "saved_object:fleet-proxy/bulk_update",
                "saved_object:fleet-proxy/delete",
                "saved_object:fleet-proxy/bulk_delete",
                "saved_object:fleet-proxy/share_to_space",
                "saved_object:fleet-space-settings/bulk_get",
                "saved_object:fleet-space-settings/get",
                "saved_object:fleet-space-settings/find",
                "saved_object:fleet-space-settings/open_point_in_time",
                "saved_object:fleet-space-settings/close_point_in_time",
                "saved_object:fleet-space-settings/create",
                "saved_object:fleet-space-settings/bulk_create",
                "saved_object:fleet-space-settings/update",
                "saved_object:fleet-space-settings/bulk_update",
                "saved_object:fleet-space-settings/delete",
                "saved_object:fleet-space-settings/bulk_delete",
                "saved_object:fleet-space-settings/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:fleetv2/read",
                "ui:fleetv2/all",
                "api:infra",
                "api:rac",
                "app:infra",
                "app:logs",
                "app:kibana",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/infra",
                "ui:navLinks/logs",
                "ui:navLinks/kibana",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-ui-source/create",
                "saved_object:infrastructure-ui-source/bulk_create",
                "saved_object:infrastructure-ui-source/update",
                "saved_object:infrastructure-ui-source/bulk_update",
                "saved_object:infrastructure-ui-source/delete",
                "saved_object:infrastructure-ui-source/bulk_delete",
                "saved_object:infrastructure-ui-source/share_to_space",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/create",
                "saved_object:infrastructure-monitoring-log-view/bulk_create",
                "saved_object:infrastructure-monitoring-log-view/update",
                "saved_object:infrastructure-monitoring-log-view/bulk_update",
                "saved_object:infrastructure-monitoring-log-view/delete",
                "saved_object:infrastructure-monitoring-log-view/bulk_delete",
                "saved_object:infrastructure-monitoring-log-view/share_to_space",
                "ui:logs/show",
                "ui:logs/configureSource",
                "ui:logs/save",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/rule/create",
                "alerting:logs.alert.document.count/logs/rule/delete",
                "alerting:logs.alert.document.count/logs/rule/update",
                "alerting:logs.alert.document.count/logs/rule/updateApiKey",
                "alerting:logs.alert.document.count/logs/rule/enable",
                "alerting:logs.alert.document.count/logs/rule/disable",
                "alerting:logs.alert.document.count/logs/rule/muteAll",
                "alerting:logs.alert.document.count/logs/rule/unmuteAll",
                "alerting:logs.alert.document.count/logs/rule/muteAlert",
                "alerting:logs.alert.document.count/logs/rule/unmuteAlert",
                "alerting:logs.alert.document.count/logs/rule/snooze",
                "alerting:logs.alert.document.count/logs/rule/bulkEdit",
                "alerting:logs.alert.document.count/logs/rule/bulkDelete",
                "alerting:logs.alert.document.count/logs/rule/bulkEnable",
                "alerting:logs.alert.document.count/logs/rule/bulkDisable",
                "alerting:logs.alert.document.count/logs/rule/unsnooze",
                "alerting:logs.alert.document.count/logs/rule/runSoon",
                "alerting:logs.alert.document.count/logs/rule/scheduleBackfill",
                "alerting:logs.alert.document.count/logs/rule/deleteBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/create",
                "alerting:.es-query/logs/rule/delete",
                "alerting:.es-query/logs/rule/update",
                "alerting:.es-query/logs/rule/updateApiKey",
                "alerting:.es-query/logs/rule/enable",
                "alerting:.es-query/logs/rule/disable",
                "alerting:.es-query/logs/rule/muteAll",
                "alerting:.es-query/logs/rule/unmuteAll",
                "alerting:.es-query/logs/rule/muteAlert",
                "alerting:.es-query/logs/rule/unmuteAlert",
                "alerting:.es-query/logs/rule/snooze",
                "alerting:.es-query/logs/rule/bulkEdit",
                "alerting:.es-query/logs/rule/bulkDelete",
                "alerting:.es-query/logs/rule/bulkEnable",
                "alerting:.es-query/logs/rule/bulkDisable",
                "alerting:.es-query/logs/rule/unsnooze",
                "alerting:.es-query/logs/rule/runSoon",
                "alerting:.es-query/logs/rule/scheduleBackfill",
                "alerting:.es-query/logs/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/create",
                "alerting:observability.rules.custom_threshold/logs/rule/delete",
                "alerting:observability.rules.custom_threshold/logs/rule/update",
                "alerting:observability.rules.custom_threshold/logs/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/logs/rule/enable",
                "alerting:observability.rules.custom_threshold/logs/rule/disable",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/snooze",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/logs/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/logs/rule/runSoon",
                "alerting:observability.rules.custom_threshold/logs/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/deleteBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:logs.alert.document.count/logs/alert/update",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/update",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/update",
              ],
              "minimal_read": Array [
                "login:",
                "api:fleet-read",
                "app:fleet",
                "ui:catalogue/fleet",
                "ui:navLinks/fleet",
                "saved_object:ingest-outputs/bulk_get",
                "saved_object:ingest-outputs/get",
                "saved_object:ingest-outputs/find",
                "saved_object:ingest-outputs/open_point_in_time",
                "saved_object:ingest-outputs/close_point_in_time",
                "saved_object:ingest-agent-policies/bulk_get",
                "saved_object:ingest-agent-policies/get",
                "saved_object:ingest-agent-policies/find",
                "saved_object:ingest-agent-policies/open_point_in_time",
                "saved_object:ingest-agent-policies/close_point_in_time",
                "saved_object:ingest-package-policies/bulk_get",
                "saved_object:ingest-package-policies/get",
                "saved_object:ingest-package-policies/find",
                "saved_object:ingest-package-policies/open_point_in_time",
                "saved_object:ingest-package-policies/close_point_in_time",
                "saved_object:epm-packages/bulk_get",
                "saved_object:epm-packages/get",
                "saved_object:epm-packages/find",
                "saved_object:epm-packages/open_point_in_time",
                "saved_object:epm-packages/close_point_in_time",
                "saved_object:epm-packages-assets/bulk_get",
                "saved_object:epm-packages-assets/get",
                "saved_object:epm-packages-assets/find",
                "saved_object:epm-packages-assets/open_point_in_time",
                "saved_object:epm-packages-assets/close_point_in_time",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_get",
                "saved_object:fleet-preconfiguration-deletion-record/get",
                "saved_object:fleet-preconfiguration-deletion-record/find",
                "saved_object:fleet-preconfiguration-deletion-record/open_point_in_time",
                "saved_object:fleet-preconfiguration-deletion-record/close_point_in_time",
                "saved_object:ingest-download-sources/bulk_get",
                "saved_object:ingest-download-sources/get",
                "saved_object:ingest-download-sources/find",
                "saved_object:ingest-download-sources/open_point_in_time",
                "saved_object:ingest-download-sources/close_point_in_time",
                "saved_object:fleet-fleet-server-host/bulk_get",
                "saved_object:fleet-fleet-server-host/get",
                "saved_object:fleet-fleet-server-host/find",
                "saved_object:fleet-fleet-server-host/open_point_in_time",
                "saved_object:fleet-fleet-server-host/close_point_in_time",
                "saved_object:fleet-proxy/bulk_get",
                "saved_object:fleet-proxy/get",
                "saved_object:fleet-proxy/find",
                "saved_object:fleet-proxy/open_point_in_time",
                "saved_object:fleet-proxy/close_point_in_time",
                "saved_object:fleet-space-settings/bulk_get",
                "saved_object:fleet-space-settings/get",
                "saved_object:fleet-space-settings/find",
                "saved_object:fleet-space-settings/open_point_in_time",
                "saved_object:fleet-space-settings/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:fleetv2/read",
                "api:infra",
                "api:rac",
                "app:infra",
                "app:logs",
                "app:kibana",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/infra",
                "ui:navLinks/logs",
                "ui:navLinks/kibana",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "ui:logs/show",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
              ],
              "read": Array [
                "login:",
                "api:fleet-read",
                "app:fleet",
                "ui:catalogue/fleet",
                "ui:navLinks/fleet",
                "saved_object:ingest-outputs/bulk_get",
                "saved_object:ingest-outputs/get",
                "saved_object:ingest-outputs/find",
                "saved_object:ingest-outputs/open_point_in_time",
                "saved_object:ingest-outputs/close_point_in_time",
                "saved_object:ingest-agent-policies/bulk_get",
                "saved_object:ingest-agent-policies/get",
                "saved_object:ingest-agent-policies/find",
                "saved_object:ingest-agent-policies/open_point_in_time",
                "saved_object:ingest-agent-policies/close_point_in_time",
                "saved_object:ingest-package-policies/bulk_get",
                "saved_object:ingest-package-policies/get",
                "saved_object:ingest-package-policies/find",
                "saved_object:ingest-package-policies/open_point_in_time",
                "saved_object:ingest-package-policies/close_point_in_time",
                "saved_object:epm-packages/bulk_get",
                "saved_object:epm-packages/get",
                "saved_object:epm-packages/find",
                "saved_object:epm-packages/open_point_in_time",
                "saved_object:epm-packages/close_point_in_time",
                "saved_object:epm-packages-assets/bulk_get",
                "saved_object:epm-packages-assets/get",
                "saved_object:epm-packages-assets/find",
                "saved_object:epm-packages-assets/open_point_in_time",
                "saved_object:epm-packages-assets/close_point_in_time",
                "saved_object:fleet-preconfiguration-deletion-record/bulk_get",
                "saved_object:fleet-preconfiguration-deletion-record/get",
                "saved_object:fleet-preconfiguration-deletion-record/find",
                "saved_object:fleet-preconfiguration-deletion-record/open_point_in_time",
                "saved_object:fleet-preconfiguration-deletion-record/close_point_in_time",
                "saved_object:ingest-download-sources/bulk_get",
                "saved_object:ingest-download-sources/get",
                "saved_object:ingest-download-sources/find",
                "saved_object:ingest-download-sources/open_point_in_time",
                "saved_object:ingest-download-sources/close_point_in_time",
                "saved_object:fleet-fleet-server-host/bulk_get",
                "saved_object:fleet-fleet-server-host/get",
                "saved_object:fleet-fleet-server-host/find",
                "saved_object:fleet-fleet-server-host/open_point_in_time",
                "saved_object:fleet-fleet-server-host/close_point_in_time",
                "saved_object:fleet-proxy/bulk_get",
                "saved_object:fleet-proxy/get",
                "saved_object:fleet-proxy/find",
                "saved_object:fleet-proxy/open_point_in_time",
                "saved_object:fleet-proxy/close_point_in_time",
                "saved_object:fleet-space-settings/bulk_get",
                "saved_object:fleet-space-settings/get",
                "saved_object:fleet-space-settings/find",
                "saved_object:fleet-space-settings/open_point_in_time",
                "saved_object:fleet-space-settings/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:fleetv2/read",
                "api:infra",
                "api:rac",
                "app:infra",
                "app:logs",
                "app:kibana",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/infra",
                "ui:navLinks/logs",
                "ui:navLinks/kibana",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "ui:logs/show",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
              ],
            },
            "infrastructure": Object {
              "all": Array [
                "login:",
                "api:infra",
                "api:rac",
                "app:infra",
                "app:metrics",
                "app:kibana",
                "ui:catalogue/infraops",
                "ui:catalogue/metrics",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/infra",
                "ui:navLinks/metrics",
                "ui:navLinks/kibana",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-ui-source/create",
                "saved_object:infrastructure-ui-source/bulk_create",
                "saved_object:infrastructure-ui-source/update",
                "saved_object:infrastructure-ui-source/bulk_update",
                "saved_object:infrastructure-ui-source/delete",
                "saved_object:infrastructure-ui-source/bulk_delete",
                "saved_object:infrastructure-ui-source/share_to_space",
                "saved_object:metrics-data-source/bulk_get",
                "saved_object:metrics-data-source/get",
                "saved_object:metrics-data-source/find",
                "saved_object:metrics-data-source/open_point_in_time",
                "saved_object:metrics-data-source/close_point_in_time",
                "saved_object:metrics-data-source/create",
                "saved_object:metrics-data-source/bulk_create",
                "saved_object:metrics-data-source/update",
                "saved_object:metrics-data-source/bulk_update",
                "saved_object:metrics-data-source/delete",
                "saved_object:metrics-data-source/bulk_delete",
                "saved_object:metrics-data-source/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:infrastructure/show",
                "ui:infrastructure/configureSource",
                "ui:infrastructure/save",
                "alerting:metrics.alert.threshold/infrastructure/rule/get",
                "alerting:metrics.alert.threshold/infrastructure/rule/getRuleState",
                "alerting:metrics.alert.threshold/infrastructure/rule/getAlertSummary",
                "alerting:metrics.alert.threshold/infrastructure/rule/getExecutionLog",
                "alerting:metrics.alert.threshold/infrastructure/rule/getActionErrorLog",
                "alerting:metrics.alert.threshold/infrastructure/rule/find",
                "alerting:metrics.alert.threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.threshold/infrastructure/rule/getBackfill",
                "alerting:metrics.alert.threshold/infrastructure/rule/findBackfill",
                "alerting:metrics.alert.threshold/infrastructure/rule/create",
                "alerting:metrics.alert.threshold/infrastructure/rule/delete",
                "alerting:metrics.alert.threshold/infrastructure/rule/update",
                "alerting:metrics.alert.threshold/infrastructure/rule/updateApiKey",
                "alerting:metrics.alert.threshold/infrastructure/rule/enable",
                "alerting:metrics.alert.threshold/infrastructure/rule/disable",
                "alerting:metrics.alert.threshold/infrastructure/rule/muteAll",
                "alerting:metrics.alert.threshold/infrastructure/rule/unmuteAll",
                "alerting:metrics.alert.threshold/infrastructure/rule/muteAlert",
                "alerting:metrics.alert.threshold/infrastructure/rule/unmuteAlert",
                "alerting:metrics.alert.threshold/infrastructure/rule/snooze",
                "alerting:metrics.alert.threshold/infrastructure/rule/bulkEdit",
                "alerting:metrics.alert.threshold/infrastructure/rule/bulkDelete",
                "alerting:metrics.alert.threshold/infrastructure/rule/bulkEnable",
                "alerting:metrics.alert.threshold/infrastructure/rule/bulkDisable",
                "alerting:metrics.alert.threshold/infrastructure/rule/unsnooze",
                "alerting:metrics.alert.threshold/infrastructure/rule/runSoon",
                "alerting:metrics.alert.threshold/infrastructure/rule/scheduleBackfill",
                "alerting:metrics.alert.threshold/infrastructure/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/get",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/find",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/create",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/delete",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/update",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/enable",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/disable",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/deleteBackfill",
                "alerting:.es-query/infrastructure/rule/get",
                "alerting:.es-query/infrastructure/rule/getRuleState",
                "alerting:.es-query/infrastructure/rule/getAlertSummary",
                "alerting:.es-query/infrastructure/rule/getExecutionLog",
                "alerting:.es-query/infrastructure/rule/getActionErrorLog",
                "alerting:.es-query/infrastructure/rule/find",
                "alerting:.es-query/infrastructure/rule/getRuleExecutionKPI",
                "alerting:.es-query/infrastructure/rule/getBackfill",
                "alerting:.es-query/infrastructure/rule/findBackfill",
                "alerting:.es-query/infrastructure/rule/create",
                "alerting:.es-query/infrastructure/rule/delete",
                "alerting:.es-query/infrastructure/rule/update",
                "alerting:.es-query/infrastructure/rule/updateApiKey",
                "alerting:.es-query/infrastructure/rule/enable",
                "alerting:.es-query/infrastructure/rule/disable",
                "alerting:.es-query/infrastructure/rule/muteAll",
                "alerting:.es-query/infrastructure/rule/unmuteAll",
                "alerting:.es-query/infrastructure/rule/muteAlert",
                "alerting:.es-query/infrastructure/rule/unmuteAlert",
                "alerting:.es-query/infrastructure/rule/snooze",
                "alerting:.es-query/infrastructure/rule/bulkEdit",
                "alerting:.es-query/infrastructure/rule/bulkDelete",
                "alerting:.es-query/infrastructure/rule/bulkEnable",
                "alerting:.es-query/infrastructure/rule/bulkDisable",
                "alerting:.es-query/infrastructure/rule/unsnooze",
                "alerting:.es-query/infrastructure/rule/runSoon",
                "alerting:.es-query/infrastructure/rule/scheduleBackfill",
                "alerting:.es-query/infrastructure/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/get",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/find",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/create",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/delete",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/update",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/enable",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/disable",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/muteAll",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/snooze",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/runSoon",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/deleteBackfill",
                "alerting:metrics.alert.threshold/infrastructure/alert/get",
                "alerting:metrics.alert.threshold/infrastructure/alert/find",
                "alerting:metrics.alert.threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.threshold/infrastructure/alert/getAlertSummary",
                "alerting:metrics.alert.threshold/infrastructure/alert/update",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/get",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/find",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/update",
                "alerting:.es-query/infrastructure/alert/get",
                "alerting:.es-query/infrastructure/alert/find",
                "alerting:.es-query/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/infrastructure/alert/getAlertSummary",
                "alerting:.es-query/infrastructure/alert/update",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/get",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/find",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/update",
                "app:logs",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:navLinks/logs",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/create",
                "saved_object:infrastructure-monitoring-log-view/bulk_create",
                "saved_object:infrastructure-monitoring-log-view/update",
                "saved_object:infrastructure-monitoring-log-view/bulk_update",
                "saved_object:infrastructure-monitoring-log-view/delete",
                "saved_object:infrastructure-monitoring-log-view/bulk_delete",
                "saved_object:infrastructure-monitoring-log-view/share_to_space",
                "ui:logs/show",
                "ui:logs/configureSource",
                "ui:logs/save",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/rule/create",
                "alerting:logs.alert.document.count/logs/rule/delete",
                "alerting:logs.alert.document.count/logs/rule/update",
                "alerting:logs.alert.document.count/logs/rule/updateApiKey",
                "alerting:logs.alert.document.count/logs/rule/enable",
                "alerting:logs.alert.document.count/logs/rule/disable",
                "alerting:logs.alert.document.count/logs/rule/muteAll",
                "alerting:logs.alert.document.count/logs/rule/unmuteAll",
                "alerting:logs.alert.document.count/logs/rule/muteAlert",
                "alerting:logs.alert.document.count/logs/rule/unmuteAlert",
                "alerting:logs.alert.document.count/logs/rule/snooze",
                "alerting:logs.alert.document.count/logs/rule/bulkEdit",
                "alerting:logs.alert.document.count/logs/rule/bulkDelete",
                "alerting:logs.alert.document.count/logs/rule/bulkEnable",
                "alerting:logs.alert.document.count/logs/rule/bulkDisable",
                "alerting:logs.alert.document.count/logs/rule/unsnooze",
                "alerting:logs.alert.document.count/logs/rule/runSoon",
                "alerting:logs.alert.document.count/logs/rule/scheduleBackfill",
                "alerting:logs.alert.document.count/logs/rule/deleteBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/create",
                "alerting:.es-query/logs/rule/delete",
                "alerting:.es-query/logs/rule/update",
                "alerting:.es-query/logs/rule/updateApiKey",
                "alerting:.es-query/logs/rule/enable",
                "alerting:.es-query/logs/rule/disable",
                "alerting:.es-query/logs/rule/muteAll",
                "alerting:.es-query/logs/rule/unmuteAll",
                "alerting:.es-query/logs/rule/muteAlert",
                "alerting:.es-query/logs/rule/unmuteAlert",
                "alerting:.es-query/logs/rule/snooze",
                "alerting:.es-query/logs/rule/bulkEdit",
                "alerting:.es-query/logs/rule/bulkDelete",
                "alerting:.es-query/logs/rule/bulkEnable",
                "alerting:.es-query/logs/rule/bulkDisable",
                "alerting:.es-query/logs/rule/unsnooze",
                "alerting:.es-query/logs/rule/runSoon",
                "alerting:.es-query/logs/rule/scheduleBackfill",
                "alerting:.es-query/logs/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/create",
                "alerting:observability.rules.custom_threshold/logs/rule/delete",
                "alerting:observability.rules.custom_threshold/logs/rule/update",
                "alerting:observability.rules.custom_threshold/logs/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/logs/rule/enable",
                "alerting:observability.rules.custom_threshold/logs/rule/disable",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/snooze",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/logs/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/logs/rule/runSoon",
                "alerting:observability.rules.custom_threshold/logs/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/deleteBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:logs.alert.document.count/logs/alert/update",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/update",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/update",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "minimal_all": Array [
                "login:",
                "api:infra",
                "api:rac",
                "app:infra",
                "app:metrics",
                "app:kibana",
                "ui:catalogue/infraops",
                "ui:catalogue/metrics",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/infra",
                "ui:navLinks/metrics",
                "ui:navLinks/kibana",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:infrastructure-ui-source/create",
                "saved_object:infrastructure-ui-source/bulk_create",
                "saved_object:infrastructure-ui-source/update",
                "saved_object:infrastructure-ui-source/bulk_update",
                "saved_object:infrastructure-ui-source/delete",
                "saved_object:infrastructure-ui-source/bulk_delete",
                "saved_object:infrastructure-ui-source/share_to_space",
                "saved_object:metrics-data-source/bulk_get",
                "saved_object:metrics-data-source/get",
                "saved_object:metrics-data-source/find",
                "saved_object:metrics-data-source/open_point_in_time",
                "saved_object:metrics-data-source/close_point_in_time",
                "saved_object:metrics-data-source/create",
                "saved_object:metrics-data-source/bulk_create",
                "saved_object:metrics-data-source/update",
                "saved_object:metrics-data-source/bulk_update",
                "saved_object:metrics-data-source/delete",
                "saved_object:metrics-data-source/bulk_delete",
                "saved_object:metrics-data-source/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:infrastructure/show",
                "ui:infrastructure/configureSource",
                "ui:infrastructure/save",
                "alerting:metrics.alert.threshold/infrastructure/rule/get",
                "alerting:metrics.alert.threshold/infrastructure/rule/getRuleState",
                "alerting:metrics.alert.threshold/infrastructure/rule/getAlertSummary",
                "alerting:metrics.alert.threshold/infrastructure/rule/getExecutionLog",
                "alerting:metrics.alert.threshold/infrastructure/rule/getActionErrorLog",
                "alerting:metrics.alert.threshold/infrastructure/rule/find",
                "alerting:metrics.alert.threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.threshold/infrastructure/rule/getBackfill",
                "alerting:metrics.alert.threshold/infrastructure/rule/findBackfill",
                "alerting:metrics.alert.threshold/infrastructure/rule/create",
                "alerting:metrics.alert.threshold/infrastructure/rule/delete",
                "alerting:metrics.alert.threshold/infrastructure/rule/update",
                "alerting:metrics.alert.threshold/infrastructure/rule/updateApiKey",
                "alerting:metrics.alert.threshold/infrastructure/rule/enable",
                "alerting:metrics.alert.threshold/infrastructure/rule/disable",
                "alerting:metrics.alert.threshold/infrastructure/rule/muteAll",
                "alerting:metrics.alert.threshold/infrastructure/rule/unmuteAll",
                "alerting:metrics.alert.threshold/infrastructure/rule/muteAlert",
                "alerting:metrics.alert.threshold/infrastructure/rule/unmuteAlert",
                "alerting:metrics.alert.threshold/infrastructure/rule/snooze",
                "alerting:metrics.alert.threshold/infrastructure/rule/bulkEdit",
                "alerting:metrics.alert.threshold/infrastructure/rule/bulkDelete",
                "alerting:metrics.alert.threshold/infrastructure/rule/bulkEnable",
                "alerting:metrics.alert.threshold/infrastructure/rule/bulkDisable",
                "alerting:metrics.alert.threshold/infrastructure/rule/unsnooze",
                "alerting:metrics.alert.threshold/infrastructure/rule/runSoon",
                "alerting:metrics.alert.threshold/infrastructure/rule/scheduleBackfill",
                "alerting:metrics.alert.threshold/infrastructure/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/get",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/find",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/create",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/delete",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/update",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/enable",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/disable",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/deleteBackfill",
                "alerting:.es-query/infrastructure/rule/get",
                "alerting:.es-query/infrastructure/rule/getRuleState",
                "alerting:.es-query/infrastructure/rule/getAlertSummary",
                "alerting:.es-query/infrastructure/rule/getExecutionLog",
                "alerting:.es-query/infrastructure/rule/getActionErrorLog",
                "alerting:.es-query/infrastructure/rule/find",
                "alerting:.es-query/infrastructure/rule/getRuleExecutionKPI",
                "alerting:.es-query/infrastructure/rule/getBackfill",
                "alerting:.es-query/infrastructure/rule/findBackfill",
                "alerting:.es-query/infrastructure/rule/create",
                "alerting:.es-query/infrastructure/rule/delete",
                "alerting:.es-query/infrastructure/rule/update",
                "alerting:.es-query/infrastructure/rule/updateApiKey",
                "alerting:.es-query/infrastructure/rule/enable",
                "alerting:.es-query/infrastructure/rule/disable",
                "alerting:.es-query/infrastructure/rule/muteAll",
                "alerting:.es-query/infrastructure/rule/unmuteAll",
                "alerting:.es-query/infrastructure/rule/muteAlert",
                "alerting:.es-query/infrastructure/rule/unmuteAlert",
                "alerting:.es-query/infrastructure/rule/snooze",
                "alerting:.es-query/infrastructure/rule/bulkEdit",
                "alerting:.es-query/infrastructure/rule/bulkDelete",
                "alerting:.es-query/infrastructure/rule/bulkEnable",
                "alerting:.es-query/infrastructure/rule/bulkDisable",
                "alerting:.es-query/infrastructure/rule/unsnooze",
                "alerting:.es-query/infrastructure/rule/runSoon",
                "alerting:.es-query/infrastructure/rule/scheduleBackfill",
                "alerting:.es-query/infrastructure/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/get",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/find",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/create",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/delete",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/update",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/enable",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/disable",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/muteAll",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/snooze",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/runSoon",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/deleteBackfill",
                "alerting:metrics.alert.threshold/infrastructure/alert/get",
                "alerting:metrics.alert.threshold/infrastructure/alert/find",
                "alerting:metrics.alert.threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.threshold/infrastructure/alert/getAlertSummary",
                "alerting:metrics.alert.threshold/infrastructure/alert/update",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/get",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/find",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/update",
                "alerting:.es-query/infrastructure/alert/get",
                "alerting:.es-query/infrastructure/alert/find",
                "alerting:.es-query/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/infrastructure/alert/getAlertSummary",
                "alerting:.es-query/infrastructure/alert/update",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/get",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/find",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/update",
                "app:logs",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:navLinks/logs",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/create",
                "saved_object:infrastructure-monitoring-log-view/bulk_create",
                "saved_object:infrastructure-monitoring-log-view/update",
                "saved_object:infrastructure-monitoring-log-view/bulk_update",
                "saved_object:infrastructure-monitoring-log-view/delete",
                "saved_object:infrastructure-monitoring-log-view/bulk_delete",
                "saved_object:infrastructure-monitoring-log-view/share_to_space",
                "ui:logs/show",
                "ui:logs/configureSource",
                "ui:logs/save",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/rule/create",
                "alerting:logs.alert.document.count/logs/rule/delete",
                "alerting:logs.alert.document.count/logs/rule/update",
                "alerting:logs.alert.document.count/logs/rule/updateApiKey",
                "alerting:logs.alert.document.count/logs/rule/enable",
                "alerting:logs.alert.document.count/logs/rule/disable",
                "alerting:logs.alert.document.count/logs/rule/muteAll",
                "alerting:logs.alert.document.count/logs/rule/unmuteAll",
                "alerting:logs.alert.document.count/logs/rule/muteAlert",
                "alerting:logs.alert.document.count/logs/rule/unmuteAlert",
                "alerting:logs.alert.document.count/logs/rule/snooze",
                "alerting:logs.alert.document.count/logs/rule/bulkEdit",
                "alerting:logs.alert.document.count/logs/rule/bulkDelete",
                "alerting:logs.alert.document.count/logs/rule/bulkEnable",
                "alerting:logs.alert.document.count/logs/rule/bulkDisable",
                "alerting:logs.alert.document.count/logs/rule/unsnooze",
                "alerting:logs.alert.document.count/logs/rule/runSoon",
                "alerting:logs.alert.document.count/logs/rule/scheduleBackfill",
                "alerting:logs.alert.document.count/logs/rule/deleteBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/create",
                "alerting:.es-query/logs/rule/delete",
                "alerting:.es-query/logs/rule/update",
                "alerting:.es-query/logs/rule/updateApiKey",
                "alerting:.es-query/logs/rule/enable",
                "alerting:.es-query/logs/rule/disable",
                "alerting:.es-query/logs/rule/muteAll",
                "alerting:.es-query/logs/rule/unmuteAll",
                "alerting:.es-query/logs/rule/muteAlert",
                "alerting:.es-query/logs/rule/unmuteAlert",
                "alerting:.es-query/logs/rule/snooze",
                "alerting:.es-query/logs/rule/bulkEdit",
                "alerting:.es-query/logs/rule/bulkDelete",
                "alerting:.es-query/logs/rule/bulkEnable",
                "alerting:.es-query/logs/rule/bulkDisable",
                "alerting:.es-query/logs/rule/unsnooze",
                "alerting:.es-query/logs/rule/runSoon",
                "alerting:.es-query/logs/rule/scheduleBackfill",
                "alerting:.es-query/logs/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/create",
                "alerting:observability.rules.custom_threshold/logs/rule/delete",
                "alerting:observability.rules.custom_threshold/logs/rule/update",
                "alerting:observability.rules.custom_threshold/logs/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/logs/rule/enable",
                "alerting:observability.rules.custom_threshold/logs/rule/disable",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/logs/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/logs/rule/snooze",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/logs/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/logs/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/logs/rule/runSoon",
                "alerting:observability.rules.custom_threshold/logs/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/deleteBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:logs.alert.document.count/logs/alert/update",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/update",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/update",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "minimal_read": Array [
                "login:",
                "api:infra",
                "api:rac",
                "app:infra",
                "app:metrics",
                "app:kibana",
                "ui:catalogue/infraops",
                "ui:catalogue/metrics",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/infra",
                "ui:navLinks/metrics",
                "ui:navLinks/kibana",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:metrics-data-source/bulk_get",
                "saved_object:metrics-data-source/get",
                "saved_object:metrics-data-source/find",
                "saved_object:metrics-data-source/open_point_in_time",
                "saved_object:metrics-data-source/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:infrastructure/show",
                "alerting:metrics.alert.threshold/infrastructure/rule/get",
                "alerting:metrics.alert.threshold/infrastructure/rule/getRuleState",
                "alerting:metrics.alert.threshold/infrastructure/rule/getAlertSummary",
                "alerting:metrics.alert.threshold/infrastructure/rule/getExecutionLog",
                "alerting:metrics.alert.threshold/infrastructure/rule/getActionErrorLog",
                "alerting:metrics.alert.threshold/infrastructure/rule/find",
                "alerting:metrics.alert.threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.threshold/infrastructure/rule/getBackfill",
                "alerting:metrics.alert.threshold/infrastructure/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/get",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/find",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/findBackfill",
                "alerting:.es-query/infrastructure/rule/get",
                "alerting:.es-query/infrastructure/rule/getRuleState",
                "alerting:.es-query/infrastructure/rule/getAlertSummary",
                "alerting:.es-query/infrastructure/rule/getExecutionLog",
                "alerting:.es-query/infrastructure/rule/getActionErrorLog",
                "alerting:.es-query/infrastructure/rule/find",
                "alerting:.es-query/infrastructure/rule/getRuleExecutionKPI",
                "alerting:.es-query/infrastructure/rule/getBackfill",
                "alerting:.es-query/infrastructure/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/get",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/find",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/findBackfill",
                "alerting:metrics.alert.threshold/infrastructure/alert/get",
                "alerting:metrics.alert.threshold/infrastructure/alert/find",
                "alerting:metrics.alert.threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.threshold/infrastructure/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/get",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/find",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/getAlertSummary",
                "alerting:.es-query/infrastructure/alert/get",
                "alerting:.es-query/infrastructure/alert/find",
                "alerting:.es-query/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/infrastructure/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/get",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/find",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/getAlertSummary",
                "app:logs",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:navLinks/logs",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "ui:logs/show",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
              "read": Array [
                "login:",
                "api:infra",
                "api:rac",
                "app:infra",
                "app:metrics",
                "app:kibana",
                "ui:catalogue/infraops",
                "ui:catalogue/metrics",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/infra",
                "ui:navLinks/metrics",
                "ui:navLinks/kibana",
                "saved_object:infrastructure-ui-source/bulk_get",
                "saved_object:infrastructure-ui-source/get",
                "saved_object:infrastructure-ui-source/find",
                "saved_object:infrastructure-ui-source/open_point_in_time",
                "saved_object:infrastructure-ui-source/close_point_in_time",
                "saved_object:index-pattern/bulk_get",
                "saved_object:index-pattern/get",
                "saved_object:index-pattern/find",
                "saved_object:index-pattern/open_point_in_time",
                "saved_object:index-pattern/close_point_in_time",
                "saved_object:metrics-data-source/bulk_get",
                "saved_object:metrics-data-source/get",
                "saved_object:metrics-data-source/find",
                "saved_object:metrics-data-source/open_point_in_time",
                "saved_object:metrics-data-source/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:infrastructure/show",
                "alerting:metrics.alert.threshold/infrastructure/rule/get",
                "alerting:metrics.alert.threshold/infrastructure/rule/getRuleState",
                "alerting:metrics.alert.threshold/infrastructure/rule/getAlertSummary",
                "alerting:metrics.alert.threshold/infrastructure/rule/getExecutionLog",
                "alerting:metrics.alert.threshold/infrastructure/rule/getActionErrorLog",
                "alerting:metrics.alert.threshold/infrastructure/rule/find",
                "alerting:metrics.alert.threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.threshold/infrastructure/rule/getBackfill",
                "alerting:metrics.alert.threshold/infrastructure/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/get",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/find",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/infrastructure/rule/findBackfill",
                "alerting:.es-query/infrastructure/rule/get",
                "alerting:.es-query/infrastructure/rule/getRuleState",
                "alerting:.es-query/infrastructure/rule/getAlertSummary",
                "alerting:.es-query/infrastructure/rule/getExecutionLog",
                "alerting:.es-query/infrastructure/rule/getActionErrorLog",
                "alerting:.es-query/infrastructure/rule/find",
                "alerting:.es-query/infrastructure/rule/getRuleExecutionKPI",
                "alerting:.es-query/infrastructure/rule/getBackfill",
                "alerting:.es-query/infrastructure/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/get",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/find",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/infrastructure/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/rule/findBackfill",
                "alerting:metrics.alert.threshold/infrastructure/alert/get",
                "alerting:metrics.alert.threshold/infrastructure/alert/find",
                "alerting:metrics.alert.threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.threshold/infrastructure/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/get",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/find",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/infrastructure/alert/getAlertSummary",
                "alerting:.es-query/infrastructure/alert/get",
                "alerting:.es-query/infrastructure/alert/find",
                "alerting:.es-query/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/infrastructure/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/get",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/find",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/infrastructure/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/infrastructure/alert/getAlertSummary",
                "app:logs",
                "ui:catalogue/infralogging",
                "ui:catalogue/logs",
                "ui:navLinks/logs",
                "saved_object:infrastructure-monitoring-log-view/bulk_get",
                "saved_object:infrastructure-monitoring-log-view/get",
                "saved_object:infrastructure-monitoring-log-view/find",
                "saved_object:infrastructure-monitoring-log-view/open_point_in_time",
                "saved_object:infrastructure-monitoring-log-view/close_point_in_time",
                "ui:logs/show",
                "alerting:logs.alert.document.count/logs/rule/get",
                "alerting:logs.alert.document.count/logs/rule/getRuleState",
                "alerting:logs.alert.document.count/logs/rule/getAlertSummary",
                "alerting:logs.alert.document.count/logs/rule/getExecutionLog",
                "alerting:logs.alert.document.count/logs/rule/getActionErrorLog",
                "alerting:logs.alert.document.count/logs/rule/find",
                "alerting:logs.alert.document.count/logs/rule/getRuleExecutionKPI",
                "alerting:logs.alert.document.count/logs/rule/getBackfill",
                "alerting:logs.alert.document.count/logs/rule/findBackfill",
                "alerting:.es-query/logs/rule/get",
                "alerting:.es-query/logs/rule/getRuleState",
                "alerting:.es-query/logs/rule/getAlertSummary",
                "alerting:.es-query/logs/rule/getExecutionLog",
                "alerting:.es-query/logs/rule/getActionErrorLog",
                "alerting:.es-query/logs/rule/find",
                "alerting:.es-query/logs/rule/getRuleExecutionKPI",
                "alerting:.es-query/logs/rule/getBackfill",
                "alerting:.es-query/logs/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/get",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/logs/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/logs/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/logs/rule/find",
                "alerting:observability.rules.custom_threshold/logs/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/logs/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/logs/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/logs/rule/findBackfill",
                "alerting:logs.alert.document.count/logs/alert/get",
                "alerting:logs.alert.document.count/logs/alert/find",
                "alerting:logs.alert.document.count/logs/alert/getAuthorizedAlertsIndices",
                "alerting:logs.alert.document.count/logs/alert/getAlertSummary",
                "alerting:.es-query/logs/alert/get",
                "alerting:.es-query/logs/alert/find",
                "alerting:.es-query/logs/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/logs/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/logs/alert/get",
                "alerting:observability.rules.custom_threshold/logs/alert/find",
                "alerting:observability.rules.custom_threshold/logs/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/logs/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/logs/alert/getAlertSummary",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
            },
            "reporting": Object {
              "all": Array [
                "login:",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "api:downloadCsv",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/downloadCsv",
                "api:generateReport",
                "ui:discover/generateCsv",
              ],
              "minimal_all": Array [
                "login:",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "api:downloadCsv",
                "ui:management/insightsAndAlerting/reporting",
                "ui:dashboard/downloadCsv",
                "api:generateReport",
                "ui:discover/generateCsv",
              ],
              "minimal_read": Array [
                "login:",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
              ],
              "read": Array [
                "login:",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
              ],
            },
            "slo": Object {
              "all": Array [
                "login:",
                "api:slo_write",
                "api:slo_read",
                "api:rac",
                "app:slo",
                "app:kibana",
                "ui:catalogue/slo",
                "ui:catalogue/observability",
                "ui:navLinks/slo",
                "ui:navLinks/kibana",
                "saved_object:slo/bulk_get",
                "saved_object:slo/get",
                "saved_object:slo/find",
                "saved_object:slo/open_point_in_time",
                "saved_object:slo/close_point_in_time",
                "saved_object:slo/create",
                "saved_object:slo/bulk_create",
                "saved_object:slo/update",
                "saved_object:slo/bulk_update",
                "saved_object:slo/delete",
                "saved_object:slo/bulk_delete",
                "saved_object:slo/share_to_space",
                "saved_object:slo-settings/bulk_get",
                "saved_object:slo-settings/get",
                "saved_object:slo-settings/find",
                "saved_object:slo-settings/open_point_in_time",
                "saved_object:slo-settings/close_point_in_time",
                "saved_object:slo-settings/create",
                "saved_object:slo-settings/bulk_create",
                "saved_object:slo-settings/update",
                "saved_object:slo-settings/bulk_update",
                "saved_object:slo-settings/delete",
                "saved_object:slo-settings/bulk_delete",
                "saved_object:slo-settings/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:slo/read",
                "ui:slo/write",
                "alerting:slo.rules.burnRate/slo/rule/get",
                "alerting:slo.rules.burnRate/slo/rule/getRuleState",
                "alerting:slo.rules.burnRate/slo/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/slo/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/slo/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/slo/rule/find",
                "alerting:slo.rules.burnRate/slo/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/slo/rule/getBackfill",
                "alerting:slo.rules.burnRate/slo/rule/findBackfill",
                "alerting:slo.rules.burnRate/slo/rule/create",
                "alerting:slo.rules.burnRate/slo/rule/delete",
                "alerting:slo.rules.burnRate/slo/rule/update",
                "alerting:slo.rules.burnRate/slo/rule/updateApiKey",
                "alerting:slo.rules.burnRate/slo/rule/enable",
                "alerting:slo.rules.burnRate/slo/rule/disable",
                "alerting:slo.rules.burnRate/slo/rule/muteAll",
                "alerting:slo.rules.burnRate/slo/rule/unmuteAll",
                "alerting:slo.rules.burnRate/slo/rule/muteAlert",
                "alerting:slo.rules.burnRate/slo/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/slo/rule/snooze",
                "alerting:slo.rules.burnRate/slo/rule/bulkEdit",
                "alerting:slo.rules.burnRate/slo/rule/bulkDelete",
                "alerting:slo.rules.burnRate/slo/rule/bulkEnable",
                "alerting:slo.rules.burnRate/slo/rule/bulkDisable",
                "alerting:slo.rules.burnRate/slo/rule/unsnooze",
                "alerting:slo.rules.burnRate/slo/rule/runSoon",
                "alerting:slo.rules.burnRate/slo/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/slo/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/slo/alert/get",
                "alerting:slo.rules.burnRate/slo/alert/find",
                "alerting:slo.rules.burnRate/slo/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/slo/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/slo/alert/update",
                "app:observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "minimal_all": Array [
                "login:",
                "api:slo_write",
                "api:slo_read",
                "api:rac",
                "app:slo",
                "app:kibana",
                "ui:catalogue/slo",
                "ui:catalogue/observability",
                "ui:navLinks/slo",
                "ui:navLinks/kibana",
                "saved_object:slo/bulk_get",
                "saved_object:slo/get",
                "saved_object:slo/find",
                "saved_object:slo/open_point_in_time",
                "saved_object:slo/close_point_in_time",
                "saved_object:slo/create",
                "saved_object:slo/bulk_create",
                "saved_object:slo/update",
                "saved_object:slo/bulk_update",
                "saved_object:slo/delete",
                "saved_object:slo/bulk_delete",
                "saved_object:slo/share_to_space",
                "saved_object:slo-settings/bulk_get",
                "saved_object:slo-settings/get",
                "saved_object:slo-settings/find",
                "saved_object:slo-settings/open_point_in_time",
                "saved_object:slo-settings/close_point_in_time",
                "saved_object:slo-settings/create",
                "saved_object:slo-settings/bulk_create",
                "saved_object:slo-settings/update",
                "saved_object:slo-settings/bulk_update",
                "saved_object:slo-settings/delete",
                "saved_object:slo-settings/bulk_delete",
                "saved_object:slo-settings/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:slo/read",
                "ui:slo/write",
                "alerting:slo.rules.burnRate/slo/rule/get",
                "alerting:slo.rules.burnRate/slo/rule/getRuleState",
                "alerting:slo.rules.burnRate/slo/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/slo/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/slo/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/slo/rule/find",
                "alerting:slo.rules.burnRate/slo/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/slo/rule/getBackfill",
                "alerting:slo.rules.burnRate/slo/rule/findBackfill",
                "alerting:slo.rules.burnRate/slo/rule/create",
                "alerting:slo.rules.burnRate/slo/rule/delete",
                "alerting:slo.rules.burnRate/slo/rule/update",
                "alerting:slo.rules.burnRate/slo/rule/updateApiKey",
                "alerting:slo.rules.burnRate/slo/rule/enable",
                "alerting:slo.rules.burnRate/slo/rule/disable",
                "alerting:slo.rules.burnRate/slo/rule/muteAll",
                "alerting:slo.rules.burnRate/slo/rule/unmuteAll",
                "alerting:slo.rules.burnRate/slo/rule/muteAlert",
                "alerting:slo.rules.burnRate/slo/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/slo/rule/snooze",
                "alerting:slo.rules.burnRate/slo/rule/bulkEdit",
                "alerting:slo.rules.burnRate/slo/rule/bulkDelete",
                "alerting:slo.rules.burnRate/slo/rule/bulkEnable",
                "alerting:slo.rules.burnRate/slo/rule/bulkDisable",
                "alerting:slo.rules.burnRate/slo/rule/unsnooze",
                "alerting:slo.rules.burnRate/slo/rule/runSoon",
                "alerting:slo.rules.burnRate/slo/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/slo/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/slo/alert/get",
                "alerting:slo.rules.burnRate/slo/alert/find",
                "alerting:slo.rules.burnRate/slo/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/slo/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/slo/alert/update",
                "app:observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "minimal_read": Array [
                "login:",
                "api:slo_read",
                "api:rac",
                "app:slo",
                "app:kibana",
                "ui:catalogue/slo",
                "ui:catalogue/observability",
                "ui:navLinks/slo",
                "ui:navLinks/kibana",
                "saved_object:slo/bulk_get",
                "saved_object:slo/get",
                "saved_object:slo/find",
                "saved_object:slo/open_point_in_time",
                "saved_object:slo/close_point_in_time",
                "saved_object:slo-settings/bulk_get",
                "saved_object:slo-settings/get",
                "saved_object:slo-settings/find",
                "saved_object:slo-settings/open_point_in_time",
                "saved_object:slo-settings/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:slo/read",
                "alerting:slo.rules.burnRate/slo/rule/get",
                "alerting:slo.rules.burnRate/slo/rule/getRuleState",
                "alerting:slo.rules.burnRate/slo/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/slo/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/slo/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/slo/rule/find",
                "alerting:slo.rules.burnRate/slo/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/slo/rule/getBackfill",
                "alerting:slo.rules.burnRate/slo/rule/findBackfill",
                "alerting:slo.rules.burnRate/slo/alert/get",
                "alerting:slo.rules.burnRate/slo/alert/find",
                "alerting:slo.rules.burnRate/slo/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/slo/alert/getAlertSummary",
                "app:observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
              "read": Array [
                "login:",
                "api:slo_read",
                "api:rac",
                "app:slo",
                "app:kibana",
                "ui:catalogue/slo",
                "ui:catalogue/observability",
                "ui:navLinks/slo",
                "ui:navLinks/kibana",
                "saved_object:slo/bulk_get",
                "saved_object:slo/get",
                "saved_object:slo/find",
                "saved_object:slo/open_point_in_time",
                "saved_object:slo/close_point_in_time",
                "saved_object:slo-settings/bulk_get",
                "saved_object:slo-settings/get",
                "saved_object:slo-settings/find",
                "saved_object:slo-settings/open_point_in_time",
                "saved_object:slo-settings/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:slo/read",
                "alerting:slo.rules.burnRate/slo/rule/get",
                "alerting:slo.rules.burnRate/slo/rule/getRuleState",
                "alerting:slo.rules.burnRate/slo/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/slo/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/slo/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/slo/rule/find",
                "alerting:slo.rules.burnRate/slo/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/slo/rule/getBackfill",
                "alerting:slo.rules.burnRate/slo/rule/findBackfill",
                "alerting:slo.rules.burnRate/slo/alert/get",
                "alerting:slo.rules.burnRate/slo/alert/find",
                "alerting:slo.rules.burnRate/slo/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/slo/alert/getAlertSummary",
                "app:observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
            },
            "uptime": Object {
              "all": Array [
                "login:",
                "api:uptime-read",
                "api:uptime-write",
                "api:lists-all",
                "api:rac",
                "app:uptime",
                "app:kibana",
                "app:synthetics",
                "ui:catalogue/uptime",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/uptime",
                "ui:navLinks/kibana",
                "ui:navLinks/synthetics",
                "saved_object:synthetics-dynamic-settings/bulk_get",
                "saved_object:synthetics-dynamic-settings/get",
                "saved_object:synthetics-dynamic-settings/find",
                "saved_object:synthetics-dynamic-settings/open_point_in_time",
                "saved_object:synthetics-dynamic-settings/close_point_in_time",
                "saved_object:synthetics-dynamic-settings/create",
                "saved_object:synthetics-dynamic-settings/bulk_create",
                "saved_object:synthetics-dynamic-settings/update",
                "saved_object:synthetics-dynamic-settings/bulk_update",
                "saved_object:synthetics-dynamic-settings/delete",
                "saved_object:synthetics-dynamic-settings/bulk_delete",
                "saved_object:synthetics-dynamic-settings/share_to_space",
                "saved_object:synthetics-monitor/bulk_get",
                "saved_object:synthetics-monitor/get",
                "saved_object:synthetics-monitor/find",
                "saved_object:synthetics-monitor/open_point_in_time",
                "saved_object:synthetics-monitor/close_point_in_time",
                "saved_object:synthetics-monitor/create",
                "saved_object:synthetics-monitor/bulk_create",
                "saved_object:synthetics-monitor/update",
                "saved_object:synthetics-monitor/bulk_update",
                "saved_object:synthetics-monitor/delete",
                "saved_object:synthetics-monitor/bulk_delete",
                "saved_object:synthetics-monitor/share_to_space",
                "saved_object:uptime-synthetics-api-key/bulk_get",
                "saved_object:uptime-synthetics-api-key/get",
                "saved_object:uptime-synthetics-api-key/find",
                "saved_object:uptime-synthetics-api-key/open_point_in_time",
                "saved_object:uptime-synthetics-api-key/close_point_in_time",
                "saved_object:uptime-synthetics-api-key/create",
                "saved_object:uptime-synthetics-api-key/bulk_create",
                "saved_object:uptime-synthetics-api-key/update",
                "saved_object:uptime-synthetics-api-key/bulk_update",
                "saved_object:uptime-synthetics-api-key/delete",
                "saved_object:uptime-synthetics-api-key/bulk_delete",
                "saved_object:uptime-synthetics-api-key/share_to_space",
                "saved_object:synthetics-privates-locations/bulk_get",
                "saved_object:synthetics-privates-locations/get",
                "saved_object:synthetics-privates-locations/find",
                "saved_object:synthetics-privates-locations/open_point_in_time",
                "saved_object:synthetics-privates-locations/close_point_in_time",
                "saved_object:synthetics-privates-locations/create",
                "saved_object:synthetics-privates-locations/bulk_create",
                "saved_object:synthetics-privates-locations/update",
                "saved_object:synthetics-privates-locations/bulk_update",
                "saved_object:synthetics-privates-locations/delete",
                "saved_object:synthetics-privates-locations/bulk_delete",
                "saved_object:synthetics-privates-locations/share_to_space",
                "saved_object:synthetics-param/bulk_get",
                "saved_object:synthetics-param/get",
                "saved_object:synthetics-param/find",
                "saved_object:synthetics-param/open_point_in_time",
                "saved_object:synthetics-param/close_point_in_time",
                "saved_object:synthetics-param/create",
                "saved_object:synthetics-param/bulk_create",
                "saved_object:synthetics-param/update",
                "saved_object:synthetics-param/bulk_update",
                "saved_object:synthetics-param/delete",
                "saved_object:synthetics-param/bulk_delete",
                "saved_object:synthetics-param/share_to_space",
                "saved_object:uptime-dynamic-settings/bulk_get",
                "saved_object:uptime-dynamic-settings/get",
                "saved_object:uptime-dynamic-settings/find",
                "saved_object:uptime-dynamic-settings/open_point_in_time",
                "saved_object:uptime-dynamic-settings/close_point_in_time",
                "saved_object:uptime-dynamic-settings/create",
                "saved_object:uptime-dynamic-settings/bulk_create",
                "saved_object:uptime-dynamic-settings/update",
                "saved_object:uptime-dynamic-settings/bulk_update",
                "saved_object:uptime-dynamic-settings/delete",
                "saved_object:uptime-dynamic-settings/bulk_delete",
                "saved_object:uptime-dynamic-settings/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:uptime/save",
                "ui:uptime/configureSettings",
                "ui:uptime/show",
                "ui:uptime/alerting:save",
                "ui:uptime/elasticManagedLocationsEnabled",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/get",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/find",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/create",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/delete",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/update",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/updateApiKey",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/enable",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/disable",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/muteAll",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/unmuteAll",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/muteAlert",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/unmuteAlert",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/snooze",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/bulkEdit",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/bulkDelete",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/bulkEnable",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/bulkDisable",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/unsnooze",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/runSoon",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/scheduleBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/deleteBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/get",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/find",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/create",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/delete",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/update",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/updateApiKey",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/enable",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/disable",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/muteAll",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/unmuteAll",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/muteAlert",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/unmuteAlert",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/snooze",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/bulkEdit",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/bulkDelete",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/bulkEnable",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/bulkDisable",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/unsnooze",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/runSoon",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/scheduleBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/deleteBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/get",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/find",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/create",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/delete",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/update",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/updateApiKey",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/enable",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/disable",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/muteAll",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/unmuteAll",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/muteAlert",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/unmuteAlert",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/snooze",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/bulkEdit",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/bulkDelete",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/bulkEnable",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/bulkDisable",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/unsnooze",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/runSoon",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/scheduleBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/deleteBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/get",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/find",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/create",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/delete",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/update",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/updateApiKey",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/enable",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/disable",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/muteAll",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/unmuteAll",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/muteAlert",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/unmuteAlert",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/snooze",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/bulkEdit",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/bulkDelete",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/bulkEnable",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/bulkDisable",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/unsnooze",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/runSoon",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/scheduleBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/deleteBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/get",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getRuleState",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getAlertSummary",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getExecutionLog",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getActionErrorLog",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/find",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/findBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/create",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/delete",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/update",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/updateApiKey",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/enable",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/disable",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/muteAll",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/unmuteAll",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/muteAlert",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/unmuteAlert",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/snooze",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/bulkEdit",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/bulkDelete",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/bulkEnable",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/bulkDisable",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/unsnooze",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/runSoon",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/scheduleBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/deleteBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/get",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getRuleState",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getAlertSummary",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getExecutionLog",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getActionErrorLog",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/find",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/findBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/create",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/delete",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/update",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/updateApiKey",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/enable",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/disable",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/muteAll",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/unmuteAll",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/muteAlert",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/unmuteAlert",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/snooze",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/bulkEdit",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/bulkDelete",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/bulkEnable",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/bulkDisable",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/unsnooze",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/runSoon",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/scheduleBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/deleteBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/get",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/find",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/update",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/get",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/find",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/update",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/get",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/find",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/update",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/get",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/find",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/update",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/get",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/find",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/getAlertSummary",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/update",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/get",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/find",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/getAlertSummary",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/update",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "elastic_managed_locations_enabled": Array [
                "login:",
                "ui:uptime/elasticManagedLocationsEnabled",
              ],
              "minimal_all": Array [
                "login:",
                "api:uptime-read",
                "api:uptime-write",
                "api:lists-all",
                "api:rac",
                "app:uptime",
                "app:kibana",
                "app:synthetics",
                "ui:catalogue/uptime",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/uptime",
                "ui:navLinks/kibana",
                "ui:navLinks/synthetics",
                "saved_object:synthetics-dynamic-settings/bulk_get",
                "saved_object:synthetics-dynamic-settings/get",
                "saved_object:synthetics-dynamic-settings/find",
                "saved_object:synthetics-dynamic-settings/open_point_in_time",
                "saved_object:synthetics-dynamic-settings/close_point_in_time",
                "saved_object:synthetics-dynamic-settings/create",
                "saved_object:synthetics-dynamic-settings/bulk_create",
                "saved_object:synthetics-dynamic-settings/update",
                "saved_object:synthetics-dynamic-settings/bulk_update",
                "saved_object:synthetics-dynamic-settings/delete",
                "saved_object:synthetics-dynamic-settings/bulk_delete",
                "saved_object:synthetics-dynamic-settings/share_to_space",
                "saved_object:synthetics-monitor/bulk_get",
                "saved_object:synthetics-monitor/get",
                "saved_object:synthetics-monitor/find",
                "saved_object:synthetics-monitor/open_point_in_time",
                "saved_object:synthetics-monitor/close_point_in_time",
                "saved_object:synthetics-monitor/create",
                "saved_object:synthetics-monitor/bulk_create",
                "saved_object:synthetics-monitor/update",
                "saved_object:synthetics-monitor/bulk_update",
                "saved_object:synthetics-monitor/delete",
                "saved_object:synthetics-monitor/bulk_delete",
                "saved_object:synthetics-monitor/share_to_space",
                "saved_object:uptime-synthetics-api-key/bulk_get",
                "saved_object:uptime-synthetics-api-key/get",
                "saved_object:uptime-synthetics-api-key/find",
                "saved_object:uptime-synthetics-api-key/open_point_in_time",
                "saved_object:uptime-synthetics-api-key/close_point_in_time",
                "saved_object:uptime-synthetics-api-key/create",
                "saved_object:uptime-synthetics-api-key/bulk_create",
                "saved_object:uptime-synthetics-api-key/update",
                "saved_object:uptime-synthetics-api-key/bulk_update",
                "saved_object:uptime-synthetics-api-key/delete",
                "saved_object:uptime-synthetics-api-key/bulk_delete",
                "saved_object:uptime-synthetics-api-key/share_to_space",
                "saved_object:synthetics-privates-locations/bulk_get",
                "saved_object:synthetics-privates-locations/get",
                "saved_object:synthetics-privates-locations/find",
                "saved_object:synthetics-privates-locations/open_point_in_time",
                "saved_object:synthetics-privates-locations/close_point_in_time",
                "saved_object:synthetics-privates-locations/create",
                "saved_object:synthetics-privates-locations/bulk_create",
                "saved_object:synthetics-privates-locations/update",
                "saved_object:synthetics-privates-locations/bulk_update",
                "saved_object:synthetics-privates-locations/delete",
                "saved_object:synthetics-privates-locations/bulk_delete",
                "saved_object:synthetics-privates-locations/share_to_space",
                "saved_object:synthetics-param/bulk_get",
                "saved_object:synthetics-param/get",
                "saved_object:synthetics-param/find",
                "saved_object:synthetics-param/open_point_in_time",
                "saved_object:synthetics-param/close_point_in_time",
                "saved_object:synthetics-param/create",
                "saved_object:synthetics-param/bulk_create",
                "saved_object:synthetics-param/update",
                "saved_object:synthetics-param/bulk_update",
                "saved_object:synthetics-param/delete",
                "saved_object:synthetics-param/bulk_delete",
                "saved_object:synthetics-param/share_to_space",
                "saved_object:uptime-dynamic-settings/bulk_get",
                "saved_object:uptime-dynamic-settings/get",
                "saved_object:uptime-dynamic-settings/find",
                "saved_object:uptime-dynamic-settings/open_point_in_time",
                "saved_object:uptime-dynamic-settings/close_point_in_time",
                "saved_object:uptime-dynamic-settings/create",
                "saved_object:uptime-dynamic-settings/bulk_create",
                "saved_object:uptime-dynamic-settings/update",
                "saved_object:uptime-dynamic-settings/bulk_update",
                "saved_object:uptime-dynamic-settings/delete",
                "saved_object:uptime-dynamic-settings/bulk_delete",
                "saved_object:uptime-dynamic-settings/share_to_space",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:telemetry/create",
                "saved_object:telemetry/bulk_create",
                "saved_object:telemetry/update",
                "saved_object:telemetry/bulk_update",
                "saved_object:telemetry/delete",
                "saved_object:telemetry/bulk_delete",
                "saved_object:telemetry/share_to_space",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:uptime/save",
                "ui:uptime/configureSettings",
                "ui:uptime/show",
                "ui:uptime/alerting:save",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/get",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/find",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/create",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/delete",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/update",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/updateApiKey",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/enable",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/disable",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/muteAll",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/unmuteAll",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/muteAlert",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/unmuteAlert",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/snooze",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/bulkEdit",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/bulkDelete",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/bulkEnable",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/bulkDisable",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/unsnooze",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/runSoon",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/scheduleBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/deleteBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/get",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/find",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/create",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/delete",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/update",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/updateApiKey",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/enable",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/disable",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/muteAll",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/unmuteAll",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/muteAlert",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/unmuteAlert",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/snooze",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/bulkEdit",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/bulkDelete",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/bulkEnable",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/bulkDisable",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/unsnooze",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/runSoon",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/scheduleBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/deleteBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/get",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/find",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/create",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/delete",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/update",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/updateApiKey",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/enable",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/disable",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/muteAll",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/unmuteAll",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/muteAlert",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/unmuteAlert",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/snooze",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/bulkEdit",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/bulkDelete",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/bulkEnable",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/bulkDisable",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/unsnooze",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/runSoon",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/scheduleBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/deleteBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/get",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/find",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/create",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/delete",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/update",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/updateApiKey",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/enable",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/disable",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/muteAll",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/unmuteAll",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/muteAlert",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/unmuteAlert",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/snooze",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/bulkEdit",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/bulkDelete",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/bulkEnable",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/bulkDisable",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/unsnooze",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/runSoon",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/scheduleBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/deleteBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/get",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getRuleState",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getAlertSummary",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getExecutionLog",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getActionErrorLog",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/find",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/findBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/create",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/delete",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/update",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/updateApiKey",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/enable",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/disable",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/muteAll",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/unmuteAll",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/muteAlert",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/unmuteAlert",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/snooze",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/bulkEdit",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/bulkDelete",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/bulkEnable",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/bulkDisable",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/unsnooze",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/runSoon",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/scheduleBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/deleteBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/get",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getRuleState",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getAlertSummary",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getExecutionLog",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getActionErrorLog",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/find",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/findBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/create",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/delete",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/update",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/updateApiKey",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/enable",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/disable",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/muteAll",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/unmuteAll",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/muteAlert",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/unmuteAlert",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/snooze",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/bulkEdit",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/bulkDelete",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/bulkEnable",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/bulkDisable",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/unsnooze",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/runSoon",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/scheduleBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/deleteBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/get",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/find",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/update",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/get",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/find",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/update",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/get",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/find",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/update",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/get",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/find",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/update",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/get",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/find",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/getAlertSummary",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/update",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/get",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/find",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/getAlertSummary",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/update",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "ui:observability/write",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/rule/create",
                "alerting:slo.rules.burnRate/observability/rule/delete",
                "alerting:slo.rules.burnRate/observability/rule/update",
                "alerting:slo.rules.burnRate/observability/rule/updateApiKey",
                "alerting:slo.rules.burnRate/observability/rule/enable",
                "alerting:slo.rules.burnRate/observability/rule/disable",
                "alerting:slo.rules.burnRate/observability/rule/muteAll",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAll",
                "alerting:slo.rules.burnRate/observability/rule/muteAlert",
                "alerting:slo.rules.burnRate/observability/rule/unmuteAlert",
                "alerting:slo.rules.burnRate/observability/rule/snooze",
                "alerting:slo.rules.burnRate/observability/rule/bulkEdit",
                "alerting:slo.rules.burnRate/observability/rule/bulkDelete",
                "alerting:slo.rules.burnRate/observability/rule/bulkEnable",
                "alerting:slo.rules.burnRate/observability/rule/bulkDisable",
                "alerting:slo.rules.burnRate/observability/rule/unsnooze",
                "alerting:slo.rules.burnRate/observability/rule/runSoon",
                "alerting:slo.rules.burnRate/observability/rule/scheduleBackfill",
                "alerting:slo.rules.burnRate/observability/rule/deleteBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/create",
                "alerting:observability.rules.custom_threshold/observability/rule/delete",
                "alerting:observability.rules.custom_threshold/observability/rule/update",
                "alerting:observability.rules.custom_threshold/observability/rule/updateApiKey",
                "alerting:observability.rules.custom_threshold/observability/rule/enable",
                "alerting:observability.rules.custom_threshold/observability/rule/disable",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAll",
                "alerting:observability.rules.custom_threshold/observability/rule/muteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/unmuteAlert",
                "alerting:observability.rules.custom_threshold/observability/rule/snooze",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEdit",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDelete",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkEnable",
                "alerting:observability.rules.custom_threshold/observability/rule/bulkDisable",
                "alerting:observability.rules.custom_threshold/observability/rule/unsnooze",
                "alerting:observability.rules.custom_threshold/observability/rule/runSoon",
                "alerting:observability.rules.custom_threshold/observability/rule/scheduleBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/deleteBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/create",
                "alerting:.es-query/observability/rule/delete",
                "alerting:.es-query/observability/rule/update",
                "alerting:.es-query/observability/rule/updateApiKey",
                "alerting:.es-query/observability/rule/enable",
                "alerting:.es-query/observability/rule/disable",
                "alerting:.es-query/observability/rule/muteAll",
                "alerting:.es-query/observability/rule/unmuteAll",
                "alerting:.es-query/observability/rule/muteAlert",
                "alerting:.es-query/observability/rule/unmuteAlert",
                "alerting:.es-query/observability/rule/snooze",
                "alerting:.es-query/observability/rule/bulkEdit",
                "alerting:.es-query/observability/rule/bulkDelete",
                "alerting:.es-query/observability/rule/bulkEnable",
                "alerting:.es-query/observability/rule/bulkDisable",
                "alerting:.es-query/observability/rule/unsnooze",
                "alerting:.es-query/observability/rule/runSoon",
                "alerting:.es-query/observability/rule/scheduleBackfill",
                "alerting:.es-query/observability/rule/deleteBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/create",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/delete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/updateApiKey",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/enable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/disable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAll",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/muteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unmuteAlert",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/snooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEdit",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDelete",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkEnable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/bulkDisable",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/unsnooze",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/runSoon",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/scheduleBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/deleteBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/create",
                "alerting:metrics.alert.inventory.threshold/observability/rule/delete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/update",
                "alerting:metrics.alert.inventory.threshold/observability/rule/updateApiKey",
                "alerting:metrics.alert.inventory.threshold/observability/rule/enable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/disable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAll",
                "alerting:metrics.alert.inventory.threshold/observability/rule/muteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unmuteAlert",
                "alerting:metrics.alert.inventory.threshold/observability/rule/snooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEdit",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDelete",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkEnable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/bulkDisable",
                "alerting:metrics.alert.inventory.threshold/observability/rule/unsnooze",
                "alerting:metrics.alert.inventory.threshold/observability/rule/runSoon",
                "alerting:metrics.alert.inventory.threshold/observability/rule/scheduleBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/deleteBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/create",
                "alerting:apm.error_rate/observability/rule/delete",
                "alerting:apm.error_rate/observability/rule/update",
                "alerting:apm.error_rate/observability/rule/updateApiKey",
                "alerting:apm.error_rate/observability/rule/enable",
                "alerting:apm.error_rate/observability/rule/disable",
                "alerting:apm.error_rate/observability/rule/muteAll",
                "alerting:apm.error_rate/observability/rule/unmuteAll",
                "alerting:apm.error_rate/observability/rule/muteAlert",
                "alerting:apm.error_rate/observability/rule/unmuteAlert",
                "alerting:apm.error_rate/observability/rule/snooze",
                "alerting:apm.error_rate/observability/rule/bulkEdit",
                "alerting:apm.error_rate/observability/rule/bulkDelete",
                "alerting:apm.error_rate/observability/rule/bulkEnable",
                "alerting:apm.error_rate/observability/rule/bulkDisable",
                "alerting:apm.error_rate/observability/rule/unsnooze",
                "alerting:apm.error_rate/observability/rule/runSoon",
                "alerting:apm.error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/create",
                "alerting:apm.transaction_error_rate/observability/rule/delete",
                "alerting:apm.transaction_error_rate/observability/rule/update",
                "alerting:apm.transaction_error_rate/observability/rule/updateApiKey",
                "alerting:apm.transaction_error_rate/observability/rule/enable",
                "alerting:apm.transaction_error_rate/observability/rule/disable",
                "alerting:apm.transaction_error_rate/observability/rule/muteAll",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAll",
                "alerting:apm.transaction_error_rate/observability/rule/muteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/unmuteAlert",
                "alerting:apm.transaction_error_rate/observability/rule/snooze",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEdit",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDelete",
                "alerting:apm.transaction_error_rate/observability/rule/bulkEnable",
                "alerting:apm.transaction_error_rate/observability/rule/bulkDisable",
                "alerting:apm.transaction_error_rate/observability/rule/unsnooze",
                "alerting:apm.transaction_error_rate/observability/rule/runSoon",
                "alerting:apm.transaction_error_rate/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/deleteBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/create",
                "alerting:apm.transaction_duration/observability/rule/delete",
                "alerting:apm.transaction_duration/observability/rule/update",
                "alerting:apm.transaction_duration/observability/rule/updateApiKey",
                "alerting:apm.transaction_duration/observability/rule/enable",
                "alerting:apm.transaction_duration/observability/rule/disable",
                "alerting:apm.transaction_duration/observability/rule/muteAll",
                "alerting:apm.transaction_duration/observability/rule/unmuteAll",
                "alerting:apm.transaction_duration/observability/rule/muteAlert",
                "alerting:apm.transaction_duration/observability/rule/unmuteAlert",
                "alerting:apm.transaction_duration/observability/rule/snooze",
                "alerting:apm.transaction_duration/observability/rule/bulkEdit",
                "alerting:apm.transaction_duration/observability/rule/bulkDelete",
                "alerting:apm.transaction_duration/observability/rule/bulkEnable",
                "alerting:apm.transaction_duration/observability/rule/bulkDisable",
                "alerting:apm.transaction_duration/observability/rule/unsnooze",
                "alerting:apm.transaction_duration/observability/rule/runSoon",
                "alerting:apm.transaction_duration/observability/rule/scheduleBackfill",
                "alerting:apm.transaction_duration/observability/rule/deleteBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/create",
                "alerting:apm.anomaly/observability/rule/delete",
                "alerting:apm.anomaly/observability/rule/update",
                "alerting:apm.anomaly/observability/rule/updateApiKey",
                "alerting:apm.anomaly/observability/rule/enable",
                "alerting:apm.anomaly/observability/rule/disable",
                "alerting:apm.anomaly/observability/rule/muteAll",
                "alerting:apm.anomaly/observability/rule/unmuteAll",
                "alerting:apm.anomaly/observability/rule/muteAlert",
                "alerting:apm.anomaly/observability/rule/unmuteAlert",
                "alerting:apm.anomaly/observability/rule/snooze",
                "alerting:apm.anomaly/observability/rule/bulkEdit",
                "alerting:apm.anomaly/observability/rule/bulkDelete",
                "alerting:apm.anomaly/observability/rule/bulkEnable",
                "alerting:apm.anomaly/observability/rule/bulkDisable",
                "alerting:apm.anomaly/observability/rule/unsnooze",
                "alerting:apm.anomaly/observability/rule/runSoon",
                "alerting:apm.anomaly/observability/rule/scheduleBackfill",
                "alerting:apm.anomaly/observability/rule/deleteBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/alert/update",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/update",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/update",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/update",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/update",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/update",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/update",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/update",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/update",
              ],
              "minimal_read": Array [
                "login:",
                "api:uptime-read",
                "api:lists-read",
                "api:rac",
                "app:uptime",
                "app:kibana",
                "app:synthetics",
                "ui:catalogue/uptime",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/uptime",
                "ui:navLinks/kibana",
                "ui:navLinks/synthetics",
                "saved_object:synthetics-param/bulk_get",
                "saved_object:synthetics-param/get",
                "saved_object:synthetics-param/find",
                "saved_object:synthetics-param/open_point_in_time",
                "saved_object:synthetics-param/close_point_in_time",
                "saved_object:synthetics-dynamic-settings/bulk_get",
                "saved_object:synthetics-dynamic-settings/get",
                "saved_object:synthetics-dynamic-settings/find",
                "saved_object:synthetics-dynamic-settings/open_point_in_time",
                "saved_object:synthetics-dynamic-settings/close_point_in_time",
                "saved_object:synthetics-monitor/bulk_get",
                "saved_object:synthetics-monitor/get",
                "saved_object:synthetics-monitor/find",
                "saved_object:synthetics-monitor/open_point_in_time",
                "saved_object:synthetics-monitor/close_point_in_time",
                "saved_object:uptime-synthetics-api-key/bulk_get",
                "saved_object:uptime-synthetics-api-key/get",
                "saved_object:uptime-synthetics-api-key/find",
                "saved_object:uptime-synthetics-api-key/open_point_in_time",
                "saved_object:uptime-synthetics-api-key/close_point_in_time",
                "saved_object:synthetics-privates-locations/bulk_get",
                "saved_object:synthetics-privates-locations/get",
                "saved_object:synthetics-privates-locations/find",
                "saved_object:synthetics-privates-locations/open_point_in_time",
                "saved_object:synthetics-privates-locations/close_point_in_time",
                "saved_object:uptime-dynamic-settings/bulk_get",
                "saved_object:uptime-dynamic-settings/get",
                "saved_object:uptime-dynamic-settings/find",
                "saved_object:uptime-dynamic-settings/open_point_in_time",
                "saved_object:uptime-dynamic-settings/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:uptime/show",
                "ui:uptime/alerting:save",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/get",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/find",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/get",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/find",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/get",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/find",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/get",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/find",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/findBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/get",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getRuleState",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getAlertSummary",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getExecutionLog",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getActionErrorLog",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/find",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/findBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/get",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getRuleState",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getAlertSummary",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getExecutionLog",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getActionErrorLog",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/find",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/get",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/find",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/get",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/find",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/get",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/find",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/get",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/find",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/getAlertSummary",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/get",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/find",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/getAlertSummary",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/get",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/find",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/getAlertSummary",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
              "read": Array [
                "login:",
                "api:uptime-read",
                "api:lists-read",
                "api:rac",
                "app:uptime",
                "app:kibana",
                "app:synthetics",
                "ui:catalogue/uptime",
                "ui:management/insightsAndAlerting/triggersActions",
                "ui:navLinks/uptime",
                "ui:navLinks/kibana",
                "ui:navLinks/synthetics",
                "saved_object:synthetics-param/bulk_get",
                "saved_object:synthetics-param/get",
                "saved_object:synthetics-param/find",
                "saved_object:synthetics-param/open_point_in_time",
                "saved_object:synthetics-param/close_point_in_time",
                "saved_object:synthetics-dynamic-settings/bulk_get",
                "saved_object:synthetics-dynamic-settings/get",
                "saved_object:synthetics-dynamic-settings/find",
                "saved_object:synthetics-dynamic-settings/open_point_in_time",
                "saved_object:synthetics-dynamic-settings/close_point_in_time",
                "saved_object:synthetics-monitor/bulk_get",
                "saved_object:synthetics-monitor/get",
                "saved_object:synthetics-monitor/find",
                "saved_object:synthetics-monitor/open_point_in_time",
                "saved_object:synthetics-monitor/close_point_in_time",
                "saved_object:uptime-synthetics-api-key/bulk_get",
                "saved_object:uptime-synthetics-api-key/get",
                "saved_object:uptime-synthetics-api-key/find",
                "saved_object:uptime-synthetics-api-key/open_point_in_time",
                "saved_object:uptime-synthetics-api-key/close_point_in_time",
                "saved_object:synthetics-privates-locations/bulk_get",
                "saved_object:synthetics-privates-locations/get",
                "saved_object:synthetics-privates-locations/find",
                "saved_object:synthetics-privates-locations/open_point_in_time",
                "saved_object:synthetics-privates-locations/close_point_in_time",
                "saved_object:uptime-dynamic-settings/bulk_get",
                "saved_object:uptime-dynamic-settings/get",
                "saved_object:uptime-dynamic-settings/find",
                "saved_object:uptime-dynamic-settings/open_point_in_time",
                "saved_object:uptime-dynamic-settings/close_point_in_time",
                "saved_object:config/bulk_get",
                "saved_object:config/get",
                "saved_object:config/find",
                "saved_object:config/open_point_in_time",
                "saved_object:config/close_point_in_time",
                "saved_object:config-global/bulk_get",
                "saved_object:config-global/get",
                "saved_object:config-global/find",
                "saved_object:config-global/open_point_in_time",
                "saved_object:config-global/close_point_in_time",
                "saved_object:telemetry/bulk_get",
                "saved_object:telemetry/get",
                "saved_object:telemetry/find",
                "saved_object:telemetry/open_point_in_time",
                "saved_object:telemetry/close_point_in_time",
                "saved_object:url/bulk_get",
                "saved_object:url/get",
                "saved_object:url/find",
                "saved_object:url/open_point_in_time",
                "saved_object:url/close_point_in_time",
                "ui:uptime/show",
                "ui:uptime/alerting:save",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/get",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/find",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/get",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/find",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/get",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/find",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/get",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getRuleState",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getAlertSummary",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getExecutionLog",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getActionErrorLog",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/find",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/getBackfill",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/rule/findBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/get",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getRuleState",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getAlertSummary",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getExecutionLog",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getActionErrorLog",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/find",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/getBackfill",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/rule/findBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/get",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getRuleState",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getAlertSummary",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getExecutionLog",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getActionErrorLog",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/find",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getRuleExecutionKPI",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/getBackfill",
                "alerting:xpack.synthetics.alerts.tls/uptime/rule/findBackfill",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/get",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/find",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.tls/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/get",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/find",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.tlsCertificate/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/get",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/find",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.monitorStatus/uptime/alert/getAlertSummary",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/get",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/find",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.uptime.alerts.durationAnomaly/uptime/alert/getAlertSummary",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/get",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/find",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.synthetics.alerts.monitorStatus/uptime/alert/getAlertSummary",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/get",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/find",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.synthetics.alerts.tls/uptime/alert/getAlertSummary",
                "app:observability",
                "ui:catalogue/observability",
                "ui:navLinks/observability",
                "ui:observability/read",
                "alerting:slo.rules.burnRate/observability/rule/get",
                "alerting:slo.rules.burnRate/observability/rule/getRuleState",
                "alerting:slo.rules.burnRate/observability/rule/getAlertSummary",
                "alerting:slo.rules.burnRate/observability/rule/getExecutionLog",
                "alerting:slo.rules.burnRate/observability/rule/getActionErrorLog",
                "alerting:slo.rules.burnRate/observability/rule/find",
                "alerting:slo.rules.burnRate/observability/rule/getRuleExecutionKPI",
                "alerting:slo.rules.burnRate/observability/rule/getBackfill",
                "alerting:slo.rules.burnRate/observability/rule/findBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/get",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleState",
                "alerting:observability.rules.custom_threshold/observability/rule/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/rule/getExecutionLog",
                "alerting:observability.rules.custom_threshold/observability/rule/getActionErrorLog",
                "alerting:observability.rules.custom_threshold/observability/rule/find",
                "alerting:observability.rules.custom_threshold/observability/rule/getRuleExecutionKPI",
                "alerting:observability.rules.custom_threshold/observability/rule/getBackfill",
                "alerting:observability.rules.custom_threshold/observability/rule/findBackfill",
                "alerting:.es-query/observability/rule/get",
                "alerting:.es-query/observability/rule/getRuleState",
                "alerting:.es-query/observability/rule/getAlertSummary",
                "alerting:.es-query/observability/rule/getExecutionLog",
                "alerting:.es-query/observability/rule/getActionErrorLog",
                "alerting:.es-query/observability/rule/find",
                "alerting:.es-query/observability/rule/getRuleExecutionKPI",
                "alerting:.es-query/observability/rule/getBackfill",
                "alerting:.es-query/observability/rule/findBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleState",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getExecutionLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getActionErrorLog",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getRuleExecutionKPI",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/getBackfill",
                "alerting:xpack.ml.anomaly_detection_alert/observability/rule/findBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/get",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleState",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getExecutionLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getActionErrorLog",
                "alerting:metrics.alert.inventory.threshold/observability/rule/find",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getRuleExecutionKPI",
                "alerting:metrics.alert.inventory.threshold/observability/rule/getBackfill",
                "alerting:metrics.alert.inventory.threshold/observability/rule/findBackfill",
                "alerting:apm.error_rate/observability/rule/get",
                "alerting:apm.error_rate/observability/rule/getRuleState",
                "alerting:apm.error_rate/observability/rule/getAlertSummary",
                "alerting:apm.error_rate/observability/rule/getExecutionLog",
                "alerting:apm.error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.error_rate/observability/rule/find",
                "alerting:apm.error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.error_rate/observability/rule/getBackfill",
                "alerting:apm.error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/get",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleState",
                "alerting:apm.transaction_error_rate/observability/rule/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/rule/getExecutionLog",
                "alerting:apm.transaction_error_rate/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_error_rate/observability/rule/find",
                "alerting:apm.transaction_error_rate/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_error_rate/observability/rule/getBackfill",
                "alerting:apm.transaction_error_rate/observability/rule/findBackfill",
                "alerting:apm.transaction_duration/observability/rule/get",
                "alerting:apm.transaction_duration/observability/rule/getRuleState",
                "alerting:apm.transaction_duration/observability/rule/getAlertSummary",
                "alerting:apm.transaction_duration/observability/rule/getExecutionLog",
                "alerting:apm.transaction_duration/observability/rule/getActionErrorLog",
                "alerting:apm.transaction_duration/observability/rule/find",
                "alerting:apm.transaction_duration/observability/rule/getRuleExecutionKPI",
                "alerting:apm.transaction_duration/observability/rule/getBackfill",
                "alerting:apm.transaction_duration/observability/rule/findBackfill",
                "alerting:apm.anomaly/observability/rule/get",
                "alerting:apm.anomaly/observability/rule/getRuleState",
                "alerting:apm.anomaly/observability/rule/getAlertSummary",
                "alerting:apm.anomaly/observability/rule/getExecutionLog",
                "alerting:apm.anomaly/observability/rule/getActionErrorLog",
                "alerting:apm.anomaly/observability/rule/find",
                "alerting:apm.anomaly/observability/rule/getRuleExecutionKPI",
                "alerting:apm.anomaly/observability/rule/getBackfill",
                "alerting:apm.anomaly/observability/rule/findBackfill",
                "alerting:slo.rules.burnRate/observability/alert/get",
                "alerting:slo.rules.burnRate/observability/alert/find",
                "alerting:slo.rules.burnRate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:slo.rules.burnRate/observability/alert/getAlertSummary",
                "alerting:observability.rules.custom_threshold/observability/alert/get",
                "alerting:observability.rules.custom_threshold/observability/alert/find",
                "alerting:observability.rules.custom_threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:observability.rules.custom_threshold/observability/alert/getAlertSummary",
                "alerting:.es-query/observability/alert/get",
                "alerting:.es-query/observability/alert/find",
                "alerting:.es-query/observability/alert/getAuthorizedAlertsIndices",
                "alerting:.es-query/observability/alert/getAlertSummary",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/get",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/find",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAuthorizedAlertsIndices",
                "alerting:xpack.ml.anomaly_detection_alert/observability/alert/getAlertSummary",
                "alerting:metrics.alert.inventory.threshold/observability/alert/get",
                "alerting:metrics.alert.inventory.threshold/observability/alert/find",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAuthorizedAlertsIndices",
                "alerting:metrics.alert.inventory.threshold/observability/alert/getAlertSummary",
                "alerting:apm.error_rate/observability/alert/get",
                "alerting:apm.error_rate/observability/alert/find",
                "alerting:apm.error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_error_rate/observability/alert/get",
                "alerting:apm.transaction_error_rate/observability/alert/find",
                "alerting:apm.transaction_error_rate/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_error_rate/observability/alert/getAlertSummary",
                "alerting:apm.transaction_duration/observability/alert/get",
                "alerting:apm.transaction_duration/observability/alert/find",
                "alerting:apm.transaction_duration/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.transaction_duration/observability/alert/getAlertSummary",
                "alerting:apm.anomaly/observability/alert/get",
                "alerting:apm.anomaly/observability/alert/find",
                "alerting:apm.anomaly/observability/alert/getAuthorizedAlertsIndices",
                "alerting:apm.anomaly/observability/alert/getAlertSummary",
              ],
            },
          }
        `);
      });
    });
  });
}
